@@ -10,7 +10,7 @@
 #                         University of Stuttgart.  All rights reserved.
 # Copyright (c) 2004-2005 The Regents of the University of California.
 #                         All rights reserved.
-# Copyright (c) 2007-2008 Cisco, Inc.  All rights reserved.
+# Copyright (c) 2007-2010 Cisco Systems, Inc.  All rights reserved.
 # $COPYRIGHT$
 # 
 # Additional copyrights may follow
@@ -477,13 +477,34 @@ EOF
             chmod a+x configure
         fi
 
-        # See http://lists.gnu.org/archive/html/bug-libtool/2008-05/msg00045.html
-        if check_version "2.1.9999" $ompi_libtoolize_found_version ; then
-            echo "   ++ patching for ifort (LT 2.2.0-4)"
-            cd opal/libltdl/m4
-            patch -N -p0 < ../../../config/lt224-icc.diff > /dev/null 2>&1
-            rm -f libtool.m4.orig
-            cd ../../..
+        # See
+        # http://lists.gnu.org/archive/html/bug-libtool/2008-05/msg00045.html.
+        # Note that this issue was fixed in LT 2.2.6, so don't patch if
+        # you have a version after that (because this patch changes
+        # the timestamp on opal/libltdl/m4/libtool.m4, which should be
+        # avoided if possible...).
+        echo "   ++ patching for ifort (LT 2.2.0-4)"
+        patched=0
+        if check_version "2.1.9999" $ompi_libtoolize_found_version; then
+            if ! check_version "2.2.6" $ompi_libtoolize_found_version; then
+                cd opal/libltdl/m4
+                patch -N -p0 < ../../../config/lt224-icc.diff > /dev/null 2>&1
+                rm -f libtool.m4.orig
+                rm -f libtool.m4.rej
+
+                # We must touch aclocal.m4 here, because it must be
+                # newer than libtool.m4, otherwise a whole bunch of
+                # Automake-mandated timestamps may be off (depending
+                # on the resolution of timestamps on your
+                # filesystem).
+                touch -r ../aclocal.m4 libtool.m4
+
+                cd ../../..
+                patched=1
+            fi
+        fi
+        if test "$patched" != "1"; then
+            echo "     -- your libltdl doesn't need this! yay!"
         fi
     else
 	run_and_check $ompi_libtoolize --automake --copy
@@ -513,16 +534,26 @@ EOF
             echo "   ++ patching for pathscale multi-line output (LT 2.x)"
             patch -N -p0 < config/lt21a-pathCC.diff > /dev/null 2>&1
         fi
-        # See http://lists.gnu.org/archive/html/bug-libtool/2008-05/msg00045.html
-        if check_version "2.1.9999" $ompi_libtoolize_found_version ; then
-            echo "   ++ patching for ifort (LT 2.2.0-4)"
-            cd config
-            patch -N -p0 < lt224-icc.diff > /dev/null 2>&1
-            rm -f libtool.m4.orig
-            cd ..
-        fi
         rm -f aclocal.m4.orig
+        rm -f aclocal.m4.rej
 
+        # See note above about lt224-icc.diff
+        echo "   ++ patching for ifort (LT 2.2.0-4)"
+        patched=0
+        if check_version "2.1.9999" $ompi_libtoolize_found_version; then
+            if ! check_version "2.2.6" $ompi_libtoolize_found_version; then
+                cd config
+                patch -N -p0 < lt224-icc.diff > /dev/null 2>&1
+                rm -f libtool.m4.orig
+                rm -f libtool.m4.rej
+                # We'll touch aclocal.m4 below (see comment below).
+                cd ..
+                patched=1
+            fi
+        fi
+        if test "$patched" != "1"; then
+            echo "     -- your libltdl doesn't need this! yay!"
+        fi
 
         # Libtool 1.5.2x and 2.1x automatically link in the "Cstd" STL library
         # when using the Sun compilers on Linux or Solaris, even if the
@@ -547,11 +578,45 @@ EOF
         #   http://www.open-mpi.org/community/lists/devel/2008/11/4920.php
         echo "   ++ patching for Sun Studio Fortran compilers"
         patch -N -p0 < config/lt-sun-fortran.diff > /dev/null 2>&1
+        # We must touch aclocal.m4 here, because it must be newer than
+        # libtool.m4, otherwise a whole bunch of Automake-mandated
+        # timestamps may be off (depending on the resolution of
+        # timestamps on your filesystem).
+        touch -r aclocal.m4 config/libtool.m4
     fi
 
     run_and_check $ompi_autoconf
 
     run_and_check $ompi_automake --foreign -a --copy --include-deps
+
+    # Ugh.  LT <=2.2.6b need to be patched for the PGI 10.0 fortran
+    # compiler name (pgfortran).  The following comes from the
+    # upstream LT patches:
+    # http://lists.gnu.org/archive/html/libtool-patches/2009-11/msg00012.html
+    # http://lists.gnu.org/archive/html/bug-libtool/2009-11/msg00045.html
+    # Note that that patch is part of Libtool (which is not in this
+    # OMPI source tree); we can't fix it.  So all we can do is patch
+    # the resulting configure script.  :-(
+    echo "  ++ Patching configure for Libtool PGI 10 fortran compiler name"
+    rm -f configure.patched
+    sed -e 's/gfortran g95 xlf95 f95 fort ifort ifc efc pgf95 lf95 ftn/gfortran g95 xlf95 f95 fort ifort ifc efc pgfortran pgf95 lf95 ftn/' \
+        -e 's/pgcc\* | pgf77\* | pgf90\* | pgf95\*)/pgcc* | pgf77* | pgf90* | pgf95* | pgfortran*)/' \
+        -e 's/pgf77\* | pgf90\* | pgf95\*)/pgf77* | pgf90* | pgf95* | pgfortran*)/' \
+        configure > configure.patched
+    cp configure.patched configure
+    rm -f configure.patched
+
+    # Similar issue as above -- the PGI 10 version number broke <=LT
+    # 2.2.6b's version number checking regexps.  Again, we can't fix
+    # the Libtool install; all we can do is patch the resulting
+    # configure script.  :-( The following comes from the upstream
+    # patch:
+    # http://lists.gnu.org/archive/html/libtool-patches/2009-11/msg00016.html
+    echo "  ++ Patching configure for Libtool PGI version number regexps"
+    rm -f configure.patched
+    sed -e 's/\*pgCC\\ \[1-5\]\* | \*pgcpp\\ \[1-5\]\*/*pgCC\\ [1-5]\.* | *pgcpp\\ [1-5]\.*/' configure > configure.patched
+    cp configure.patched configure
+    rm -f configure.patched
 }
 
 
