@@ -81,20 +81,18 @@
 
 #include "ompi_config.h"
 
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #include "mpi.h"
 
 #include "ompi/attribute/attribute.h"
 
 #include "ompi/errhandler/errcode.h"
 #include "ompi/communicator/communicator.h"
-#include "orte/util/proc_info.h"
 #include "ompi/mca/pml/pml.h"
-#include "orte/dss/dss.h"
-#include "orte/mca/ns/ns.h"
-#include "orte/mca/gpr/gpr.h"
-#include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/schema/schema.h"
-
+#include "orte/util/proc_info.h"
 
 /*
  * Private functions
@@ -102,7 +100,6 @@
 static int create_comm(int target_keyval, bool want_inherit);
 static int free_comm(int keyval);
 
-/* JMS for when we implement windows */
 static int create_win(int target_keyval);
 static int free_win(int keyval);
 
@@ -111,11 +108,9 @@ static int set_f(int keyval, MPI_Fint value);
 
 int ompi_attr_create_predefined(void)
 {
-    int rc, ret;
-    orte_gpr_subscription_t *subs, sub = ORTE_GPR_SUBSCRIPTION_EMPTY;
-    orte_gpr_trigger_t *trigs, trig = ORTE_GPR_TRIGGER_EMPTY;
-    orte_gpr_value_t *values[1];
-    char *jobseg;
+    int ret;
+    char *univ_size;
+    int usize;
 
     /* Create all the keyvals */
 
@@ -144,11 +139,7 @@ int ompi_attr_create_predefined(void)
         return ret;
     }
 
-    /* Set default values for everything except APPNUM.  Set UNIVERSE
-       size to comm_world size.  It might grow later, it might not
-       (tiggers are not fired in all environments.  In environments
-       where triggers aren't set, there won't be COMM_SPAWN, so APPNUM
-       probably isn't a big deal. */
+    /* Set default values for everything except MPI_UNIVERSE_SIZE */
 
     if (OMPI_SUCCESS != (ret = set_f(MPI_TAG_UB, mca_pml.pml_max_tag)) ||
         OMPI_SUCCESS != (ret = set_f(MPI_HOST, MPI_PROC_NULL)) ||
@@ -156,8 +147,6 @@ int ompi_attr_create_predefined(void)
         OMPI_SUCCESS != (ret = set_f(MPI_WTIME_IS_GLOBAL, 0)) ||
         OMPI_SUCCESS != (ret = set_f(MPI_LASTUSEDCODE,
                                      ompi_mpi_errcode_lastused)) ||
-        OMPI_SUCCESS != (ret = set_f(MPI_UNIVERSE_SIZE,
-                                    ompi_comm_size(MPI_COMM_WORLD))) ||
 #if 0
         /* JMS For when we implement IMPI */
         OMPI_SUCCESS != (ret = set(IMPI_CLIENT_SIZE,
@@ -173,76 +162,24 @@ int ompi_attr_create_predefined(void)
         return ret;
     }
 
-    /* Now that those are all created, setup the trigger to get the
-       UNIVERSE_SIZE and APPNUM values once everyone has passed
-       stg1. */
-
-    /* we have to create two subscriptions - one to retrieve the number of slots on
-     * each node so we can estimate the universe size, and the other to return our
-     * app_context index to properly set the appnum attribute.
-     *
-     * NOTE: when the 2.0 registry becomes available, this should be consolidated to
-     * a single subscription
-     */
-
-    /* indicate that this is a standard subscription. This indicates
-       that the subscription will be common to all processes. Thus,
-       the resulting data can be consolidated into a
-       process-independent message and broadcast to all processes */
-    subs = &sub;
-    if (ORTE_SUCCESS !=
-        (rc = orte_schema.get_std_subscription_name(&sub.name,
-                                                    OMPI_ATTRIBUTE_SUBSCRIPTION, ORTE_PROC_MY_NAME->jobid))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
+    /* If the universe size is set, then use it. Otherwise default
+     * to the size of MPI_COMM_WORLD */
+    univ_size = getenv("OMPI_UNIVERSE_SIZE");
+    if (NULL == univ_size || (usize = strtol(univ_size, NULL, 0)) <= 0) {
+        ret = set_f(MPI_UNIVERSE_SIZE, ompi_comm_size(MPI_COMM_WORLD));
+    } else {
+        ret = set_f(MPI_UNIVERSE_SIZE, usize);
+    }
+    if (OMPI_SUCCESS != ret) {
+        return ret;
     }
-    sub.action = ORTE_GPR_NOTIFY_DELETE_AFTER_TRIG;
-    sub.values = values;
-    sub.cnt = 1;
     
-    if (ORTE_SUCCESS != (rc = orte_schema.get_job_segment_name(&jobseg, ORTE_PROC_MY_NAME->jobid))) {
-        ORTE_ERROR_LOG(rc);
-        free(sub.name);
-        return rc;
+    /* check the app_num - if it was set, then define it - otherwise, don't */
+    if (orte_process_info.app_num >= 0) {
+        ret = set_f(MPI_APPNUM, orte_process_info.app_num);
     }
-
-    if (ORTE_SUCCESS != (rc = orte_gpr.create_value(&(values[0]), ORTE_GPR_TOKENS_OR | ORTE_GPR_KEYS_OR | ORTE_GPR_STRIPPED,
-                                                    jobseg, 1, 0))) {
-        ORTE_ERROR_LOG(rc);
-        free(jobseg);
-        free(sub.name);
-        return rc;
-    }
-    free(jobseg);
     
-    if (ORTE_SUCCESS != (rc = orte_gpr.create_keyval(&(values[0]->keyvals[0]), ORTE_JOB_TOTAL_SLOTS_ALLOC_KEY, ORTE_UNDEF, NULL))) {
-        ORTE_ERROR_LOG(rc);
-        OBJ_RELEASE(values[0]);
-        free(sub.name);
-        return rc;
-    }
-
-    sub.cbfunc = ompi_attr_create_predefined_callback;
-    
-    /* attach ourselves to the standard stage-1 trigger */
-    trigs = &trig;
-    if (ORTE_SUCCESS !=
-        (rc = orte_schema.get_std_trigger_name(&trig.name,
-                                               ORTE_STG1_TRIGGER, ORTE_PROC_MY_NAME->jobid))) {
-        ORTE_ERROR_LOG(rc);
-        OBJ_RELEASE(values[0]);
-        free(sub.name);
-        return rc;
-    }
-
-    if (ORTE_SUCCESS != (rc = orte_gpr.subscribe(1, &subs, 1, &trigs))) {
-        ORTE_ERROR_LOG(rc);
-    }
-    OBJ_RELEASE(values[0]);
-    free(sub.name);
-    free(trig.name);
-
-    return rc;
+    return ret;
 }
 
 
@@ -274,70 +211,6 @@ int ompi_attr_free_predefined(void)
 }
 
 
-void ompi_attr_create_predefined_callback(
-    orte_gpr_notify_data_t *data,
-    void *cbdata)
-{
-    orte_gpr_value_t **value;
-    orte_std_cntr_t *cptr;
-    unsigned int universe_size;
-    int rc;
-
-    /* Query the gpr to find out how many CPUs there will be.
-       This will only return a non-empty list in a persistent
-       universe.  If we don't have a persistent universe, then just
-       default to the size of MPI_COMM_WORLD.
-
-       JMS: I think we need more here -- there are cases where you
-       wouldn't have a persistent universe but still may have a
-       comm_size(COMM_WORLD) != UNIVERSE_SIZE.  For example, say you
-       reserve 8 CPUs in a batch environment and then run ./master,
-       where the master is supposed to SPAWN the other processes.
-       Perhaps need some integration with the LLM here...?  [shrug] */
-
-    /* RHC: Needed to change this code so it wouldn't issue a gpr.get
-     * during the compound command phase of mpi_init. Since all you need
-     * is to have the data prior to dtypes etc., and since this function
-     * is called right before we send the compound command, I've changed
-     * it to a subscription and a callback function. This allows you to
-     * get the data AFTER the compound command executes. Nothing else
-     * happens in-between anyway, so this shouldn't cause a problem.
-     */
-
-    if (1 != data->cnt) {  /* only one data value should be returned, or else something is wrong - use default */
-        universe_size = ompi_comm_size(MPI_COMM_WORLD);
-    } else {
-        value = (orte_gpr_value_t**)(data->values)->addr;
-        if (NULL == value[0]) {
-            /* again, got an error - use default */
-            universe_size = ompi_comm_size(MPI_COMM_WORLD);
-        } else {
-            if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&cptr, value[0]->keyvals[0]->value, ORTE_STD_CNTR))) {
-                ORTE_ERROR_LOG(rc);
-                return;
-            }
-            universe_size = (unsigned int)(*cptr);
-        }
-    }
-
-    /* ignore errors here because there's nothing we
-       can do if there's any error anyway */
-    set_f(MPI_UNIVERSE_SIZE, universe_size);
-        
-        
-    /* the app_context index for this app was passed in via the ODLS
-     * framework and stored in the orte_process_info structure when
-     * that struct was initialized - set the corresponding attribute
-     * here
-     */
-    if (orte_process_info.app_num >= 0) {
-        set_f(MPI_APPNUM, (MPI_Fint) orte_process_info.app_num);
-    }
-
-    return;
-}
-
-
 static int create_comm(int target_keyval, bool want_inherit)
 {
     int err;
