@@ -10,7 +10,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2009 University of Houston. All rights reserved.
+ * Copyright (c) 2006-2010 University of Houston. All rights reserved.
  * Copyright (c) 2007      Cisco, Inc. All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
  * $COPYRIGHT$
@@ -258,11 +258,22 @@ int ompi_comm_finalize(void)
             comm=(ompi_communicator_t *)opal_pointer_array_get_item(&ompi_mpi_communicators, i);
             if ( NULL != comm ) {
                 /* Still here ? */
-                if ( ompi_debug_show_handle_leaks && !(OMPI_COMM_IS_FREED(comm)) ){
-                    opal_output(0,"WARNING: MPI_Comm still allocated in MPI_Finalize\n");
-                    ompi_comm_dump ( comm);
-                    OBJ_RELEASE(comm);
-                }
+		if ( !OMPI_COMM_IS_EXTRA_RETAIN(comm)) {
+		    /* For communicator that have been marked as extra_retain, we do not further
+		     * enforce to decrease the reference counter once more. These extra_retain
+		     * communicators created e.g. by the hierarch or inter module did increase
+		     * the reference count by one more than other communicators, on order to
+		     * allow for deallocation with the parent communicator. Note, that 
+		     * this only occurs if the cid of the local_comm is lower than of its
+		     * parent communicator. Read the comment in comm_activate for 
+		     * a full explanation.
+		     */
+		    if ( ompi_debug_show_handle_leaks && !(OMPI_COMM_IS_FREED(comm)) ){
+			opal_output(0,"WARNING: MPI_Comm still allocated in MPI_Finalize\n");
+			ompi_comm_dump ( comm);
+			OBJ_RELEASE(comm);
+		    }
+		}
             }
         }
     }
