@@ -23,54 +23,51 @@
 #include "ompi_config.h"
 #include "ompi/constants.h"
 #include "ompi/datatype/datatype.h"
+#include "opal/prefetch.h"
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
-
-#include "opal/util/output.h"
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
+#ifdef HAVE_NET_UIO_H
+#include <net/uio.h>
 #endif
 
+BEGIN_C_DECLS
 /*
  * CONVERTOR SECTION
  */
 /* keep the last 16 bits free for data flags */
 #define CONVERTOR_DATATYPE_MASK    0x0000FFFF
-#define CONVERTOR_USELESS          0x00010000
+#define CONVERTOR_SEND_CONVERSION  0x00010000
 #define CONVERTOR_RECV             0x00020000
 #define CONVERTOR_SEND             0x00040000
 #define CONVERTOR_HOMOGENEOUS      0x00080000
-#define CONVERTOR_CLONE            0x00100000
+#define CONVERTOR_NO_OP            0x00100000
 #define CONVERTOR_WITH_CHECKSUM    0x00200000
 #define CONVERTOR_TYPE_MASK        0x00FF0000
 #define CONVERTOR_STATE_START      0x01000000
 #define CONVERTOR_STATE_COMPLETE   0x02000000
 #define CONVERTOR_STATE_ALLOC      0x04000000
 #define CONVERTOR_COMPLETED        0x08000000
-#define CONVERTOR_COMPUTE_CRC      0x10000000
 
 typedef struct ompi_convertor_t ompi_convertor_t;
 
 typedef int32_t (*convertor_advance_fct_t)( ompi_convertor_t* pConvertor,
                                             struct iovec* iov,
                                             uint32_t* out_size,
-                                            size_t* max_data,
-                                            int32_t* freeAfter );
+                                            size_t* max_data );
 typedef void*(*memalloc_fct_t)( size_t* pLength, void* userdata );
 
 /* The master convertor struct (defined in convertor_internal.h) */
 struct ompi_convertor_master_t;
 
 typedef struct dt_stack {
-    int16_t index;    /**< index in the element description */
-    int16_t type;     /**< the type used for the last pack/unpack (original or DT_BYTE) */
-    int32_t count;    /**< number of times we still have to do it */
-    int32_t end_loop; /**< for loops the end of the loop, otherwise useless */
-    long    disp;     /**< actual displacement depending on the count field */
+    int32_t   index;    /**< index in the element description */
+    int16_t   type;     /**< the type used for the last pack/unpack (original or DT_BYTE) */
+    size_t    count;    /**< number of times we still have to do it */
+    ptrdiff_t disp;     /**< actual displacement depending on the count field */
 } dt_stack_t;
 
 /**
@@ -87,13 +84,11 @@ struct ompi_convertor_t {
     size_t                        remote_size;
     const struct ompi_datatype_t* pDesc;        /**< the datatype description associated with the convertor */
     const struct dt_type_desc*    use_desc;     /**< the version used by the convertor (normal or optimized) */
-    uint32_t                      count;        /**< the total number of full datatype elements */
-    char*                         pBaseBuf;     /**< initial buffer as supplied by the user */
+    opal_ddt_count_t              count;        /**< the total number of full datatype elements */
+    unsigned char*                pBaseBuf;     /**< initial buffer as supplied by the user */
     dt_stack_t*                   pStack;       /**< the local stack for the actual conversion */
     uint32_t                      stack_size;   /**< size of the allocated stack */
     convertor_advance_fct_t       fAdvance;     /**< pointer to the pack/unpack functions */
-    memalloc_fct_t                memAlloc_fn;  /**< pointer to the memory allocation function */
-    void*                         memAlloc_userdata;  /**< user data for the malloc function */
     struct ompi_convertor_master_t* master;     /* the master convertor */
     /* All others fields get modified for every call to pack/unpack functions */
     uint32_t                      stack_pos;    /**< the actual position on the stack */
@@ -101,10 +96,10 @@ struct ompi_convertor_t {
     uint32_t                      partial_length;  /**< amount of data left over from the last unpack */
     uint32_t                      checksum;     /**< checksum computed by pack/unpack operation */
     uint32_t                      csum_ui1;     /**< partial checksum computed by pack/unpack operation */
-    uint32_t                      csum_ui2;     /**< partial checksum computed by pack/unpack operation */
+    size_t                        csum_ui2;     /**< partial checksum computed by pack/unpack operation */
     dt_stack_t                    static_stack[DT_STATIC_STACK_SIZE];  /**< local stack for small datatypes */
 };
-OBJ_CLASS_DECLARATION( ompi_convertor_t );
+OMPI_DECLSPEC OBJ_CLASS_DECLARATION( ompi_convertor_t );
 
 /* Base convertor for all external32 operations */
 OMPI_DECLSPEC extern ompi_convertor_t* ompi_mpi_external32_convertor;
@@ -127,8 +122,7 @@ OMPI_DECLSPEC int32_t
 ompi_convertor_pack( ompi_convertor_t* pConv,
                      struct iovec* iov,
                      uint32_t* out_size,
-                     size_t* max_data,
-                     int32_t* freeAfter );
+                     size_t* max_data );
 
 /*
  *
@@ -137,8 +131,7 @@ OMPI_DECLSPEC int32_t
 ompi_convertor_unpack( ompi_convertor_t* pConv,
                        struct iovec* iov,
                        uint32_t* out_size,
-                       size_t* max_data,
-                       int32_t* freeAfter );
+                       size_t* max_data );
 
 /*
  *
@@ -154,7 +147,7 @@ OMPI_DECLSPEC ompi_convertor_t* ompi_convertor_create( int32_t remote_arch, int3
  */
 static inline int ompi_convertor_cleanup( ompi_convertor_t* convertor )
 {
-    if( convertor->stack_size > DT_STATIC_STACK_SIZE ) {
+    if( OPAL_UNLIKELY(convertor->stack_size > DT_STATIC_STACK_SIZE) ) {
         free( convertor->pStack );
         convertor->pStack     = convertor->static_stack;
         convertor->stack_size = DT_STATIC_STACK_SIZE;
@@ -165,12 +158,22 @@ static inline int ompi_convertor_cleanup( ompi_convertor_t* convertor )
 }
 
 /*
- *
+ * Return:   0 if no packing is required for sending (the upper layer
+ *             can use directly the pointer to the contiguous user
+ *             buffer).
+ *           1 if data does need to be packed, i.e. heterogeneous peers
+ *             (source arch != dest arch) or non contiguous memory
+ *             layout.
  */
 static inline int32_t
 ompi_convertor_need_buffers( const ompi_convertor_t* pConvertor )
 {
-    return !ompi_ddt_is_contiguous_memory_layout( pConvertor->pDesc, pConvertor->count );
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+    if (OPAL_UNLIKELY(0 == (pConvertor->flags & CONVERTOR_HOMOGENEOUS))) return 1;
+#endif
+    if( pConvertor->flags & DT_FLAG_NO_GAPS ) return 0;
+    if( (pConvertor->count == 1) && (pConvertor->flags & DT_FLAG_CONTIGUOUS) ) return 0;
+    return 1;
 }
 
 /*
@@ -193,6 +196,19 @@ ompi_convertor_get_unpacked_size( const ompi_convertor_t* pConv,
     *pSize = pConv->remote_size;
 }
 
+/**
+ * Return the current absolute position of the next pack/unpack. This function is
+ * mostly useful for contiguous datatypes, when we need to get the pointer to the
+ * contiguous piece of memory.
+ */
+static inline void
+ompi_convertor_get_current_pointer( const ompi_convertor_t* pConv,
+                                    void** position )
+{
+    unsigned char* base = pConv->pBaseBuf + pConv->bConverted + pConv->pDesc->true_lb;
+    *position = (void*)base;
+}
+
 /*
  *
  */
@@ -212,6 +228,7 @@ ompi_convertor_copy_and_prepare_for_send( const ompi_convertor_t* pSrcConv,
     convertor->remoteArch = pSrcConv->remoteArch;
     convertor->flags      = (pSrcConv->flags | flags);
     convertor->master     = pSrcConv->master;
+
     return ompi_convertor_prepare_for_send( convertor, datatype, count, pUserBuf );
 }
 
@@ -248,29 +265,28 @@ static inline int32_t
 ompi_convertor_set_position( ompi_convertor_t* convertor,
                              size_t* position )
 {
-    /*
-     * If the convertor is already at the correct position we are happy.
-     */
-    if( (*position) == convertor->bConverted ) return OMPI_SUCCESS;
-
     /*
      * Do not allow the convertor to go outside the data boundaries. This test include
      * the check for datatype with size zero as well as for convertors with a count of zero.
      */
-    if( convertor->local_size <= *position) {
+    if( OPAL_UNLIKELY(convertor->local_size <= *position) ) {
         convertor->flags |= CONVERTOR_COMPLETED;
         convertor->bConverted = convertor->local_size;
         *position = convertor->bConverted;
         return OMPI_SUCCESS;
     }
 
+    /*
+     * If the convertor is already at the correct position we are happy.
+     */
+    if( OPAL_LIKELY((*position) == convertor->bConverted) ) return OMPI_SUCCESS;
+
     /* Remove the completed flag if it's already set */
     convertor->flags &= ~CONVERTOR_COMPLETED;
 
     if( !(convertor->flags & CONVERTOR_WITH_CHECKSUM) &&
         (convertor->flags & DT_FLAG_NO_GAPS) &&
-        ((convertor->flags & CONVERTOR_SEND) ||
-         (convertor->flags & CONVERTOR_HOMOGENEOUS)) ) {
+        (convertor->flags & (CONVERTOR_SEND | CONVERTOR_HOMOGENEOUS)) ) {
         /* Contiguous and no checkpoint and no homogeneous unpack */
         convertor->bConverted = *position;
         return OMPI_SUCCESS;
@@ -283,13 +299,12 @@ ompi_convertor_set_position( ompi_convertor_t* convertor,
  */
 static inline int32_t
 ompi_convertor_personalize( ompi_convertor_t* convertor, uint32_t flags,
-                            size_t* position,
-                            memalloc_fct_t allocfn, void* userdata )
+                            size_t* position )
 {
     convertor->flags |= flags;
-    convertor->memAlloc_fn = allocfn;
-    convertor->memAlloc_userdata = userdata;
 
+    if( OPAL_UNLIKELY(NULL == position) )
+        return OMPI_SUCCESS;
     return ompi_convertor_set_position( convertor, position );
 }
 
@@ -324,8 +339,5 @@ OMPI_DECLSPEC int
 ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
                                         size_t* position );
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
-
+END_C_DECLS
 #endif  /* CONVERTOR_H_HAS_BEEN_INCLUDED */
