@@ -56,7 +56,7 @@ ompi_pack_homogeneous_contig_function( ompi_convertor_t* pConv,
                                        size_t* max_data )
 {
     dt_stack_t* pStack = pConv->pStack;
-    char *source_base = NULL;
+    unsigned char *source_base = NULL;
     uint32_t iov_count;
     size_t length = pConv->local_size - pConv->bConverted, initial_amount = pConv->bConverted;
     ptrdiff_t initial_displ = pConv->use_desc->desc[pConv->use_desc->used].end_loop.first_elem_disp;
@@ -103,7 +103,7 @@ ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
 {
     const ompi_datatype_t* pData = pConv->pDesc;
     dt_stack_t* pStack = pConv->pStack;
-    char *user_memory, *packed_buffer;
+    unsigned char *user_memory, *packed_buffer;
     uint32_t i, index, iov_count;
     size_t max_allowed, total_bytes_converted = 0;
     ptrdiff_t extent;
@@ -185,7 +185,7 @@ ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
             uint32_t counter;
             size_t done;
 
-            packed_buffer = iov[iov_count].iov_base;
+            packed_buffer = (unsigned char *) iov[iov_count].iov_base;
             done = pConv->bConverted - i * pData->size;  /* partial data from last pack */
             if( done != 0 ) {  /* still some data to copy from the last time */
                 done = pData->size - done;
@@ -253,7 +253,7 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
     dt_elem_desc_t* description;
     dt_elem_desc_t* pElem;
     const ompi_datatype_t *pData = pConvertor->pDesc;
-    char *source_base, *destination;
+    unsigned char *source_base, *destination;
     size_t iov_len_local;
     uint32_t iov_count;
 
@@ -281,7 +281,7 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
                            pConvertor->stack_pos, pStack->index, (int)pStack->count, (long)pStack->disp ); );
 
     for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
-        destination = iov[iov_count].iov_base;
+        destination = (unsigned char *) iov[iov_count].iov_base;
         iov_len_local = iov[iov_count].iov_len;
         while( 1 ) {
             while( pElem->elem.common.flags & DT_FLAG_DATA ) {
