@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -40,18 +40,18 @@ size_t ompi_datatype_memcpy_block_size = 128 * 1024;
 
 static inline void copy_predefined_data( const dt_elem_desc_t* ELEM,
                                          const ompi_datatype_t* DATATYPE,
-                                         char* SOURCE_BASE,
+                                         unsigned char* SOURCE_BASE,
                                          int32_t TOTAL_COUNT,
                                          uint32_t COUNT,
-                                         char* SOURCE,
-                                         char* DESTINATION,
+                                         unsigned char* SOURCE,
+                                         unsigned char* DESTINATION,
                                          size_t* SPACE )
 {
     uint32_t _copy_count = (COUNT);
 	size_t _copy_blength;
     const ddt_elem_desc_t* _elem = &((ELEM)->elem);
-    char* _source = (SOURCE) + _elem->disp;
-    char* _destination = (DESTINATION) + _elem->disp;
+    unsigned char* _source = (SOURCE) + _elem->disp;
+    unsigned char* _destination = (DESTINATION) + _elem->disp;
 
     _copy_blength = ompi_ddt_basicDatatypes[_elem->common.type]->size;
 
@@ -83,17 +83,17 @@ static inline void copy_predefined_data( const dt_elem_desc_t* ELEM,
 
 static inline void copy_contiguous_loop( const dt_elem_desc_t* ELEM,
                                          const ompi_datatype_t* DATATYPE,
-                                         char* SOURCE_BASE,
+                                         unsigned char* SOURCE_BASE,
                                          int32_t TOTAL_COUNT,
                                          uint32_t COUNT,
-                                         char* SOURCE,
-                                         char* DESTINATION,
+                                         unsigned char* SOURCE,
+                                         unsigned char* DESTINATION,
                                          size_t* SPACE )
 {
     ddt_loop_desc_t *_loop = (ddt_loop_desc_t*)(ELEM);
     ddt_endloop_desc_t* _end_loop = (ddt_endloop_desc_t*)((ELEM) + _loop->items);
-    char* _source = (SOURCE) + _end_loop->first_elem_disp;
-    char* _destination = (DESTINATION) + _end_loop->first_elem_disp;
+    unsigned char* _source = (SOURCE) + _end_loop->first_elem_disp;
+    unsigned char* _destination = (DESTINATION) + _end_loop->first_elem_disp;
     size_t _copy_loops = (COUNT);
     uint32_t _i;
 
@@ -135,7 +135,8 @@ int32_t ompi_ddt_copy_content_same_ddt( const ompi_datatype_t* datatype, int32_t
     dt_elem_desc_t* description;
     dt_elem_desc_t* pElem;
     size_t iov_len_local;
-    char *source = source_base, *destination = destination_base;
+    unsigned char *source = (unsigned char*)source_base,
+                  *destination = (unsigned char*)destination_base;
 
     DO_DEBUG( opal_output( 0, "ompi_ddt_copy_content_same_ddt( %p, %d, dst %p, src %p )\n",
                            (void*)datatype, count, destination_base, source_base ); );
@@ -149,41 +150,44 @@ int32_t ompi_ddt_copy_content_same_ddt( const ompi_datatype_t* datatype, int32_t
     /* If we have to copy a contiguous datatype then simply
      * do a memcpy.
      */
-    if( (datatype->flags & DT_FLAG_CONTIGUOUS) == DT_FLAG_CONTIGUOUS ) {
+    if( datatype->flags & DT_FLAG_CONTIGUOUS ) {
         ptrdiff_t extent = (datatype->ub - datatype->lb);
         /* Now that we know the datatype is contiguous, we should move the 2 pointers
          * source and destination to the correct displacement.
          */
-        destination += datatype->lb;
-        source      += datatype->lb;
+        destination += datatype->true_lb;
+        source      += datatype->true_lb;
         if( (ptrdiff_t)datatype->size == extent ) {  /* all contiguous == no gaps around */
-            size_t total_length = datatype->size * count;
+            size_t total_length = iov_len_local;
             size_t memcpy_chunk = ompi_datatype_memcpy_block_size;
             while( total_length > 0 ) {
                 if( memcpy_chunk > total_length ) memcpy_chunk = total_length;
                 OMPI_DDT_SAFEGUARD_POINTER( destination, memcpy_chunk,
-                                            destination, datatype, count );
-                DO_DEBUG( opal_output( 0, "copy c1. memcpy( %p, %p, %ld ) => space %lu\n",
-                                       destination, source, memcpy_chunk, (unsigned long)iov_len_local ); );
+                                            (unsigned char*)destination_base, datatype, count );
+                OMPI_DDT_SAFEGUARD_POINTER( source, memcpy_chunk,
+                                            (unsigned char*)source_base, datatype, count );
+                DO_DEBUG( opal_output( 0, "copy c1. memcpy( %p, %p, %lu ) => space %lu\n",
+                                       destination, source, (unsigned long)memcpy_chunk, (unsigned long)total_length ); );
                 MEMCPY( destination, source, memcpy_chunk );
                 destination   += memcpy_chunk;
                 source        += memcpy_chunk;
                 total_length  -= memcpy_chunk;
-                iov_len_local -= memcpy_chunk;
-            }
-        } else {
-            for( pos_desc = 0; (int32_t)pos_desc < count; pos_desc++ ) {
-                OMPI_DDT_SAFEGUARD_POINTER( destination, datatype->size,
-                                            destination, datatype, count );
-                DO_DEBUG( opal_output( 0, "copy c2. memcpy( %p, %p, %ld ) => space %lu\n",
-                                       destination, source, datatype->size,
-                                       (unsigned long)(iov_len_local - (pos_desc * datatype->size)) ); );
-                MEMCPY( destination, source, datatype->size );
-                destination += extent;
-                source += extent;
             }
+            return 0;  /* completed */
         }
-        return 0;
+        for( pos_desc = 0; (int32_t)pos_desc < count; pos_desc++ ) {
+            OMPI_DDT_SAFEGUARD_POINTER( destination, datatype->size,
+                                        (unsigned char*)destination_base, datatype, count );
+            OMPI_DDT_SAFEGUARD_POINTER( source, datatype->size,
+                                        (unsigned char*)source_base, datatype, count );
+            DO_DEBUG( opal_output( 0, "copy c2. memcpy( %p, %p, %lu ) => space %lu\n",
+                                   destination, source, (unsigned long)datatype->size,
+                                   (unsigned long)(iov_len_local - (pos_desc * datatype->size)) ); );
+            MEMCPY( destination, source, datatype->size );
+            destination += extent;
+            source += extent;
+        }
+        return 0;  /* completed */
     }
 
     pStack = (dt_stack_t*)alloca( sizeof(dt_stack_t) * (datatype->btypes[DT_LOOP] + 1) );
@@ -208,7 +212,7 @@ int32_t ompi_ddt_copy_content_same_ddt( const ompi_datatype_t* datatype, int32_t
     while( 1 ) {
         while( OPAL_LIKELY(pElem->elem.common.flags & DT_FLAG_DATA) ) {
             /* now here we have a basic datatype */
-            COPY_PREDEFINED_DATATYPE( pElem, datatype, source_base, count, count_desc,
+            COPY_PREDEFINED_DATATYPE( pElem, datatype, (unsigned char*)source_base, count, count_desc,
                                       source, destination, iov_len_local );
             pos_desc++;  /* advance to the next data */
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
@@ -233,8 +237,8 @@ int32_t ompi_ddt_copy_content_same_ddt( const ompi_datatype_t* datatype, int32_t
                     pStack->disp += description[pStack->index].loop.extent;
                 }
             }
-            source      = source_base + pStack->disp;
-            destination = destination_base + pStack->disp;
+            source      = (unsigned char*)source_base + pStack->disp;
+            destination = (unsigned char*)destination_base + pStack->disp;
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
             DO_DEBUG( opal_output( 0, "copy new_loop count %d stack_pos %d pos_desc %d disp %ld space %lu\n",
                                    (int)pStack->count, stack_pos, pos_desc, (long)pStack->disp, (unsigned long)iov_len_local ); );
@@ -242,7 +246,7 @@ int32_t ompi_ddt_copy_content_same_ddt( const ompi_datatype_t* datatype, int32_t
         if( DT_LOOP == pElem->elem.common.type ) {
             ptrdiff_t local_disp = (ptrdiff_t)source;
             if( pElem->loop.common.flags & DT_FLAG_CONTIGUOUS ) {
-                COPY_CONTIGUOUS_LOOP( pElem, datatype, source_base, count, count_desc,
+                COPY_CONTIGUOUS_LOOP( pElem, datatype, (unsigned char*)source_base, count, count_desc,
                                       source, destination, iov_len_local );
                 pos_desc += pElem->loop.items + 1;
                 goto update_loop_description;
@@ -252,11 +256,12 @@ int32_t ompi_ddt_copy_content_same_ddt( const ompi_datatype_t* datatype, int32_t
                         pStack->disp + local_disp);
             pos_desc++;
         update_loop_description:  /* update the current state */
-            source      = source_base + pStack->disp;
-            destination = destination_base + pStack->disp;
+            source      = (unsigned char*)source_base + pStack->disp;
+            destination = (unsigned char*)destination_base + pStack->disp;
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
             DDT_DUMP_STACK( pStack, stack_pos, pElem, "advance loop" );
             continue;
         }
     }
 }
+
