@@ -9,7 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -26,18 +27,20 @@
 #include <stdio.h>
 
 #include "orte/util/proc_info.h"
-#include "orte/util/sys_info.h"
+#include "orte/util/name_fns.h"
+#include "orte/util/show_help.h"
+#include "orte/runtime/orte_globals.h"
 
-extern int mca_btl_base_debug;
+OMPI_DECLSPEC extern int mca_btl_base_verbose;
 
-extern int mca_btl_base_err(const char*, ...);
-extern int mca_btl_base_out(const char*, ...);
+OMPI_DECLSPEC extern int mca_btl_base_err(const char*, ...);
+OMPI_DECLSPEC extern int mca_btl_base_out(const char*, ...);
 
 #define BTL_OUTPUT(args)                                     \
 do {                                                         \
-    mca_btl_base_out("[%s][%lu,%lu,%lu][%s:%d:%s] ",         \
-            orte_system_info.nodename,                       \
-            ORTE_NAME_ARGS(orte_process_info.my_name),       \
+    mca_btl_base_out("[%s]%s[%s:%d:%s] ",         \
+            orte_process_info.nodename,                       \
+            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),       \
             __FILE__, __LINE__, __func__);                   \
     mca_btl_base_out args;                                   \
     mca_btl_base_out("\n");                                  \
@@ -46,9 +49,9 @@ do {                                                         \
 
 #define BTL_ERROR(args)                                      \
 do {                                                         \
-    mca_btl_base_err("[%s][%lu,%lu,%lu][%s:%d:%s] ",         \
-            orte_system_info.nodename,                       \
-            ORTE_NAME_ARGS(orte_process_info.my_name),       \
+    mca_btl_base_err("[%s]%s[%s:%d:%s] ",         \
+            orte_process_info.nodename,                       \
+            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),       \
             __FILE__, __LINE__, __func__);                   \
     mca_btl_base_err args;                                   \
     mca_btl_base_err("\n");                                  \
@@ -56,10 +59,10 @@ do {                                                         \
 
 #define BTL_PEER_ERROR(proc, args)                               \
 do {                                                             \
-    mca_btl_base_err("[%lu,%lu,%lu][%s:%d:%s] from %s ",         \
-                     ORTE_NAME_ARGS(orte_process_info.my_name),  \
+    mca_btl_base_err("%s[%s:%d:%s] from %s ",         \
+                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),  \
                      __FILE__, __LINE__, __func__,               \
-                     orte_system_info.nodename);                 \
+                     orte_process_info.nodename);                 \
     if(proc && proc->proc_hostname) {                            \
         mca_btl_base_err("to: %s ", proc->proc_hostname);        \
     }                                                            \
@@ -69,43 +72,27 @@ do {                                                             \
 
 
 #if OMPI_ENABLE_DEBUG
-#define BTL_DEBUG(args)                                      \
-do {                                                         \
-   if(mca_btl_base_debug) {                                  \
-        mca_btl_base_err("[%s][%lu,%lu,%lu][%s:%d:%s] ",     \
-                orte_system_info.nodename,                   \
-                ORTE_NAME_ARGS(orte_process_info.my_name),   \
-                __FILE__, __LINE__, __func__);               \
-        mca_btl_base_err args;                               \
-        mca_btl_base_err("\n");                              \
-   }                                                         \
-} while(0); 
 #define BTL_VERBOSE(args)                                    \
 do {                                                         \
-   if(mca_btl_base_debug > 1) {                              \
-        mca_btl_base_err("[%s][%lu,%lu,%lu][%s:%d:%s] ",     \
-                orte_system_info.nodename,                   \
-                ORTE_NAME_ARGS(orte_process_info.my_name),   \
+   if(mca_btl_base_verbose > 0) {                            \
+        mca_btl_base_err("[%s]%s[%s:%d:%s] ",                \
+                orte_process_info.nodename,                   \
+                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),  \
                 __FILE__, __LINE__, __func__);               \
         mca_btl_base_err args;                               \
         mca_btl_base_err("\n");                              \
    }                                                         \
 } while(0); 
 #else
-#define BTL_DEBUG(args) 
 #define BTL_VERBOSE(args) 
 #endif
 
 #endif
 
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
-extern void mca_btl_base_error_no_nics(const char* transport, 
-                                       const char* nic_name);
+OMPI_DECLSPEC extern void mca_btl_base_error_no_nics(const char* transport, 
+                                                     const char* nic_name);
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
