@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -22,15 +23,19 @@
 #include "opal/event/event.h"
 #include "opal/util/if.h"
 #include "opal/util/argv.h"
-#include "opal/util/output.h"
+#include "orte/util/show_help.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/request/request.h"
 
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #include "opal/mca/base/mca_base_param.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "ompi/mca/mpool/base/base.h" 
-#include "ompi/mca/mpool/gm/mpool_gm.h"
+#include "ompi/mca/mpool/rdma/mpool_rdma.h"
 #include "btl_gm.h"
 #include "btl_gm_frag.h"
 #include "btl_gm_endpoint.h" 
@@ -39,12 +44,18 @@
 #include "ompi/datatype/convertor.h" 
 #include "btl_gm_endpoint.h"
 #include "orte/util/proc_info.h"
-#include "ompi/mca/pml/base/pml_base_module_exchange.h"
+#include "orte/runtime/orte_globals.h"
+#include "orte/util/name_fns.h"
+#include "ompi/runtime/ompi_module_exchange.h"
+#include "ompi/runtime/mpiruntime.h"
 
 
 #if OMPI_ENABLE_PROGRESS_THREADS
 static void* mca_btl_gm_progress_thread( opal_object_t* arg );
 #endif
+static int gm_reg_mr(void *reg_data, void *base, size_t size,
+        mca_mpool_base_registration_t *reg);
+static int gm_dereg_mr(void *reg_data, mca_mpool_base_registration_t *reg);
 
 
 mca_btl_gm_component_t mca_btl_gm_component = {
@@ -53,10 +64,7 @@ mca_btl_gm_component_t mca_btl_gm_component = {
            about the component itself */
 
         {
-            /* Indicate that we are a pml v1.0.0 component (which also implies a
-               specific MCA version) */
-
-            MCA_BTL_BASE_VERSION_1_0_0,
+            MCA_BTL_BASE_VERSION_2_0_0,
 
             "gm", /* MCA component name */
             OMPI_MAJOR_VERSION,  /* MCA component major version */
@@ -65,13 +73,9 @@ mca_btl_gm_component_t mca_btl_gm_component = {
             mca_btl_gm_component_open,  /* component open */
             mca_btl_gm_component_close  /* component close */
         },
-
-        /* Next the MCA v1.0.0 component meta data */
-
         {
-            /* Whether the component is checkpointable or not */
-
-            false
+            /* The component is not checkpoint ready */
+            MCA_BASE_METADATA_PARAM_NONE
         },
 
         mca_btl_gm_component_init,  
@@ -111,8 +115,6 @@ static inline int mca_btl_gm_param_register_int(
 
 int mca_btl_gm_component_open(void)
 {  
-    int param, value;
-
     /* initialize state */
     mca_btl_gm_component.gm_num_btls=0;
     mca_btl_gm_component.gm_btls=NULL;
@@ -131,7 +133,7 @@ int mca_btl_gm_component_open(void)
     mca_btl_gm_component.gm_debug = 
         mca_btl_gm_param_register_int("debug", 0); 
     mca_btl_gm_component.gm_mpool_name = 
-        mca_btl_gm_param_register_string("mpool", "gm"); 
+        mca_btl_gm_param_register_string("mpool", "rdma");
     mca_btl_gm_component.gm_max_ports = 
         mca_btl_gm_param_register_int("max_ports", 16); 
     mca_btl_gm_component.gm_max_boards = 
@@ -142,33 +144,28 @@ int mca_btl_gm_component_open(void)
         mca_btl_gm_param_register_int("num_high_priority", 8); 
     mca_btl_gm_component.gm_num_repost = 
         mca_btl_gm_param_register_int("num_repost", 4); 
-    mca_btl_gm_component.gm_num_mru = 
-        mca_btl_gm_param_register_int("num_mru", 64); 
     mca_btl_gm_component.gm_port_name=
         mca_btl_gm_param_register_string("port_name", "OMPI"); 
 
     /* register gm module parameters */
-    mca_btl_gm_module.super.btl_exclusivity =
-        mca_btl_gm_param_register_int ("exclusivity", MCA_BTL_EXCLUSIVITY_DEFAULT);
-    mca_btl_gm_module.super.btl_eager_limit = 
-        mca_btl_gm_param_register_int ("eager_limit", 32*1024);
-    mca_btl_gm_module.super.btl_min_send_size =
-        mca_btl_gm_param_register_int ("min_send_size", 32*1024);
-    mca_btl_gm_module.super.btl_max_send_size =
-        mca_btl_gm_param_register_int ("max_send_size", 64*1024);
-    mca_btl_gm_module.super.btl_min_rdma_size = 
-        mca_btl_gm_param_register_int("min_rdma_size", 512*1024); 
-    mca_btl_gm_module.super.btl_max_rdma_size = 
-        mca_btl_gm_param_register_int("max_rdma_size", 128*1024); 
+    mca_btl_gm_module.super.btl_exclusivity = MCA_BTL_EXCLUSIVITY_DEFAULT;
+    mca_btl_gm_module.super.btl_eager_limit = 32*1024;
+    mca_btl_gm_module.super.btl_rndv_eager_limit = 32*1024;
+    mca_btl_gm_module.super.btl_max_send_size = 64*1024;
+    mca_btl_gm_module.super.btl_rdma_pipeline_send_length = 512*1024;
+    mca_btl_gm_module.super.btl_rdma_pipeline_frag_size = 128*1024;
+    mca_btl_gm_module.super.btl_min_rdma_pipeline_size = 128*1024;
 #if OMPI_MCA_BTL_GM_HAVE_RDMA_PUT 
-    mca_btl_gm_module.super.btl_flags  = 
-        mca_btl_gm_param_register_int("flags", MCA_BTL_FLAGS_PUT); 
+    mca_btl_gm_module.super.btl_flags  =  MCA_BTL_FLAGS_PUT |
+        MCA_BTL_FLAGS_NEED_ACK | MCA_BTL_FLAGS_NEED_CSUM; 
 #else
     mca_btl_gm_module.super.btl_flags = MCA_BTL_FLAGS_SEND;
 #endif
-    mca_btl_gm_module.super.btl_bandwidth  = 
-        mca_btl_gm_param_register_int("bandwidth", 250); 
-
+    mca_btl_gm_module.super.btl_bandwidth = 250;
+    mca_btl_gm_module.super.btl_latency = 0;
+    mca_btl_base_param_register(&mca_btl_gm_component.super.btl_version,
+            &mca_btl_gm_module.super);
+    
     /* compute the eager frag size */
     mca_btl_gm_component.gm_eager_frag_size =
         gm_min_size_for_length(mca_btl_gm_module.super.btl_eager_limit) - 1;
@@ -183,11 +180,6 @@ int mca_btl_gm_component_open(void)
         gm_max_length_for_size(mca_btl_gm_component.gm_max_frag_size) -
         sizeof(mca_btl_base_header_t);
 
-    /* leave pinned option */
-    value = 0;
-    param = mca_base_param_find("mpi", NULL, "leave_pinned");
-    mca_base_param_lookup_int(param, &value);
-    mca_btl_gm_component.leave_pinned = value;
     return OMPI_SUCCESS;
 }
 
@@ -201,6 +193,35 @@ int mca_btl_gm_component_close(void)
     return OMPI_SUCCESS;
 }
 
+static int gm_reg_mr(void *reg_data, void *base, size_t size,
+        mca_mpool_base_registration_t *reg)
+{
+    struct gm_port *port = (struct gm_port*)reg_data;
+    int rc;
+
+    rc = gm_register_memory(port, base, size);
+
+    if(rc != GM_SUCCESS)
+        return OMPI_ERR_OUT_OF_RESOURCE;
+
+    return MPI_SUCCESS;
+}
+
+static int gm_dereg_mr(void *reg_data, mca_mpool_base_registration_t *reg)
+{
+    struct gm_port *port = (struct gm_port*)reg_data;
+    int rc;
+
+    rc = gm_deregister_memory(port, reg->base, reg->bound - reg->base + 1);
+
+    if(rc != GM_SUCCESS) {
+        opal_output(0, "%s: error unpinning gm memory errno says %s\n",
+                 __func__, strerror(errno));
+        return OMPI_ERROR;
+    }
+
+    return OMPI_SUCCESS;
+}
 
 /**
  * Initialize module instance
@@ -220,11 +241,11 @@ mca_btl_gm_module_init (mca_btl_gm_module_t * btl)
     OBJ_CONSTRUCT(&btl->gm_frag_user, ompi_free_list_t);
     OBJ_CONSTRUCT(&btl->gm_pending, opal_list_t);
     OBJ_CONSTRUCT(&btl->gm_repost, opal_list_t);
-    OBJ_CONSTRUCT(&btl->gm_mru_reg, opal_list_t);
 #if OMPI_ENABLE_PROGRESS_THREADS
     OBJ_CONSTRUCT(&btl->gm_thread, opal_thread_t);
 #endif
 
+    btl->error_cb = NULL;
     /* query nic tokens */
     btl->gm_num_send_tokens = gm_num_send_tokens (btl->port);
     btl->gm_max_send_tokens = btl->gm_num_send_tokens;
@@ -243,7 +264,10 @@ mca_btl_gm_module_init (mca_btl_gm_module_t * btl)
     }
                                                                                                   
     /* initialize memory pool */
-    resources.port = btl->port;
+    resources.reg_data = (void*)btl->port;
+    resources.sizeof_reg = sizeof(mca_mpool_base_registration_t);
+    resources.register_mem = gm_reg_mr;
+    resources.deregister_mem = gm_dereg_mr;
     btl->super.btl_mpool = mca_mpool_base_module_create(
         mca_btl_gm_component.gm_mpool_name,
         &btl->super,
@@ -254,29 +278,37 @@ mca_btl_gm_module_init (mca_btl_gm_module_t * btl)
     }
  
     /* initialize free lists */
-    ompi_free_list_init( &btl->gm_frag_eager,
-                         sizeof (mca_btl_gm_frag_eager_t) + (1 << mca_btl_gm_component.gm_eager_frag_size),
-                         OBJ_CLASS (mca_btl_gm_frag_eager_t),
-                         btl->gm_max_send_tokens,  
-                         mca_btl_gm_component.gm_free_list_max, 
-                         mca_btl_gm_component.gm_free_list_inc,
-                         btl->super.btl_mpool ); 
+    ompi_free_list_init_new( &btl->gm_frag_eager,
+            sizeof (mca_btl_gm_frag_eager_t),
+            CACHE_LINE_SIZE,
+            OBJ_CLASS (mca_btl_gm_frag_eager_t),
+            (1 << mca_btl_gm_component.gm_eager_frag_size) + sizeof (uintptr_t),
+            CACHE_LINE_SIZE, 
+            btl->gm_max_send_tokens,  
+            mca_btl_gm_component.gm_free_list_max, 
+            mca_btl_gm_component.gm_free_list_inc,
+            btl->super.btl_mpool ); 
 
-    ompi_free_list_init( &btl->gm_frag_max,
-                         sizeof (mca_btl_gm_frag_max_t) + (1 << mca_btl_gm_component.gm_max_frag_size),
-                         OBJ_CLASS (mca_btl_gm_frag_max_t),
-                         btl->gm_max_recv_tokens,
-                         mca_btl_gm_component.gm_free_list_max, 
-                         mca_btl_gm_component.gm_free_list_inc,
-                         btl->super.btl_mpool ); 
+    ompi_free_list_init_new( &btl->gm_frag_max,
+            sizeof (mca_btl_gm_frag_max_t),
+            CACHE_LINE_SIZE,
+            OBJ_CLASS (mca_btl_gm_frag_max_t),
+            (1 << mca_btl_gm_component.gm_max_frag_size) + sizeof (uintptr_t),
+            CACHE_LINE_SIZE,
+            btl->gm_max_recv_tokens,
+            mca_btl_gm_component.gm_free_list_max, 
+            mca_btl_gm_component.gm_free_list_inc,
+            btl->super.btl_mpool ); 
 
-    ompi_free_list_init( &btl->gm_frag_user,
-                         sizeof (mca_btl_gm_frag_user_t),
-                         OBJ_CLASS (mca_btl_gm_frag_user_t),
-                         mca_btl_gm_component.gm_free_list_num,  
-                         mca_btl_gm_component.gm_free_list_max, 
-                         mca_btl_gm_component.gm_free_list_inc,
-                         NULL );
+    ompi_free_list_init_new( &btl->gm_frag_user,
+            sizeof (mca_btl_gm_frag_user_t),
+            CACHE_LINE_SIZE,
+            OBJ_CLASS (mca_btl_gm_frag_user_t),
+            0,CACHE_LINE_SIZE,
+            mca_btl_gm_component.gm_free_list_num,  
+            mca_btl_gm_component.gm_free_list_max, 
+            mca_btl_gm_component.gm_free_list_inc,
+            NULL );
 
 
     /* post receive buffers */
@@ -286,6 +318,7 @@ mca_btl_gm_module_init (mca_btl_gm_module_t * btl)
         if(NULL == frag) {
             return rc;
         }
+        frag->type = MCA_BTL_GM_EAGER;
         frag->base.des_src = NULL;
         frag->base.des_src_cnt = 0;
         frag->base.des_dst = &frag->segment;
@@ -300,6 +333,7 @@ mca_btl_gm_module_init (mca_btl_gm_module_t * btl)
         if(NULL == frag) {
             return rc;
         }
+        frag->type = MCA_BTL_GM_SEND;
         frag->base.des_src = NULL;
         frag->base.des_src_cnt = 0;
         frag->base.des_dst = &frag->segment;
@@ -399,8 +433,12 @@ static int mca_btl_gm_discover( void )
 #endif  /* GM_API_VERSION > 0x200 */
 
         if(mca_btl_gm_component.gm_debug > 0) {
-            opal_output(0, "[%d,%d,%d] gm_port %08X, board %lu, global %lu node %lu port %lu\n", 
-                ORTE_NAME_ARGS(orte_process_info.my_name), port, board_no, global_id, node_id, port_no);
+            opal_output(0,
+                        "%s gm_port %08lX, "
+                        "board %" PRIu32 ", global %" PRIu32 " "
+                        "node %" PRIu32 "port %" PRIu32 "\n", 
+                        ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), 
+                        (unsigned long) port, board_no, global_id, node_id, port_no);
         }
 
         if((rc = mca_btl_gm_module_init(btl)) != OMPI_SUCCESS) {
@@ -417,8 +455,6 @@ static int mca_btl_gm_discover( void )
     return OMPI_SUCCESS;
 }
 
-
-
 /*
  *  Register GM component addressing information. The MCA framework
  *  will make this available to all peers.
@@ -445,7 +481,7 @@ mca_btl_gm_modex_send(void)
             MCA_BTL_GM_ADDR_HTON(addrs[i]);
         }
     }
-    rc = mca_pml_base_modex_send (&mca_btl_gm_component.super.btl_version, addrs, size);
+    rc = ompi_modex_send (&mca_btl_gm_component.super.btl_version, addrs, size);
     if (NULL != addrs) {
         free (addrs);
     }
@@ -468,6 +504,14 @@ mca_btl_gm_component_init (int *num_btl_modules,
 
     OPAL_THREAD_LOCK(&mca_btl_gm_component.gm_lock);
 
+    /* Currently refuse to run if MPI_THREAD_MULTIPLE is enabled */
+    if (ompi_mpi_thread_multiple && !mca_btl_base_thread_multiple_override) {
+        mca_btl_gm_component.gm_num_btls = 0;
+        mca_btl_gm_modex_send();
+        OPAL_THREAD_UNLOCK(&mca_btl_gm_component.gm_lock);
+        return NULL;
+    }
+
     /* try to initialize GM */
     if( GM_SUCCESS != gm_init() ) {
         opal_output( 0, "[%s:%d] error in initializing the gm library\n", __FILE__, __LINE__ );
@@ -553,17 +597,22 @@ int mca_btl_gm_component_progress()
             case GM_FAST_HIGH_PEER_RECV_EVENT:
                 {
                 unsigned char* buffer = (unsigned char*)gm_ntohp(event->recv.buffer);
-                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)(buffer - sizeof(mca_btl_gm_frag_t));
+                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)*((uintptr_t*)(buffer - sizeof(uintptr_t)));
                 mca_btl_base_header_t* hdr = (mca_btl_base_header_t *)gm_ntohp(event->recv.message);
-                mca_btl_base_recv_reg_t* reg;
+                mca_btl_active_message_callback_t* reg;
                 frag->segment.seg_addr.pval = (hdr+1);
                 frag->segment.seg_len = gm_ntohl(event->recv.length) - sizeof(mca_btl_base_header_t);
-                reg = &btl->gm_reg[hdr->tag];
-
-                OPAL_THREAD_UNLOCK(&mca_btl_gm_component.gm_lock);
-                reg->cbfunc(&btl->super, hdr->tag, &frag->base, reg->cbdata);
-                OPAL_THREAD_LOCK(&mca_btl_gm_component.gm_lock);
+                reg = mca_btl_base_active_message_trigger + hdr->tag;
 
+                /* cbfunc may be null if interface goes down.. */
+                if(reg->cbfunc) { 
+                    OPAL_THREAD_UNLOCK(&mca_btl_gm_component.gm_lock);
+                    reg->cbfunc(&btl->super, hdr->tag, &frag->base, reg->cbdata);
+                    OPAL_THREAD_LOCK(&mca_btl_gm_component.gm_lock);
+                } else { 
+                    btl->error_cb(&btl->super, MCA_BTL_ERROR_FLAGS_FATAL);
+                    return 0;
+                }
                 MCA_BTL_GM_FRAG_POST(btl,frag);
                 count++;
                 break;
@@ -574,18 +623,22 @@ int mca_btl_gm_component_progress()
             case GM_HIGH_PEER_RECV_EVENT:
                 {
                 unsigned char* buffer = (unsigned char*)gm_ntohp(event->recv.buffer);
-                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)(buffer - sizeof(mca_btl_gm_frag_t));
+                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)*((uintptr_t*)(buffer - sizeof(uintptr_t)));
                 mca_btl_base_header_t* hdr = (mca_btl_base_header_t*)buffer;
-                mca_btl_base_recv_reg_t* reg;
+                mca_btl_active_message_callback_t* reg;
                 frag->segment.seg_addr.pval = (hdr+1);
                 frag->segment.seg_len = gm_ntohl(event->recv.length) - sizeof(mca_btl_base_header_t);
-                reg = &btl->gm_reg[hdr->tag];
-
-                OPAL_THREAD_UNLOCK(&mca_btl_gm_component.gm_lock);
-                reg->cbfunc(&btl->super, hdr->tag, &frag->base, reg->cbdata);
-                OPAL_THREAD_LOCK(&mca_btl_gm_component.gm_lock);
-
-                MCA_BTL_GM_FRAG_POST(btl,frag);
+                reg = mca_btl_base_active_message_trigger + hdr->tag;
+                if(reg->cbfunc) { 
+                    OPAL_THREAD_UNLOCK(&mca_btl_gm_component.gm_lock);
+                    reg->cbfunc(&btl->super, hdr->tag, &frag->base, reg->cbdata);
+                    OPAL_THREAD_LOCK(&mca_btl_gm_component.gm_lock);
+                    
+                    MCA_BTL_GM_FRAG_POST(btl,frag);
+                } else { 
+                    btl->error_cb(&btl->super, MCA_BTL_ERROR_FLAGS_FATAL);
+                    return 0;
+                }
                 count++;
                 break;
                 }
@@ -635,12 +688,12 @@ static void* mca_btl_gm_progress_thread( opal_object_t* arg )
             case GM_FAST_HIGH_PEER_RECV_EVENT:
                 {
                 unsigned char* buffer = (unsigned char*)gm_ntohp(event->recv.buffer);
-                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)(buffer - sizeof(mca_btl_gm_frag_t));
+                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)*((uintptr_t*)(buffer - sizeof(uintptr_t)));
                 mca_btl_base_header_t* hdr = (mca_btl_base_header_t *)gm_ntohp(event->recv.message);
-                mca_btl_base_recv_reg_t* reg;
+                mca_btl_active_message_callback_t* reg;
                 frag->segment.seg_addr.pval = (hdr+1);
                 frag->segment.seg_len = gm_ntohl(event->recv.length) - sizeof(mca_btl_base_header_t);
-                reg = &btl->gm_reg[hdr->tag];
+                reg = mca_btl_base_active_message_trigger + hdr->tag;
 
                 OPAL_THREAD_UNLOCK(&mca_btl_gm_component.gm_lock);
                 reg->cbfunc(&btl->super, hdr->tag, &frag->base, reg->cbdata);
@@ -655,12 +708,12 @@ static void* mca_btl_gm_progress_thread( opal_object_t* arg )
             case GM_HIGH_PEER_RECV_EVENT:
                 {
                 unsigned char* buffer = (unsigned char*)gm_ntohp(event->recv.buffer);
-                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)(buffer - sizeof(mca_btl_gm_frag_t));
+                mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)*((uintptr_t*)(buffer - sizeof(uintptr_t)));
                 mca_btl_base_header_t* hdr = (mca_btl_base_header_t*)buffer;
-                mca_btl_base_recv_reg_t* reg;
+                mca_btl_active_message_callback_t* reg;
                 frag->segment.seg_addr.pval = (hdr+1);
                 frag->segment.seg_len = gm_ntohl(event->recv.length) - sizeof(mca_btl_base_header_t);
-                reg = &btl->gm_reg[hdr->tag];
+                reg = mca_btl_base_active_message_trigger + hdr->tag;
 
                 OPAL_THREAD_UNLOCK(&mca_btl_gm_component.gm_lock);
                 reg->cbfunc(&btl->super, hdr->tag, &frag->base, reg->cbdata);
