@@ -16,6 +16,7 @@
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2009      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -38,6 +39,8 @@
 #include "ompi/runtime/ompi_cr.h"
 #endif
 
+#include "btl_openib_ini.h"
+
 #include "btl_openib.h"
 #include "btl_openib_frag.h"
 #include "btl_openib_proc.h"
@@ -87,7 +90,7 @@ mca_btl_openib_module_t mca_btl_openib_module = {
         mca_btl_openib_prepare_src,
         mca_btl_openib_prepare_dst,
         mca_btl_openib_send,
-        NULL, /* send immediate */
+        mca_btl_openib_sendi, /* send immediate */
         mca_btl_openib_put,
         mca_btl_openib_get,
         mca_btl_base_dump,
@@ -97,10 +100,17 @@ mca_btl_openib_module_t mca_btl_openib_module = {
     }
 };
 
+char* const mca_btl_openib_transport_name_strings[MCA_BTL_OPENIB_TRANSPORT_SIZE] = {
+    "MCA_BTL_OPENIB_TRANSPORT_IB",
+    "MCA_BTL_OPENIB_TRANSPORT_IWARP",
+    "MCA_BTL_OPENIB_TRANSPORT_RDMAOE",
+    "MCA_BTL_OPENIB_TRANSPORT_UNKNOWN"
+};
+
 static int mca_btl_openib_finalize_resources(struct mca_btl_base_module_t* btl);
 
-static void show_init_error(const char *file, int line,
-                            const char *func, const char *dev)
+void mca_btl_openib_show_init_error(const char *file, int line,
+                                    const char *func, const char *dev)
 {
     if (ENOMEM == errno) {
         int ret;
@@ -166,15 +176,16 @@ static int adjust_cq(mca_btl_openib_device_t *device, const int cq)
                 0);
 
         if (NULL == device->ib_cq[cq]) {
-            show_init_error(__FILE__, __LINE__, "ibv_create_cq",
-                        ibv_get_device_name(device->ib_dev));
+            mca_btl_openib_show_init_error(__FILE__, __LINE__, "ibv_create_cq",
+                                           ibv_get_device_name(device->ib_dev));
             return OMPI_ERROR;
         }
 
 #if OMPI_ENABLE_PROGRESS_THREADS == 1
         if(ibv_req_notify_cq(device->ib_cq[cq], 0)) {
-            show_init_error(__FILE__, __LINE__, "ibv_req_notify_cq",
-                            ibv_get_device_name(device->ib_dev));
+            mca_btl_openib_show_init_error(__FILE__, __LINE__,
+                                           "ibv_req_notify_cq",
+                                           ibv_get_device_name(device->ib_dev));
             return OMPI_ERROR;
         }
 
@@ -236,8 +247,9 @@ static int create_srq(mca_btl_openib_module_t *openib_btl)
                    ibv_create_srq(openib_btl->device->ib_pd, &attr);
             }
             if (NULL == openib_btl->qps[qp].u.srq_qp.srq) {
-                show_init_error(__FILE__, __LINE__, "ibv_create_srq",
-                                ibv_get_device_name(openib_btl->device->ib_dev));
+                mca_btl_openib_show_init_error(__FILE__, __LINE__,
+                                               "ibv_create_srq",
+                                               ibv_get_device_name(openib_btl->device->ib_dev));
                 return OMPI_ERROR;
             }
         }
@@ -285,6 +297,160 @@ out:
     return rc;
 }
 
+mca_btl_openib_transport_type_t mca_btl_openib_get_transport_type(mca_btl_openib_module_t* openib_btl)
+{
+/* If we have a driver with RDMAoE supporting as the device struct contains the same type (IB) for 
+   IBV_LINK_LAYER_INFINIBAND and IBV_LINK_LAYER_ETHERNET link layers and the single way
+   to detect this fact is to check their link_layer fields in a port_attr struct.
+   If our driver doesn't support this feature => the checking of transport type in device struct will be enough.
+   If the driver doesn't support completely transport types =>
+   our assumption that it is very old driver - that supports IB devices only */
+
+#ifdef HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE
+    switch(openib_btl->device->ib_dev->transport_type) {
+        case IBV_TRANSPORT_IB:
+#ifdef OMPI_HAVE_RDMAOE
+            switch(openib_btl->ib_port_attr.link_layer) {
+                case IBV_LINK_LAYER_ETHERNET:
+                    return MCA_BTL_OPENIB_TRANSPORT_RDMAOE;
+
+                case IBV_LINK_LAYER_INFINIBAND:
+                    return MCA_BTL_OPENIB_TRANSPORT_IB;
+            /* It is not possible that a device struct contains
+               IB transport and port was configured to IBV_LINK_LAYER_UNSPECIFIED */
+                case IBV_LINK_LAYER_UNSPECIFIED:
+                default:
+                    return MCA_BTL_OPENIB_TRANSPORT_UNKNOWN;
+            }
+#endif
+            return MCA_BTL_OPENIB_TRANSPORT_IB;
+
+        case IBV_TRANSPORT_IWARP:
+            return MCA_BTL_OPENIB_TRANSPORT_IWARP;
+
+        case IBV_TRANSPORT_UNKNOWN:		 
+        default:
+            return MCA_BTL_OPENIB_TRANSPORT_UNKNOWN;
+    }
+#else
+    return MCA_BTL_OPENIB_TRANSPORT_IB;
+#endif
+}
+
+static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl, 
+                                            mca_btl_base_endpoint_t* endpoint)
+{
+    int ret = OMPI_SUCCESS;
+
+    char* recv_qps = NULL;
+
+    ompi_btl_openib_ini_values_t values;
+
+    if(mca_btl_openib_get_transport_type(openib_btl) != endpoint->rem_info.rem_transport_type) {
+        orte_show_help("help-mpi-btl-openib.txt",
+                "conflicting transport types", true,
+                orte_process_info.nodename,
+                        ibv_get_device_name(openib_btl->device->ib_dev),
+                        (openib_btl->device->ib_dev_attr).vendor_id,
+                        (openib_btl->device->ib_dev_attr).vendor_part_id,
+                        mca_btl_openib_transport_name_strings[mca_btl_openib_get_transport_type(openib_btl)],
+                        endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                        endpoint->rem_info.rem_vendor_id,
+                        endpoint->rem_info.rem_vendor_part_id,
+                        mca_btl_openib_transport_name_strings[endpoint->rem_info.rem_transport_type]);
+    
+        return OMPI_ERROR;
+    }
+
+    memset(&values, 0, sizeof(ompi_btl_openib_ini_values_t));
+    ret = ompi_btl_openib_ini_query(endpoint->rem_info.rem_vendor_id,
+                          endpoint->rem_info.rem_vendor_part_id, &values);
+
+    if (OMPI_SUCCESS != ret && OMPI_ERR_NOT_FOUND != ret) {
+        orte_show_help("help-mpi-btl-openib.txt",
+                       "error in device init", true,
+                       orte_process_info.nodename,
+                       ibv_get_device_name(openib_btl->device->ib_dev));
+        return ret;
+    }
+
+    if(openib_btl->device->mtu < endpoint->rem_info.rem_mtu) {
+        endpoint->rem_info.rem_mtu = openib_btl->device->mtu; 
+    }
+
+    endpoint->use_eager_rdma = openib_btl->device->use_eager_rdma &
+                               endpoint->use_eager_rdma;
+
+    /* Receive queues checking */
+
+    /* In this check we assume that the command line or INI file parameters are the same
+       for all processes on all machines. The assumption is correct for 99.9999% of users,
+       if a user distributes different INI files or parameters for different node/procs,
+       it is on his own responsibility */
+    switch(mca_btl_openib_component.receive_queues_source) {
+        case BTL_OPENIB_RQ_SOURCE_MCA:
+        case BTL_OPENIB_RQ_SOURCE_MAX:
+            break;
+
+        /* If the queues configuration was set from command line 
+           (with --mca btl_openib_receive_queues parameter) => both sides have a same configuration */
+
+        /* In this case the local queues configuration was gotten from INI file =>
+           not possible that remote side got its queues configuration from command line => 
+           (by prio) the configuration was set from INI file or (if not configure)
+           by default queues configuration */
+        case BTL_OPENIB_RQ_SOURCE_DEVICE_INI:
+            if(NULL != values.receive_queues) {
+                recv_qps = values.receive_queues;
+            } else {
+                recv_qps = mca_btl_openib_component.default_recv_qps;
+            }
+
+            if(0 != strcmp(mca_btl_openib_component.receive_queues,
+                                                         recv_qps)) {
+                orte_show_help("help-mpi-btl-openib.txt",
+                               "unsupported queues configuration", true,
+                               orte_process_info.nodename,
+                               ibv_get_device_name(openib_btl->device->ib_dev),
+                               (openib_btl->device->ib_dev_attr).vendor_id,
+                               (openib_btl->device->ib_dev_attr).vendor_part_id,
+                               mca_btl_openib_component.receive_queues,
+                               endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                               endpoint->rem_info.rem_vendor_id,
+                               endpoint->rem_info.rem_vendor_part_id,
+                               recv_qps);
+
+                return OMPI_ERROR;
+            }
+            break;
+
+        /* If the local queues configuration was set 
+           by default queues => check all possible cases for remote side and compare */
+        case  BTL_OPENIB_RQ_SOURCE_DEFAULT:
+            if(NULL != values.receive_queues) {
+                if(0 != strcmp(mca_btl_openib_component.receive_queues,
+                                                values.receive_queues)) {
+                     orte_show_help("help-mpi-btl-openib.txt",
+                               "unsupported queues configuration", true,
+                               orte_process_info.nodename,
+                               ibv_get_device_name(openib_btl->device->ib_dev),
+                               (openib_btl->device->ib_dev_attr).vendor_id,
+                               (openib_btl->device->ib_dev_attr).vendor_part_id,
+                               mca_btl_openib_component.receive_queues,
+                               endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                               endpoint->rem_info.rem_vendor_id,
+                               endpoint->rem_info.rem_vendor_part_id,
+                               values.receive_queues);
+
+                    return OMPI_ERROR;
+                }
+            }
+            break;
+    }
+
+    return OMPI_SUCCESS;
+}
+
 /*
  *  add a proc to this btl module
  *    creates an endpoint that is setup on the
@@ -469,6 +635,12 @@ int mca_btl_openib_add_procs(
             continue;
         }
 
+         if(OMPI_SUCCESS != mca_btl_openib_tune_endpoint(openib_btl, endpoint)) {
+            OBJ_RELEASE(endpoint);
+            OPAL_THREAD_UNLOCK(&ib_proc->proc_lock);
+            return OMPI_ERROR;
+        }
+
         endpoint->index = opal_pointer_array_add(openib_btl->device->endpoints, (void*)endpoint);
         if( 0 > endpoint->index ) {
             OBJ_RELEASE(endpoint);
@@ -639,7 +811,7 @@ mca_btl_base_descriptor_t* mca_btl_openib_alloc(
        (flags & MCA_BTL_DES_FLAGS_BTL_OWNERSHIP)) {
         int prio = !(flags & MCA_BTL_DES_FLAGS_PRIORITY);
         sfrag = check_coalescing(&ep->qps[qp].no_wqe_pending_frags[prio],
-                &ep->qps[qp].qp->lock, ep, size);
+                &ep->endpoint_lock, ep, size);
 
         if(NULL == sfrag) {
             if(BTL_OPENIB_QP_TYPE_PP(qp)) {
@@ -1052,6 +1224,171 @@ int mca_btl_openib_finalize(struct mca_btl_base_module_t* btl)
     return rc;
 }
 
+/*
+ *  Send immediate - Minimum function calls minimum checks, send the data ASAP.
+ *  If BTL can't to send the messages imidiate, it creates messages descriptor 
+ *  returns it to PML.
+ */
+int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
+        struct mca_btl_base_endpoint_t* ep,
+        struct ompi_convertor_t* convertor,
+        void* header,
+        size_t header_size,
+        size_t payload_size,
+        uint8_t order,
+        uint32_t flags,
+        mca_btl_base_tag_t tag,
+        mca_btl_base_descriptor_t** descriptor) 
+{
+    mca_btl_openib_module_t *obtl = (mca_btl_openib_module_t*)btl;
+    size_t size = payload_size + header_size;
+    size_t eager_limit;
+    int rc,  
+        qp = frag_size_to_order(obtl, size), 
+        prio = !(flags & MCA_BTL_DES_FLAGS_PRIORITY), 
+        ib_rc;
+    int32_t cm_return;
+    bool do_rdma = false;
+    ompi_free_list_item_t* item = NULL;
+    mca_btl_openib_frag_t *frag;
+    mca_btl_openib_header_t *hdr;
+
+    OPAL_THREAD_LOCK(&ep->endpoint_lock);
+
+    if (OPAL_UNLIKELY(MCA_BTL_IB_CONNECTED != ep->endpoint_state)) {
+        goto cant_send;
+    }
+
+    /* If it is pending messages on the qp - we can not send */
+    if(OPAL_UNLIKELY(!opal_list_is_empty(&ep->qps[qp].no_wqe_pending_frags[prio]))) {
+        goto cant_send;
+    }
+
+    /* Allocate WQE */
+    if(OPAL_UNLIKELY(qp_get_wqe(ep, qp) < 0)) {
+        goto no_credits_or_wqe;
+    }
+
+    /* eager rdma or send ? Check eager rdma credits */
+    /* Note: Maybe we want to implement isend only for eager rdma ?*/
+    eager_limit = mca_btl_openib_component.eager_limit +
+        sizeof(mca_btl_openib_header_coalesced_t) +
+        sizeof(mca_btl_openib_control_header_t);
+
+    if(OPAL_LIKELY(size <= eager_limit)) {
+        if(acquire_eager_rdma_send_credit(ep) == OMPI_SUCCESS) {
+            do_rdma = true;
+        }
+    }
+
+    /* if(!do_rdma && acquire_send_credit(ep, frag) != OMPI_SUCCESS) { */
+    /* Check send credits if it is no rdma */
+    if(!do_rdma) {
+        if(BTL_OPENIB_QP_TYPE_PP(qp)) {
+            if(OPAL_UNLIKELY(OPAL_THREAD_ADD32(&ep->qps[qp].u.pp_qp.sd_credits, -1) < 0)){
+                OPAL_THREAD_ADD32(&ep->qps[qp].u.pp_qp.sd_credits, 1);
+                goto no_credits_or_wqe;
+            }
+        } else {
+            if(OPAL_UNLIKELY(OPAL_THREAD_ADD32(&obtl->qps[qp].u.srq_qp.sd_credits, -1) < 0)){
+                OPAL_THREAD_ADD32(&obtl->qps[qp].u.srq_qp.sd_credits, 1);
+                goto no_credits_or_wqe;
+            }
+        }
+    }
+
+    /* Allocate fragment */
+    OMPI_FREE_LIST_GET(&obtl->device->qps[qp].send_free, item, rc);
+    if(OPAL_UNLIKELY(NULL == item)) {
+        /* we don't return NULL because maybe later we will try to coalesce */
+        goto no_frags;
+    }
+    frag = to_base_frag(item);
+    hdr = to_send_frag(item)->hdr;
+    frag->segment.seg_len = size;
+    frag->base.order = qp;
+    frag->base.des_flags = flags;
+    hdr->tag = tag;
+    to_com_frag(item)->endpoint = ep;
+
+    /* put match header */
+    memcpy(frag->segment.seg_addr.pval, header, header_size);
+
+    /* Pack data */
+    if(payload_size) {
+        size_t max_data;
+        struct iovec iov;
+        uint32_t iov_count;
+        /* pack the data into the supplied buffer */
+        iov.iov_base = (IOVBASE_TYPE*)((unsigned char*)frag->segment.seg_addr.pval + header_size);
+        iov.iov_len  = max_data = payload_size;
+        iov_count    = 1;
+
+        (void)ompi_convertor_pack( convertor, &iov, &iov_count, &max_data);
+
+        assert(max_data == payload_size);
+    }
+
+    /* Set all credits */
+    BTL_OPENIB_GET_CREDITS(ep->eager_rdma_local.credits, hdr->credits);
+    if(hdr->credits)
+        hdr->credits |= BTL_OPENIB_RDMA_CREDITS_FLAG;
+
+    if(!do_rdma) {
+        if(BTL_OPENIB_QP_TYPE_PP(qp) && 0 == hdr->credits) {
+            BTL_OPENIB_GET_CREDITS(ep->qps[qp].u.pp_qp.rd_credits, hdr->credits);
+        }
+    } else {
+        hdr->credits |= (qp << 11);
+    }
+
+    BTL_OPENIB_GET_CREDITS(ep->qps[qp].u.pp_qp.cm_return, cm_return);
+    /* cm_seen is only 8 bytes, but cm_return is 32 bytes */
+    if(cm_return > 255) {
+        hdr->cm_seen = 255;
+        cm_return -= 255;
+        OPAL_THREAD_ADD32(&ep->qps[qp].u.pp_qp.cm_return, cm_return);
+    } else {
+        hdr->cm_seen = cm_return;
+    }
+
+    ib_rc = post_send(ep, to_send_frag(item), do_rdma);
+
+    if(!ib_rc) {
+        OPAL_THREAD_UNLOCK(&ep->endpoint_lock);
+        return OMPI_SUCCESS;
+    }
+
+    /* Failed to send, do clean up all allocated resources */
+    if(ep->nbo) {
+        BTL_OPENIB_HEADER_NTOH(*hdr);
+    }
+    if(BTL_OPENIB_IS_RDMA_CREDITS(hdr->credits)) {
+        OPAL_THREAD_ADD32(&ep->eager_rdma_local.credits,
+                BTL_OPENIB_CREDITS(hdr->credits));
+    }
+    if (!do_rdma && BTL_OPENIB_QP_TYPE_PP(qp)) {
+        OPAL_THREAD_ADD32(&ep->qps[qp].u.pp_qp.rd_credits,
+                hdr->credits);
+    }
+no_frags:
+    if(do_rdma) {
+        OPAL_THREAD_ADD32(&ep->eager_rdma_remote.tokens, 1);
+    } else {
+        if(BTL_OPENIB_QP_TYPE_PP(qp)) {
+            OPAL_THREAD_ADD32(&ep->qps[qp].u.pp_qp.sd_credits, 1);
+        } else if BTL_OPENIB_QP_TYPE_SRQ(qp){
+            OPAL_THREAD_ADD32(&obtl->qps[qp].u.srq_qp.sd_credits, 1);
+        }
+    }
+no_credits_or_wqe:
+    qp_put_wqe(ep, qp);
+cant_send:
+    OPAL_THREAD_UNLOCK(&ep->endpoint_lock);
+    /* We can not send the data directly, so we just return descriptor */
+    *descriptor = mca_btl_openib_alloc(btl, ep, order, size, flags);
+    return OMPI_ERR_RESOURCE_BUSY;
+}
 /*
  *  Initiate a send.
  */
@@ -1080,6 +1417,8 @@ int mca_btl_openib_send(
         frag->hdr->tag = tag;
     }
 
+    des->des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
+
     return mca_btl_openib_endpoint_send(ep, frag);
 }
 
@@ -1100,6 +1439,8 @@ int mca_btl_openib_put( mca_btl_base_module_t* btl,
     assert(openib_frag_type(frag) == MCA_BTL_OPENIB_FRAG_SEND_USER ||
             openib_frag_type(frag) == MCA_BTL_OPENIB_FRAG_SEND);
 
+    descriptor->des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
+
     if(ep->endpoint_state != MCA_BTL_IB_CONNECTED) {
         int rc;
         OPAL_THREAD_LOCK(&ep->endpoint_lock);
@@ -1169,6 +1510,8 @@ int mca_btl_openib_get(mca_btl_base_module_t* btl,
 
     assert(openib_frag_type(frag) == MCA_BTL_OPENIB_FRAG_RECV_USER);
 
+    descriptor->des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
+
     if(ep->endpoint_state != MCA_BTL_IB_CONNECTED) {
         int rc;
         OPAL_THREAD_LOCK(&ep->endpoint_lock);
