@@ -15,6 +15,7 @@
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
  * Copyright (c) 2006-2009 Mellanox Technologies, Inc.  All rights reserved.
+ * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -49,69 +50,6 @@
 static void mca_btl_openib_endpoint_construct(mca_btl_base_endpoint_t* endpoint);
 static void mca_btl_openib_endpoint_destruct(mca_btl_base_endpoint_t* endpoint);
 
-static int post_send(mca_btl_openib_endpoint_t *ep,
-        mca_btl_openib_send_frag_t *frag, const bool rdma)
-{
-    mca_btl_openib_module_t *openib_btl = ep->endpoint_btl;
-    mca_btl_base_segment_t *seg = &to_base_frag(frag)->segment;
-    struct ibv_sge *sg = &to_com_frag(frag)->sg_entry;
-    struct ibv_send_wr *sr_desc = &to_out_frag(frag)->sr_desc;
-    struct ibv_send_wr *bad_wr;
-    int qp = to_base_frag(frag)->base.order;
-
-    sg->length = seg->seg_len + sizeof(mca_btl_openib_header_t) +
-        (rdma ? sizeof(mca_btl_openib_footer_t) : 0) + frag->coalesced_length;
-
-    sr_desc->send_flags = ib_send_flags(sg->length, &(ep->qps[qp]));
-
-    if(ep->nbo)
-        BTL_OPENIB_HEADER_HTON(*frag->hdr);
-
-    if(rdma) {
-        int32_t head;
-        mca_btl_openib_footer_t* ftr =
-            (mca_btl_openib_footer_t*)(((char*)frag->hdr) + sg->length -
-                    sizeof(mca_btl_openib_footer_t));
-        sr_desc->opcode = IBV_WR_RDMA_WRITE;
-        MCA_BTL_OPENIB_RDMA_FRAG_SET_SIZE(ftr, sg->length);
-        MCA_BTL_OPENIB_RDMA_MAKE_LOCAL(ftr);
-#if OMPI_ENABLE_DEBUG
-        ftr->seq = ep->eager_rdma_remote.seq++;
-#endif
-        if(ep->nbo)
-            BTL_OPENIB_FOOTER_HTON(*ftr);
-
-        sr_desc->wr.rdma.rkey = ep->eager_rdma_remote.rkey;
-        MCA_BTL_OPENIB_RDMA_MOVE_INDEX(ep->eager_rdma_remote.head, head);
-        sr_desc->wr.rdma.remote_addr =
-            ep->eager_rdma_remote.base.lval +
-            head * openib_btl->eager_rdma_frag_size +
-            sizeof(mca_btl_openib_header_t) +
-            mca_btl_openib_component.eager_limit +
-            sizeof(mca_btl_openib_footer_t);
-        sr_desc->wr.rdma.remote_addr -= sg->length;
-    } else {
-        if(BTL_OPENIB_QP_TYPE_PP(qp)) {
-            sr_desc->opcode = IBV_WR_SEND;
-        } else {
-            sr_desc->opcode = IBV_WR_SEND_WITH_IMM;
-#if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
-            sr_desc->imm_data = htonl(ep->rem_info.rem_index);
-#else
-            sr_desc->imm_data = ep->rem_info.rem_index;
-#endif
-        }
-    }
-
-#if HAVE_XRC
-    if(BTL_OPENIB_QP_TYPE_XRC(qp))
-        sr_desc->xrc_remote_srq_num = ep->rem_info.rem_srqs[qp].rem_srq_num;
-#endif
-    assert(sg->addr == (uint64_t)(uintptr_t)frag->hdr);
-
-    return ibv_post_send(ep->qps[qp].qp->lcl_qp, sr_desc, &bad_wr);
- }
-
 static inline int acruire_wqe(mca_btl_openib_endpoint_t *ep,
         mca_btl_openib_send_frag_t *frag)
 {
@@ -128,17 +66,6 @@ static inline int acruire_wqe(mca_btl_openib_endpoint_t *ep,
     return OMPI_SUCCESS;
 }
 
-static inline int
-acquire_eager_rdma_send_credit(mca_btl_openib_endpoint_t *endpoint)
-{
-    if(OPAL_THREAD_ADD32(&endpoint->eager_rdma_remote.tokens, -1) < 0) {
-        OPAL_THREAD_ADD32(&endpoint->eager_rdma_remote.tokens, 1);
-        return OMPI_ERR_OUT_OF_RESOURCE;
-    }
-
-    return OMPI_SUCCESS;
-}
-
 static int acquire_send_credit(mca_btl_openib_endpoint_t *endpoint,
         mca_btl_openib_send_frag_t *frag)
 {
@@ -168,11 +95,6 @@ static int acquire_send_credit(mca_btl_openib_endpoint_t *endpoint,
     return OMPI_SUCCESS;
 }
 
-#define GET_CREDITS(FROM, TO)                                        \
-            do {                                                     \
-                TO = FROM;                                           \
-            } while(0 == OPAL_ATOMIC_CMPSET_32(&FROM, TO, 0))
-
 /* this function is called with endpoint->endpoint_lock held */
 int mca_btl_openib_endpoint_post_send(mca_btl_openib_endpoint_t *endpoint,
         mca_btl_openib_send_frag_t *frag)
@@ -207,19 +129,19 @@ int mca_btl_openib_endpoint_post_send(mca_btl_openib_endpoint_t *endpoint,
         return OMPI_ERR_RESOURCE_BUSY;
     }
 
-    GET_CREDITS(endpoint->eager_rdma_local.credits, hdr->credits);
+    BTL_OPENIB_GET_CREDITS(endpoint->eager_rdma_local.credits, hdr->credits);
     if(hdr->credits)
         hdr->credits |= BTL_OPENIB_RDMA_CREDITS_FLAG;
 
     if(!do_rdma) {
         if(BTL_OPENIB_QP_TYPE_PP(qp) && 0 == hdr->credits) {
-            GET_CREDITS(endpoint->qps[qp].u.pp_qp.rd_credits, hdr->credits);
+            BTL_OPENIB_GET_CREDITS(endpoint->qps[qp].u.pp_qp.rd_credits, hdr->credits);
         }
     } else {
         hdr->credits |= (qp << 11);
     }
 
-    GET_CREDITS(endpoint->qps[qp].u.pp_qp.cm_return, cm_return);
+    BTL_OPENIB_GET_CREDITS(endpoint->qps[qp].u.pp_qp.cm_return, cm_return);
     /* cm_seen is only 8 bytes, but cm_return is 32 bytes */
     if(cm_return > 255) {
         hdr->cm_seen = 255;
@@ -391,6 +313,11 @@ void mca_btl_openib_endpoint_init(mca_btl_openib_module_t *btl,
                 ep->rem_info.rem_subnet_id,
                 ep->rem_info.rem_mtu);
 
+    ep->rem_info.rem_vendor_id = (remote_proc_info->pm_port_info).vendor_id;
+    ep->rem_info.rem_vendor_part_id = (remote_proc_info->pm_port_info).vendor_part_id;
+
+    ep->rem_info.rem_transport_type = (remote_proc_info->pm_port_info).transport_type;
+
     for (qp = 0; qp < mca_btl_openib_component.num_qps; qp++) {
         endpoint_init_qp(ep, qp);
     }
@@ -607,7 +534,7 @@ void mca_btl_openib_endpoint_send_cts(mca_btl_openib_endpoint_t *endpoint)
 
     base_des->des_cbfunc = cts_sent;
     base_des->des_cbdata = NULL;
-    base_des->des_flags |= MCA_BTL_DES_FLAGS_PRIORITY;
+    base_des->des_flags |= MCA_BTL_DES_FLAGS_PRIORITY|MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
     base_des->order = mca_btl_openib_component.credits_qp;
     openib_frag->segment.seg_len = sizeof(mca_btl_openib_control_header_t);
     com_frag->endpoint = endpoint;
@@ -637,6 +564,7 @@ void mca_btl_openib_endpoint_cpc_complete(mca_btl_openib_endpoint_t *endpoint)
 {
     /* If the CPC uses the CTS protocol, then start it up */
     if (endpoint->endpoint_local_cpc->cbm_uses_cts) {
+        int transport_type_ib_p = 0;
         /* Post our receives, which will make credit management happy
            (i.e., rd_credits will be 0) */
         if (OMPI_SUCCESS != mca_btl_openib_endpoint_post_recvs(endpoint)) {
@@ -652,14 +580,15 @@ void mca_btl_openib_endpoint_cpc_complete(mca_btl_openib_endpoint_t *endpoint)
            receives this side's CTS).  Also send the CTS if we already
            received the peer's CTS (e.g., if this process was slow to
            call cpc_complete(). */
+#if defined(HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE)
+        transport_type_ib_p = (IBV_TRANSPORT_IB == endpoint->endpoint_btl->device->ib_dev->transport_type);
+#endif
         OPAL_OUTPUT((-1, "cpc_complete to peer %s: is IB %d, initiatior %d, cts received: %d",
                      endpoint->endpoint_proc->proc_ompi->proc_hostname,
-                     (IBV_TRANSPORT_IB == 
-                      endpoint->endpoint_btl->device->ib_dev->transport_type),
+                     transport_type_ib_p,
                      endpoint->endpoint_initiator,
                      endpoint->endpoint_cts_received));
-        if (IBV_TRANSPORT_IB == 
-            endpoint->endpoint_btl->device->ib_dev->transport_type ||
+        if (transport_type_ib_p ||
             endpoint->endpoint_initiator ||
             endpoint->endpoint_cts_received) {
             mca_btl_openib_endpoint_send_cts(endpoint);
@@ -729,21 +658,6 @@ void mca_btl_openib_endpoint_connected(mca_btl_openib_endpoint_t *endpoint)
        event trigger. */
     opal_progress_event_users_decrement();
 
-    /* While there are frags in the list, process them */
-    while (!opal_list_is_empty(&(endpoint->pending_lazy_frags))) {
-        frag_item = opal_list_remove_first(&(endpoint->pending_lazy_frags));
-        frag = to_send_frag(frag_item);
-        /* We need to post this one */
-
-        if(OMPI_ERROR == mca_btl_openib_endpoint_post_send(endpoint, frag))
-            BTL_ERROR(("Error posting send"));
-    }
-
-    /* if upper layer called put or get before connection moved to connected
-     * state then we restart them here */
-    mca_btl_openib_frag_progress_pending_put_get(endpoint,
-            mca_btl_openib_component.rdma_qp);
-
     if(MCA_BTL_XRC_ENABLED) {
         while(master && !opal_list_is_empty(&endpoint->ib_addr->pending_ep)) {
             ep_item = opal_list_remove_first(&endpoint->ib_addr->pending_ep);
@@ -756,6 +670,25 @@ void mca_btl_openib_endpoint_connected(mca_btl_openib_endpoint_t *endpoint)
         }
         OPAL_THREAD_UNLOCK(&endpoint->ib_addr->addr_lock);
     }
+
+
+    /* Process pending packet on the endpoint */
+
+    /* While there are frags in the list, process them */
+    while (!opal_list_is_empty(&(endpoint->pending_lazy_frags))) {
+        frag_item = opal_list_remove_first(&(endpoint->pending_lazy_frags));
+        frag = to_send_frag(frag_item);
+        /* We need to post this one */
+
+        if(OMPI_ERROR == mca_btl_openib_endpoint_post_send(endpoint, frag))
+            BTL_ERROR(("Error posting send"));
+    }
+    OPAL_THREAD_UNLOCK(&endpoint->endpoint_lock);
+
+    /* if upper layer called put or get before connection moved to connected
+     * state then we restart them here */
+    mca_btl_openib_frag_progress_pending_put_get(endpoint,
+            mca_btl_openib_component.rdma_qp);
 }
 
 /*
@@ -836,6 +769,7 @@ void mca_btl_openib_endpoint_send_credits(mca_btl_openib_endpoint_t* endpoint,
         to_base_frag(frag)->base.order = mca_btl_openib_component.credits_qp;
         to_base_frag(frag)->base.des_cbfunc = mca_btl_openib_endpoint_credits;
         to_base_frag(frag)->base.des_cbdata = NULL;
+        to_base_frag(frag)->base.des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;;
         to_com_frag(frag)->endpoint = endpoint;
         frag->hdr->tag = MCA_BTL_TAG_BTL;
         to_base_frag(frag)->segment.seg_len =
@@ -856,10 +790,10 @@ void mca_btl_openib_endpoint_send_credits(mca_btl_openib_endpoint_t* endpoint,
         }
      }
 
-    GET_CREDITS(endpoint->qps[qp].u.pp_qp.rd_credits, frag->hdr->credits);
+    BTL_OPENIB_GET_CREDITS(endpoint->qps[qp].u.pp_qp.rd_credits, frag->hdr->credits);
 
     frag->hdr->cm_seen = 0;
-    GET_CREDITS(endpoint->qps[qp].u.pp_qp.cm_return, cm_return);
+    BTL_OPENIB_GET_CREDITS(endpoint->qps[qp].u.pp_qp.cm_return, cm_return);
     if(cm_return > 255) {
         frag->hdr->cm_seen = 255;
         cm_return -= 255;
@@ -868,7 +802,7 @@ void mca_btl_openib_endpoint_send_credits(mca_btl_openib_endpoint_t* endpoint,
         frag->hdr->cm_seen = cm_return;
     }
 
-    GET_CREDITS(endpoint->eager_rdma_local.credits, credits_hdr->rdma_credits);
+    BTL_OPENIB_GET_CREDITS(endpoint->eager_rdma_local.credits, credits_hdr->rdma_credits);
     credits_hdr->qpn = qp;
     credits_hdr->control.type = MCA_BTL_OPENIB_CONTROL_CREDITS;
 
@@ -926,7 +860,7 @@ static int mca_btl_openib_endpoint_send_eager_rdma(
     to_base_frag(frag)->base.des_cbfunc =
         mca_btl_openib_endpoint_eager_rdma_connect_cb;
     to_base_frag(frag)->base.des_cbdata = NULL;
-    to_base_frag(frag)->base.des_flags |= MCA_BTL_DES_FLAGS_PRIORITY;
+    to_base_frag(frag)->base.des_flags |= MCA_BTL_DES_FLAGS_PRIORITY|MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
     to_base_frag(frag)->base.order = mca_btl_openib_component.credits_qp;
     to_base_frag(frag)->segment.seg_len =
         sizeof(mca_btl_openib_eager_rdma_header_t);
