@@ -187,8 +187,8 @@ static int ipaddr_specified(struct sockaddr_in *ipaddr, uint32_t netmask)
             char **temp = opal_argv_split(list[i], '/');
 
             inet_pton(ipaddr->sin_family, temp[0], &ipae);
-            list_subnet = ipae.s_addr & ~(~0 << atoi(temp[1]));
-            subnet = ipaddr->sin_addr.s_addr & ~(~0 << netmask);
+            list_subnet = ntohl(ipae.s_addr) & ~(~0 >> atoi(temp[1]));
+            subnet = ntohl(ipaddr->sin_addr.s_addr) & ~(~0 >> netmask);
 
             if (subnet == list_subnet) { 
                 return 0;
@@ -209,8 +209,8 @@ static int ipaddr_specified(struct sockaddr_in *ipaddr, uint32_t netmask)
             char **temp = opal_argv_split(list[i], '/');
 
             inet_pton(ipaddr->sin_family, temp[0], &ipae);
-            list_subnet = ipae.s_addr & ~(~0 << atoi(temp[1]));
-            subnet = ipaddr->sin_addr.s_addr & ~(~0 << netmask);
+            list_subnet = ntohl(ipae.s_addr) & ~(~0 >> atoi(temp[1]));
+            subnet = ntohl(ipaddr->sin_addr.s_addr) & ~(~0 >> netmask);
 
             if (subnet == list_subnet) { 
                 return 1;
@@ -229,6 +229,13 @@ static int add_rdma_addr(struct sockaddr *ipaddr, uint32_t netmask)
     int rc = OMPI_SUCCESS;
     struct rdma_addr_list *myaddr;
 
+    sinp = (struct sockaddr_in *)ipaddr;
+
+    if ((sinp->sin_addr.s_addr & htonl(0xff000000)) == htonl(0x7f000000)) {
+        rc = OMPI_SUCCESS;
+        goto out1;
+    }
+
     ch = rdma_create_event_channel();
     if (NULL == ch) {
         BTL_VERBOSE(("failed creating RDMA CM event channel"));
@@ -272,9 +279,8 @@ static int add_rdma_addr(struct sockaddr *ipaddr, uint32_t netmask)
         goto out3;
     }
 
-    sinp = (struct sockaddr_in *)ipaddr;
     myaddr->addr = sinp->sin_addr.s_addr;
-    myaddr->subnet = myaddr->addr & ~(~0 << netmask);
+    myaddr->subnet = ntohl(myaddr->addr) & ~(~0 >> netmask);
     inet_ntop(sinp->sin_family, &sinp->sin_addr, 
               myaddr->addr_str, sizeof(myaddr->addr_str));
     memcpy(myaddr->dev_name, cm_id->verbs->device->name, IBV_SYSFS_NAME_MAX);
