@@ -293,7 +293,7 @@ int ompi_btl_openib_connect_base_select_for_local_port(mca_btl_openib_module_t *
                        "no cpcs for port", true,
                        orte_process_info.nodename,
                        ibv_get_device_name(btl->device->ib_dev),
-                       msg);
+                       btl->port_num, msg);
         free(cpcs);
         free(msg);
         return OMPI_ERR_NOT_SUPPORTED;
