@@ -46,6 +46,7 @@
 
 #if OPAL_ENABLE_FT    == 1
 #include "opal/mca/crs/base/base.h"
+#include "opal/util/basename.h"
 #include "ompi/runtime/ompi_cr.h"
 #endif
 
@@ -127,13 +128,14 @@ static void init_maffinity(int *my_mem_node, int *max_mem_node)
     *my_mem_node = 0;
     *max_mem_node = 1;
 
-    if(opal_carto_base_get_host_graph(&topo, "Memory") != OMPI_SUCCESS)
+    if (OMPI_SUCCESS != opal_carto_base_get_host_graph(&topo, "Memory")) {
         return;
+    }
 
      OBJ_CONSTRUCT(&dists, opal_value_array_t);
      opal_value_array_init(&dists, sizeof(opal_carto_node_distance_t));
 
-    if(opal_paffinity_base_get_processor_info(&num_core) != OMPI_SUCCESS)  {
+    if (OMPI_SUCCESS != opal_paffinity_base_get_processor_info(&num_core))  {
         num_core = 100;  /* set something large */
     }
 
@@ -141,9 +143,11 @@ static void init_maffinity(int *my_mem_node, int *max_mem_node)
      opal_paffinity_base_get(&cpus);
 
      /* find core we are running on */
-     for(i = 0; i < num_core; i++)
-         if(OPAL_PAFFINITY_CPU_ISSET(i, cpus))
+     for (i = 0; i < num_core; i++) {
+         if (OPAL_PAFFINITY_CPU_ISSET(i, cpus)) {
              break;
+         }
+     }
 
     if (OMPI_SUCCESS != opal_paffinity_base_get_map_to_socket_core(i, &socket, &i)) {
         /* no topology info available */
@@ -154,17 +158,21 @@ static void init_maffinity(int *my_mem_node, int *max_mem_node)
 
      slot_node = opal_carto_base_find_node(topo, myslot);
 
-     if(NULL == slot_node)
+     if(NULL == slot_node) {
          goto out;
+     }
 
      opal_carto_base_get_nodes_distance(topo, slot_node, "Memory", &dists);
-     if((*max_mem_node = opal_value_array_get_size(&dists)) < 2)
+     if((*max_mem_node = opal_value_array_get_size(&dists)) < 2) {
          goto out;
+     }
 
-     dist = opal_value_array_get_item(&dists, 0);
+     dist = (opal_carto_node_distance_t *) opal_value_array_get_item(&dists, 0);
      opal_maffinity_base_node_name_to_id(dist->node->node_name, my_mem_node);
 out:
-     if(myslot) free(myslot);
+     if (myslot) {
+         free(myslot);
+     }
      OBJ_DESTRUCT(&dists);
      opal_carto_base_free_graph(topo);
 }
@@ -173,15 +181,17 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
 {
     size_t size, length, length_payload;
     char *sm_ctl_file;
-    sm_fifo_t *my_fifos;
+    mca_btl_sm_fifo_t *my_fifos;
     int my_mem_node=-1, num_mem_nodes=-1, i;
+    ompi_proc_t **procs;
+    size_t num_procs;
 
     init_maffinity(&my_mem_node, &num_mem_nodes);
     mca_btl_sm_component.mem_node = my_mem_node;
     mca_btl_sm_component.num_mem_nodes = num_mem_nodes;
 
     /* lookup shared memory pool */
-    mca_btl_sm_component.sm_mpools = calloc(num_mem_nodes,
+    mca_btl_sm_component.sm_mpools = (mca_mpool_base_module_t **) calloc(num_mem_nodes,
                                             sizeof(mca_mpool_base_module_t*));
 
     /* create mpool for each memory node */
@@ -196,7 +206,7 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
         /*
          * This heuristic formula mostly says that we request memory for:
          * - nfifos FIFOs, each comprising:
-         *   . a sm_fifo_t structure
+         *   . an mca_btl_sm_fifo_t structure
          *   . many pointers (fifo_size of them per FIFO)
          * - eager fragments (2*n of them, allocated in sm_free_list_inc chunks)
          * - max fragments (sm_free_list_num of them)
@@ -205,7 +215,7 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
          * additions to account for some padding and edge effects that may lie
          * in the allocator.
          */
-        res.size = m->nfifos * ( sizeof(sm_fifo_t) + sizeof(void *) * m->fifo_size + 4 * CACHE_LINE_SIZE )
+        res.size = m->nfifos * ( sizeof(mca_btl_sm_fifo_t) + sizeof(void *) * m->fifo_size + 4 * CACHE_LINE_SIZE )
             + ( 2 * n + m->sm_free_list_inc ) * ( m->eager_limit   + 2 * CACHE_LINE_SIZE )
             +           m->sm_free_list_num   * ( m->max_frag_size + 2 * CACHE_LINE_SIZE );
         if ( ((double) res.size) * n > LONG_MAX )
@@ -252,18 +262,20 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
     /* Pass in a data segment alignment of 0 to get no data
        segment (only the shared control structure) */
     size = sizeof(mca_common_sm_file_header_t) +
-        n * (sizeof(sm_fifo_t*) + sizeof(char *) + sizeof(uint16_t)) + CACHE_LINE_SIZE;
-    if(!(mca_btl_sm_component.mmap_file =
-         mca_common_sm_mmap_init(size, sm_ctl_file,
-                                 sizeof(mca_common_sm_file_header_t),
-                                 CACHE_LINE_SIZE))) {
+        n * (sizeof(mca_btl_sm_fifo_t*) + sizeof(char *) + sizeof(uint16_t)) + CACHE_LINE_SIZE;
+    procs = ompi_proc_world(&num_procs);
+    if (!(mca_btl_sm_component.mmap_file =
+          mca_common_sm_mmap_init(procs, num_procs, size, sm_ctl_file,
+                                  sizeof(mca_common_sm_file_header_t),
+                                  CACHE_LINE_SIZE))) {
         opal_output(0, "mca_btl_sm_add_procs: unable to create shared memory "
                     "BTL coordinating strucure :: size %lu \n",
                     (unsigned long)size);
+        free(procs);
         free(sm_ctl_file);
         return OMPI_ERROR;
     }
-
+    free(procs);
     free(sm_ctl_file);
 
     /* set the pointer to the shared memory control structure */
@@ -279,7 +291,7 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
         return OMPI_ERROR;
     }
 
-    mca_btl_sm_component.shm_fifo = (sm_fifo_t **)mca_btl_sm_component.mmap_file->data_addr;
+    mca_btl_sm_component.shm_fifo = (mca_btl_sm_fifo_t **)mca_btl_sm_component.mmap_file->data_addr;
     mca_btl_sm_component.shm_bases = (char**)(mca_btl_sm_component.shm_fifo + n);
     mca_btl_sm_component.shm_mem_nodes = (uint16_t*)(mca_btl_sm_component.shm_bases + n);
 
@@ -304,7 +316,7 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
         (uint16_t)my_mem_node;
 
     /* initialize the array of fifo's "owned" by this process */
-    if(NULL == (my_fifos = (sm_fifo_t*)mpool_calloc(FIFO_MAP_NUM(n), sizeof(sm_fifo_t))))
+    if(NULL == (my_fifos = (mca_btl_sm_fifo_t*)mpool_calloc(FIFO_MAP_NUM(n), sizeof(mca_btl_sm_fifo_t))))
         return OMPI_ERR_OUT_OF_RESOURCE;
 
     mca_btl_sm_component.shm_fifo[mca_btl_sm_component.my_smp_rank] = my_fifos;
@@ -313,14 +325,14 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
 
     /* cache the pointer to the 2d fifo array.  These addresses
      * are valid in the current process space */
-    mca_btl_sm_component.fifo = (sm_fifo_t**)malloc(sizeof(sm_fifo_t*) * n);
+    mca_btl_sm_component.fifo = (mca_btl_sm_fifo_t**)malloc(sizeof(mca_btl_sm_fifo_t*) * n);
 
     if(NULL == mca_btl_sm_component.fifo)
         return OMPI_ERR_OUT_OF_RESOURCE;
 
     mca_btl_sm_component.fifo[mca_btl_sm_component.my_smp_rank] = my_fifos;
 
-    mca_btl_sm_component.mem_nodes = malloc(sizeof(uint16_t) * n);
+    mca_btl_sm_component.mem_nodes = (uint16_t *) malloc(sizeof(uint16_t) * n);
     if(NULL == mca_btl_sm_component.mem_nodes)
         return OMPI_ERR_OUT_OF_RESOURCE;
 
@@ -353,8 +365,11 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
     if ( OMPI_SUCCESS != i )
         return i;
 
+    mca_btl_sm_component.num_outstanding_frags = 0;
+
+    mca_btl_sm_component.num_pending_sends = 0;
     i = opal_free_list_init(&mca_btl_sm_component.pending_send_fl,
-                            sizeof(btl_sm_pending_send_item_t),
+                            sizeof(mca_btl_sm_pending_send_item_t),
                             OBJ_CLASS(opal_free_list_item_t),
                             16, -1, 32);
     if ( OMPI_SUCCESS != i )
@@ -533,7 +548,7 @@ int mca_btl_sm_add_procs(
 
         /* store local address of remote fifos */
         mca_btl_sm_component.fifo[j] =
-            (sm_fifo_t*)OFFSET2ADDR(diff, mca_btl_sm_component.shm_fifo[j]);
+            (mca_btl_sm_fifo_t*)OFFSET2ADDR(diff, mca_btl_sm_component.shm_fifo[j]);
 
         /* cache local copy of peer memory node number */
         mca_btl_sm_component.mem_nodes[j] = mca_btl_sm_component.shm_mem_nodes[j];
@@ -740,6 +755,11 @@ int mca_btl_sm_sendi( struct mca_btl_base_module_t* btl,
     mca_btl_sm_frag_t* frag;
     int rc;
 
+    if ( mca_btl_sm_component.num_outstanding_frags * 2 > (int) mca_btl_sm_component.fifo_size ) {
+        mca_btl_sm_component_progress();
+    }
+
+    /* this check should be unnecessary... turn into an assertion? */
     if( length < mca_btl_sm_component.eager_limit ) {
         MCA_BTL_SM_FRAG_ALLOC_EAGER(frag, rc);
         if( OPAL_UNLIKELY(NULL == frag) ) {
@@ -772,8 +792,9 @@ int mca_btl_sm_sendi( struct mca_btl_base_module_t* btl,
          * post the descriptor in the queue - post with the relative
          * address
          */
+        OPAL_THREAD_ADD32(&mca_btl_sm_component.num_outstanding_frags, +1);
         MCA_BTL_SM_FIFO_WRITE(endpoint, endpoint->my_smp_rank,
-                              endpoint->peer_smp_rank, (void *) VIRTUAL2RELATIVE(frag->hdr), false, rc);
+                              endpoint->peer_smp_rank, (void *) VIRTUAL2RELATIVE(frag->hdr), false, true, rc);
         return rc;
     }
     *descriptor = mca_btl_sm_alloc( btl, endpoint, order,
@@ -795,6 +816,10 @@ int mca_btl_sm_send( struct mca_btl_base_module_t* btl,
     mca_btl_sm_frag_t* frag = (mca_btl_sm_frag_t*)descriptor;
     int rc;
 
+    if ( mca_btl_sm_component.num_outstanding_frags * 2 > (int) mca_btl_sm_component.fifo_size ) {
+        mca_btl_sm_component_progress();
+    }
+
     /* available header space */
     frag->hdr->len = frag->segment.seg_len;
     /* type of message, pt-2-pt, one-sided, etc */
@@ -808,8 +833,9 @@ int mca_btl_sm_send( struct mca_btl_base_module_t* btl,
      * post the descriptor in the queue - post with the relative
      * address
      */
+    OPAL_THREAD_ADD32(&mca_btl_sm_component.num_outstanding_frags, +1);
     MCA_BTL_SM_FIFO_WRITE(endpoint, endpoint->my_smp_rank,
-                          endpoint->peer_smp_rank, (void *) VIRTUAL2RELATIVE(frag->hdr), false, rc);
+                          endpoint->peer_smp_rank, (void *) VIRTUAL2RELATIVE(frag->hdr), false, true, rc);
     if( OPAL_LIKELY(0 == rc) ) {
         return 1;  /* the data is completely gone */
     }
@@ -826,6 +852,8 @@ int mca_btl_sm_ft_event(int state) {
 }
 #else
 int mca_btl_sm_ft_event(int state) {
+    char * tmp_dir = NULL;
+
     /* Notify mpool */
     if( NULL != mca_btl_sm_component.sm_mpool &&
         NULL != mca_btl_sm_component.sm_mpool->mpool_ft_event) {
@@ -863,6 +891,12 @@ int mca_btl_sm_ft_event(int state) {
         if( NULL != mca_btl_sm_component.mmap_file ) {
             /* Add session directory */
             opal_crs_base_cleanup_append(orte_process_info.job_session_dir, true);
+            tmp_dir = opal_dirname(orte_process_info.job_session_dir);
+            if( NULL != tmp_dir ) {
+                opal_crs_base_cleanup_append(tmp_dir, true);
+                free(tmp_dir);
+                tmp_dir = NULL;
+            }
             /* Add shared memory file */
             opal_crs_base_cleanup_append(mca_btl_sm_component.mmap_file->map_path, false);
         }
