@@ -5,23 +5,30 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "ompi_config.h"
 #include "btl_sm_frag.h"
 
 
-static inline void mca_btl_sm_frag_constructor(mca_btl_sm_frag_t* frag)
+static inline void mca_btl_sm_frag_common_constructor(mca_btl_sm_frag_t* frag)
 {
-    frag->segment.seg_addr.pval = frag+1;
+    frag->hdr = (mca_btl_sm_hdr_t*)frag->base.super.ptr;
+    if(frag->hdr != NULL) {
+        frag->hdr->frag = (mca_btl_sm_frag_t*)((uintptr_t)frag |
+            MCA_BTL_SM_FRAG_ACK);
+        frag->segment.seg_addr.pval = ((char*)frag->hdr) +
+            sizeof(mca_btl_sm_hdr_t);
+        frag->hdr->my_smp_rank = mca_btl_sm_component.my_smp_rank;
+    }
     frag->segment.seg_len = frag->size;
     frag->base.des_src = &frag->segment;
     frag->base.des_src_cnt = 1;
@@ -33,15 +40,15 @@ static inline void mca_btl_sm_frag_constructor(mca_btl_sm_frag_t* frag)
 static void mca_btl_sm_frag1_constructor(mca_btl_sm_frag_t* frag)
 {
     frag->size = mca_btl_sm_component.eager_limit;
-    frag->my_list = &mca_btl_sm_component.sm_frags1;
-    mca_btl_sm_frag_constructor(frag);
+    frag->my_list = &mca_btl_sm_component.sm_frags_eager;
+    mca_btl_sm_frag_common_constructor(frag);
 }
 
 static void mca_btl_sm_frag2_constructor(mca_btl_sm_frag_t* frag)
 {
     frag->size = mca_btl_sm_component.max_frag_size;
-    frag->my_list = &mca_btl_sm_component.sm_frags2;
-    mca_btl_sm_frag_constructor(frag);
+    frag->my_list = &mca_btl_sm_component.sm_frags_max;
+    mca_btl_sm_frag_common_constructor(frag);
 }
 
 static void mca_btl_sm_frag_destructor(mca_btl_sm_frag_t* frag)
@@ -49,12 +56,6 @@ static void mca_btl_sm_frag_destructor(mca_btl_sm_frag_t* frag)
 }
 
 
-OBJ_CLASS_INSTANCE(
-    mca_btl_sm_frag_t,
-    mca_btl_base_descriptor_t,
-    mca_btl_sm_frag_constructor,
-    mca_btl_sm_frag_destructor);
-
 OBJ_CLASS_INSTANCE(
     mca_btl_sm_frag1_t,
     mca_btl_base_descriptor_t,
