@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2006 The University of Tennessee and The University
@@ -36,7 +36,7 @@
 
 /* Open MPI includes */
 #include "opal/event/event.h"
-#include "opal/util/output.h"
+#include "orte/util/show_help.h"
 #include "ompi/class/ompi_bitmap.h"
 #include "ompi/class/ompi_free_list.h"
 #include "ompi/mca/pml/pml.h"
@@ -53,12 +53,14 @@ extern "C" {
 
 
 /**
- * Infiniband (TCP) BTL component.
+ * TCP BTL component.
  */
 
 struct mca_btl_tcp_component_t {
-    mca_btl_base_component_1_0_1_t super;   /**< base BTL component */ 
+    mca_btl_base_component_2_0_0_t super;   /**< base BTL component */ 
+    uint32_t tcp_addr_count;                /**< total number of addresses */
     uint32_t tcp_num_btls;                  /**< number of hcas available to the TCP component */
+    uint32_t tcp_num_links;                 /**< number of logical links per physical device */
     struct mca_btl_tcp_module_t **tcp_btls; /**< array of available BTL modules */
     struct mca_btl_tcp_proc_t* tcp_local;   /**< local proc struct */
     int tcp_free_list_num;                  /**< initial size of free lists */
@@ -68,18 +70,34 @@ struct mca_btl_tcp_component_t {
     opal_hash_table_t tcp_procs;            /**< hash table of tcp proc structures */
     opal_list_t tcp_events;                 /**< list of pending tcp events */
     opal_mutex_t tcp_lock;                  /**< lock for accessing module state */
-    opal_event_t tcp_recv_event;            /**< recv event for listen socket */
-    int tcp_listen_sd;                      /**< listen socket for incoming connection requests */
-    unsigned short tcp_listen_port;         /**< listen port */
+
+    opal_event_t tcp_recv_event;            /**< recv event for IPv4 listen socket */
+    int tcp_listen_sd;                      /**< IPv4 listen socket for incoming connection requests */
+    unsigned short tcp_listen_port;         /**< IPv4 listen port */
+    int32_t tcp_port_min;                   /**< IPv4 minimum port */
+    int32_t tcp_port_range;                 /**< IPv4 port range */
+#if OPAL_WANT_IPV6
+    opal_event_t tcp6_recv_event;           /**< recv event for IPv6 listen socket */
+    int tcp6_listen_sd;                     /**< IPv6 listen socket for incoming connection requests */
+    unsigned short tcp6_listen_port;        /**< IPv6 listen port */
+    int32_t tcp6_port_min;                  /**< IPv4 minimum port */
+    int32_t tcp6_port_range;                /**< IPv4 port range */
+#endif
+    /* Port range restriction */
+
     char*  tcp_if_include;                  /**< comma seperated list of interface to include */
     char*  tcp_if_exclude;                  /**< comma seperated list of interface to exclude */
     int    tcp_sndbuf;                      /**< socket sndbuf size */
     int    tcp_rcvbuf;                      /**< socket rcvbuf size */
+    int    tcp_disable_family;              /**< disabled AF_family */
 
     /* free list of fragment descriptors */
     ompi_free_list_t tcp_frag_eager;
     ompi_free_list_t tcp_frag_max;
     ompi_free_list_t tcp_frag_user;
+
+    /* Do we want to use TCP_NODELAY? */
+    int    tcp_use_nodelay;
 }; 
 typedef struct mca_btl_tcp_component_t mca_btl_tcp_component_t;
 
@@ -90,10 +108,12 @@ OMPI_MODULE_DECLSPEC extern mca_btl_tcp_component_t mca_btl_tcp_component;
  */
 struct mca_btl_tcp_module_t {
     mca_btl_base_module_t  super;  /**< base BTL interface */
-    mca_btl_base_recv_reg_t tcp_reg[256]; 
+    uint16_t           tcp_ifkindex; /** <BTL kernel interface index */
+#if 0
     int                tcp_ifindex; /**< PTL interface index */
-    struct sockaddr_in tcp_ifaddr;  /**< PTL interface address */
-    struct sockaddr_in tcp_ifmask;  /**< PTL interface netmask */
+#endif
+    struct sockaddr_storage tcp_ifaddr; /**< PTL interface address */
+    uint32_t           tcp_ifmask;  /**< PTL interface netmask */
     opal_list_t        tcp_endpoints;
 #if MCA_BTL_TCP_STATISTICS
     size_t tcp_bytes_sent;
@@ -247,21 +267,6 @@ extern int mca_btl_tcp_get(
     struct mca_btl_base_descriptor_t* decriptor
 );
 
-/**
- * Register a callback function that is called on receipt
- * of a fragment.
- *
- * @param btl (IN)     BTL module
- * @return             Status indicating if registration was successful
- *
- */
-
-extern int mca_btl_tcp_register(
-    struct mca_btl_base_module_t* btl, 
-    mca_btl_base_tag_t tag, 
-    mca_btl_base_module_recv_cb_fn_t cbfunc, 
-    void* cbdata); 
-    
 /**
  * Allocate a descriptor with a segment of the requested size.
  * Note that the BTL layer may choose to return a smaller size
@@ -272,8 +277,11 @@ extern int mca_btl_tcp_register(
  */
 
 extern mca_btl_base_descriptor_t* mca_btl_tcp_alloc(
-    struct mca_btl_base_module_t* btl, 
-    size_t size); 
+    struct mca_btl_base_module_t* btl,
+    struct mca_btl_base_endpoint_t* endpoint,
+    uint8_t order,
+    size_t size,
+    uint32_t flags); 
 
 
 /**
@@ -307,8 +315,10 @@ mca_btl_base_descriptor_t* mca_btl_tcp_prepare_src(
     struct mca_btl_base_endpoint_t* peer,
     struct mca_mpool_base_registration_t*,
     struct ompi_convertor_t* convertor,
+    uint8_t order,
     size_t reserve,
-    size_t* size
+    size_t* size,
+    uint32_t flags
 );
 
 extern mca_btl_base_descriptor_t* mca_btl_tcp_prepare_dst( 
@@ -316,10 +326,19 @@ extern mca_btl_base_descriptor_t* mca_btl_tcp_prepare_dst(
     struct mca_btl_base_endpoint_t* peer,
     struct mca_mpool_base_registration_t*,
     struct ompi_convertor_t* convertor,
+    uint8_t order,
     size_t reserve,
-    size_t* size); 
+    size_t* size,
+    uint32_t flags); 
 
 
+/**
+  * Fault Tolerance Event Notification Function
+  * @param state Checkpoint Stae
+  * @return OMPI_SUCCESS or failure status
+  */
+int mca_btl_tcp_ft_event(int state);
+
 #if defined(c_plusplus) || defined(__cplusplus)
 }
 #endif
