@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -29,6 +29,9 @@
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
+#ifdef HAVE_NET_UIO_H
+#include <net/uio.h>
+#endif
 
 #include "btl_tcp.h" 
 #include "btl_tcp_hdr.h"
@@ -79,24 +82,22 @@ OBJ_CLASS_DECLARATION(mca_btl_tcp_frag_user_t);
 
 #define MCA_BTL_TCP_FRAG_ALLOC_EAGER(frag, rc)                             \
 {                                                                          \
-                                                                           \
     ompi_free_list_item_t *item;                                           \
-    OMPI_FREE_LIST_WAIT(&mca_btl_tcp_component.tcp_frag_eager, item, rc);  \
+    OMPI_FREE_LIST_GET(&mca_btl_tcp_component.tcp_frag_eager, item, rc);   \
     frag = (mca_btl_tcp_frag_t*) item;                                     \
 }
 
 #define MCA_BTL_TCP_FRAG_ALLOC_MAX(frag, rc)                               \
 {                                                                          \
-                                                                           \
     ompi_free_list_item_t *item;                                           \
-    OMPI_FREE_LIST_WAIT(&mca_btl_tcp_component.tcp_frag_max, item, rc);    \
+    OMPI_FREE_LIST_GET(&mca_btl_tcp_component.tcp_frag_max, item, rc);     \
     frag = (mca_btl_tcp_frag_t*) item;                                     \
 }
 
 #define MCA_BTL_TCP_FRAG_ALLOC_USER(frag, rc)                              \
 {                                                                          \
     ompi_free_list_item_t *item;                                           \
-    OMPI_FREE_LIST_WAIT(&mca_btl_tcp_component.tcp_frag_user, item, rc);   \
+    OMPI_FREE_LIST_GET(&mca_btl_tcp_component.tcp_frag_user, item, rc);    \
     frag = (mca_btl_tcp_frag_t*) item;                                     \
 }
 
