@@ -40,7 +40,8 @@ int
 mca_coll_basic_scan_intra(void *sbuf, void *rbuf, int count,
                           struct ompi_datatype_t *dtype,
                           struct ompi_op_t *op,
-                          struct ompi_communicator_t *comm)
+                          struct ompi_communicator_t *comm,
+                          mca_coll_base_module_t *module)
 {
     int size, rank, err;
     ptrdiff_t true_lb, true_extent, lb, extent;
