@@ -40,15 +40,12 @@ mca_coll_basic_scatterv_intra(void *sbuf, int *scounts,
                               int *disps, struct ompi_datatype_t *sdtype,
                               void *rbuf, int rcount,
                               struct ompi_datatype_t *rdtype, int root,
-                              struct ompi_communicator_t *comm)
+                              struct ompi_communicator_t *comm,
+                              mca_coll_base_module_t *module)
 {
-    int i;
-    int rank;
-    int size;
-    int err;
+    int i, rank, size, err;
     char *ptmp;
-    long lb;
-    long extent;
+    ptrdiff_t lb, extent;
 
     /* Initialize */
 
@@ -116,20 +113,16 @@ mca_coll_basic_scatterv_inter(void *sbuf, int *scounts,
                               int *disps, struct ompi_datatype_t *sdtype,
                               void *rbuf, int rcount,
                               struct ompi_datatype_t *rdtype, int root,
-                              struct ompi_communicator_t *comm)
+                              struct ompi_communicator_t *comm,
+                              mca_coll_base_module_t *module)
 {
-    int i;
-    int rank;
-    int size;
-    int err;
+    int i, size, err;
     char *ptmp;
-    long lb;
-    long extent;
-    ompi_request_t **reqs = comm->c_coll_basic_data->mccb_reqs;
+    ptrdiff_t lb, extent;
+    mca_coll_basic_module_t *basic_module = (mca_coll_basic_module_t*) module;
+    ompi_request_t **reqs = basic_module->mccb_reqs;
 
     /* Initialize */
-
-    rank = ompi_comm_rank(comm);
     size = ompi_comm_remote_size(comm);
 
     /* If not root, receive data.  Note that we will only get here if
