@@ -34,7 +34,8 @@ int mca_coll_self_gatherv_intra(void *sbuf, int scount,
                                 struct ompi_datatype_t *sdtype,
                                 void *rbuf, int *rcounts, int *disps,
                                 struct ompi_datatype_t *rdtype, int root,
-                                struct ompi_communicator_t *comm)
+                                struct ompi_communicator_t *comm,
+                                mca_coll_base_module_t *module)
 {
     if (MPI_IN_PLACE == sbuf) {
         return MPI_SUCCESS;
