@@ -46,7 +46,9 @@ extern "C" {
     ompi_coll_tree_t*
     ompi_coll_tuned_topo_build_bmtree( struct ompi_communicator_t* comm,
                                        int root );
-
+    ompi_coll_tree_t*
+    ompi_coll_tuned_topo_build_in_order_bmtree( struct ompi_communicator_t* comm,
+						int root );
     ompi_coll_tree_t*
     ompi_coll_tuned_topo_build_chain( int fanout,
                                       struct ompi_communicator_t* com,
