@@ -1,14 +1,16 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /**
-  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
   *                         University Research and Technology
   *                         Corporation.  All rights reserved.
-  * Copyright (c) 2004-2005 The University of Tennessee and The University
+  * Copyright (c) 2004-2007 The University of Tennessee and The University
   *                         of Tennessee Research Foundation.  All rights
   *                         reserved.
   * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
   *                         University of Stuttgart.  All rights reserved.
   * Copyright (c) 2004-2005 The Regents of the University of California.
   *                         All rights reserved.
+  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
   * $COPYRIGHT$
   * 
   * Additional copyrights may follow
@@ -24,11 +26,15 @@
 #include "opal/mca/mca.h"
 #include "ompi/info/info.h"
 #include "ompi/class/ompi_free_list.h" 
-#include "ompi/class/ompi_pointer_array.h" 
 
-#define MCA_MPOOL_FLAGS_CACHE 0x1
+#include "opal/mca/crs/crs.h"
+#include "opal/mca/crs/base/base.h"
+
+#define MCA_MPOOL_FLAGS_CACHE_BYPASS 0x1
 #define MCA_MPOOL_FLAGS_PERSIST 0x2 
 #define MCA_MPOOL_FLAGS_MPI_ALLOC_MEM 0x4
+#define MCA_MPOOL_FLAGS_INVALID 0x8
+#define MCA_MPOOL_FLAGS_SO_MEM 0x10
 
 struct mca_mpool_base_resources_t;
 
@@ -38,7 +44,6 @@ struct mca_mpool_base_registration_t {
     unsigned char* base;
     unsigned char* bound; 
     unsigned char* alloc_base;
-    void* user_data; 
     int32_t ref_count; 
     uint32_t flags;
 };  
@@ -47,19 +52,6 @@ typedef struct mca_mpool_base_registration_t mca_mpool_base_registration_t;
 
 OMPI_DECLSPEC OBJ_CLASS_DECLARATION(mca_mpool_base_registration_t); 
 
-#define MCA_MPOOL_REG_RETAIN(reg) { \
- do{ \
- OPAL_THREAD_ADD32(&reg->ref_count, 1); \
- } while(0); \
-}
-
-#define MCA_MPOOL_REG_RELEASE(reg) { \
- do{ \
- OPAL_THREAD_ADD32(&reg->ref_count, -1); \
- } while(0); \
-}
-
-
 /**
  * component initialize
  */
@@ -111,26 +103,12 @@ typedef int (*mca_mpool_base_module_deregister_fn_t)(
     mca_mpool_base_registration_t* registration);
 
 /**
- * find registrations in this memory pool
+ * find registration in this memory pool
  */ 
 
 typedef int (*mca_mpool_base_module_find_fn_t) (
-                                                struct mca_mpool_base_module_t* mpool, 
-                                                void* addr, 
-                                                size_t size, 
-                                                ompi_pointer_array_t* regs, 
-                                                uint32_t *cnt 
-                                                ); 
-
-
-/** 
- * retain registration
- */ 
-
-typedef int (*mca_mpool_base_module_retain_fn_t) ( 
-                                            struct mca_mpool_base_module_t* mpool, 
-                                            mca_mpool_base_registration_t* registration); 
-
+        struct mca_mpool_base_module_t* mpool, void* addr, size_t size,
+        mca_mpool_base_registration_t **reg);
 
 /** 
  * release registration
@@ -141,6 +119,12 @@ typedef int (*mca_mpool_base_module_release_fn_t) (
                                             mca_mpool_base_registration_t* registration); 
 
                                             
+/**
+ * release memory region
+ */
+typedef int (*mca_mpool_base_module_release_memory_fn_t) (
+        struct mca_mpool_base_module_t* mpool, void *base, size_t size);
+
 /**
   * if appropriate - returns base address of memory pool
   */
@@ -152,24 +136,32 @@ typedef void* (*mca_mpool_base_module_address_fn_t)(struct mca_mpool_base_module
 typedef void (*mca_mpool_base_module_finalize_fn_t)(struct mca_mpool_base_module_t*);
 
 
+/**
+ * Fault Tolerance Event Notification Function
+ * @param state Checkpoint Stae
+ * @return OMPI_SUCCESS or failure status
+ */
+typedef int (*mca_mpool_base_module_ft_event_fn_t)(int state);
+
+
 /**
  * mpool component descriptor. Contains component version information
  * and open/close/init functions.
  */
-struct mca_mpool_base_component_1_0_0_t {
+struct mca_mpool_base_component_2_0_0_t {
   mca_base_component_t mpool_version;        /**< version */
-  mca_base_component_data_1_0_0_t mpool_data;/**< metadata */
+  mca_base_component_data_t mpool_data;/**< metadata */
 
   mca_mpool_base_component_init_fn_t mpool_init;    /**< init function */
 };
 /**
  * Convenience typedef.
  */
-typedef struct mca_mpool_base_component_1_0_0_t mca_mpool_base_component_1_0_0_t;
+typedef struct mca_mpool_base_component_2_0_0_t mca_mpool_base_component_2_0_0_t;
 /**
   * Convenience typedef
   */
-typedef struct mca_mpool_base_component_1_0_0_t mca_mpool_base_component_t;
+typedef struct mca_mpool_base_component_2_0_0_t mca_mpool_base_component_t;
 
 /**
  *  mpool module descriptor. Contains the interface functions exported
@@ -185,9 +177,10 @@ struct mca_mpool_base_module_t {
     mca_mpool_base_module_register_fn_t mpool_register;  /**< register memory */
     mca_mpool_base_module_deregister_fn_t mpool_deregister; /**< deregister memory */
     mca_mpool_base_module_find_fn_t mpool_find; /**< find regisrations in the cache */
-    mca_mpool_base_module_retain_fn_t mpool_retain; /**< retain a registration from the cache */ 
     mca_mpool_base_module_release_fn_t mpool_release; /**< release a registration from the cache */ 
+    mca_mpool_base_module_release_memory_fn_t mpool_release_memory; /**< release memor region from the cache  */
     mca_mpool_base_module_finalize_fn_t mpool_finalize;  /**< finalize */
+    mca_mpool_base_module_ft_event_fn_t mpool_ft_event;  /**< ft_event */
     struct mca_rcache_base_module_t *rcache; /* the rcache associated with this mpool */ 
     uint32_t flags; /**< mpool flags */
 };
@@ -252,13 +245,11 @@ OMPI_DECLSPEC int mca_mpool_base_insert(
 OMPI_DECLSPEC int mca_mpool_base_remove(void * base); 
 
 /**
- * Macro for use in components that are of type mpool v1.0.0
+ * Macro for use in components that are of type mpool
  */
-#define MCA_MPOOL_BASE_VERSION_1_0_0 \
-  /* mpool v1.0 is chained to MCA v1.0 */ \
-  MCA_BASE_VERSION_1_0_0, \
-  /* mpool v1.0 */ \
-  "mpool", 1, 0, 0
+#define MCA_MPOOL_BASE_VERSION_2_0_0 \
+  MCA_BASE_VERSION_2_0_0, \
+  "mpool", 2, 0, 0
 
 #endif /* MCA_MPOOL_H */
 
