@@ -24,21 +24,22 @@
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/mtl/mtl.h"
+#include "opal/prefetch.h"
 
 struct mca_pml_cm_send_request_t { 
     mca_pml_cm_request_t req_base;
     mca_pml_base_send_mode_t req_send_mode;
 };
 typedef struct mca_pml_cm_send_request_t mca_pml_cm_send_request_t;
-OMPI_DECLSPEC OBJ_CLASS_DECLARATION(mca_pml_cm_send_request_t);
+OBJ_CLASS_DECLARATION(mca_pml_cm_send_request_t);
 
 
 struct mca_pml_cm_thin_send_request_t { 
     mca_pml_cm_send_request_t req_send;
-    mca_mtl_request_t req_mtl;            /**< the mtl specific memory. This field should be the last in the struct */   
+    mca_mtl_request_t req_mtl;            /**< the mtl specific memory. This field should be the last in the struct */
 };
 typedef struct mca_pml_cm_thin_send_request_t mca_pml_cm_thin_send_request_t;
-OMPI_DECLSPEC OBJ_CLASS_DECLARATION(mca_pml_cm_thin_send_request_t);
+OBJ_CLASS_DECLARATION(mca_pml_cm_thin_send_request_t);
 
 
 struct mca_pml_cm_hvy_send_request_t {
@@ -52,7 +53,7 @@ struct mca_pml_cm_hvy_send_request_t {
     mca_mtl_request_t req_mtl;            /**< the mtl specific memory. This field should be the last in the struct */
 };
 typedef struct mca_pml_cm_hvy_send_request_t mca_pml_cm_hvy_send_request_t;
-OMPI_DECLSPEC OBJ_CLASS_DECLARATION(mca_pml_cm_hvy_send_request_t);
+OBJ_CLASS_DECLARATION(mca_pml_cm_hvy_send_request_t);
 
 
 #define MCA_PML_CM_THIN_SEND_REQUEST_ALLOC(sendreq, comm, dst,          \
@@ -61,7 +62,7 @@ do {                                                                    \
     ompi_free_list_item_t* item;                                        \
     ompi_proc = ompi_comm_peer_lookup( comm, dst );                     \
                                                                         \
-    if(NULL == ompi_proc) {                              \
+    if(OPAL_UNLIKELY(NULL == ompi_proc)) {                              \
         rc = OMPI_ERR_OUT_OF_RESOURCE;                                  \
         sendreq = NULL;                                                 \
     } else {                                                            \
@@ -81,7 +82,7 @@ do {                                                                    \
 {                                                                       \
     ompi_free_list_item_t* item;                                        \
     ompi_proc = ompi_comm_peer_lookup( comm, dst );                     \
-    if(NULL == ompi_proc) {                              \
+    if(OPAL_UNLIKELY(NULL == ompi_proc)) {                              \
         rc = OMPI_ERR_OUT_OF_RESOURCE;                                  \
         sendreq = NULL;                                                 \
     } else {                                                            \
@@ -107,22 +108,22 @@ do {                                                                    \
 {                                                                       \
     OBJ_RETAIN(comm);                                                   \
     OBJ_RETAIN(datatype);                                               \
-    req_send->req_base.req_comm = comm;                                 \
-    req_send->req_base.req_datatype = datatype;                         \
+    (req_send)->req_base.req_comm = comm;                               \
+    (req_send)->req_base.req_datatype = datatype;                       \
     ompi_convertor_copy_and_prepare_for_send(                           \
                                              ompi_proc->proc_convertor, \
                                              datatype,                  \
                                              count,                     \
                                              buf,                       \
                                              0,                         \
-                                             &req_send->req_base.req_convertor ); \
-    req_send->req_base.req_ompi.req_mpi_object.comm = comm;             \
-    req_send->req_base.req_ompi.req_status.MPI_SOURCE =                  \
+                                             &(req_send)->req_base.req_convertor ); \
+    (req_send)->req_base.req_ompi.req_mpi_object.comm = comm;           \
+    (req_send)->req_base.req_ompi.req_status.MPI_SOURCE =               \
         comm->c_my_rank;                                                \
-    req_send->req_base.req_ompi.req_status.MPI_TAG = tag;                \
-    req_send->req_base.req_ompi.req_status._count = count;               \
-    req_send->req_send_mode = sendmode;                                  \
-    req_send->req_base.req_free_called = false;                          \
+    (req_send)->req_base.req_ompi.req_status.MPI_TAG = tag;             \
+    (req_send)->req_base.req_ompi.req_status._count = count;            \
+    (req_send)->req_send_mode = sendmode;                               \
+    (req_send)->req_base.req_free_called = false;                       \
 }
 
 #define MCA_PML_CM_HVY_SEND_REQUEST_INIT( sendreq,                      \
@@ -136,7 +137,6 @@ do {                                                                    \
                                           blocking,                     \
                                           buf,                          \
                                           count)                        \
-{                                                                       \
     do {                                                                \
         OMPI_REQUEST_INIT(&(sendreq->req_send.req_base.req_ompi),       \
                           persistent);                                  \
@@ -144,26 +144,22 @@ do {                                                                    \
         sendreq->req_peer = dst;                                        \
         sendreq->req_addr = buf;                                        \
         sendreq->req_count = count;                                     \
-        MCA_PML_CM_SEND_REQUEST_INIT_COMMON(                            \
-                                            (&sendreq->req_send),       \
-                                            ompi_proc,                  \
-                                            comm,                       \
-                                            tag,                        \
-                                            datatype,                   \
-                                            sendmode,                   \
-                                            buf,                        \
-                                            count);                     \
+        MCA_PML_CM_SEND_REQUEST_INIT_COMMON( (&sendreq->req_send),      \
+                                             ompi_proc,                 \
+                                             comm,                      \
+                                             tag,                       \
+                                             datatype,                  \
+                                             sendmode,                  \
+                                             buf,                       \
+                                             count);                    \
         ompi_convertor_get_packed_size(                                 \
                                        &sendreq->req_send.req_base.req_convertor, \
-                                       &sendreq->req_count  );          \
-                                                                        \
+                                       &sendreq->req_count );           \
                                                                         \
         sendreq->req_blocking = blocking;                               \
         sendreq->req_send.req_base.req_pml_complete =                   \
             (persistent ? true:false);                                  \
-    }while(0);                                                          \
-}
-
+    } while(0)
 
 
 #define MCA_PML_CM_THIN_SEND_REQUEST_INIT( sendreq,                     \
@@ -175,34 +171,29 @@ do {                                                                    \
                                            sendmode,                    \
                                            buf,                         \
                                            count)                       \
-{                                                                       \
     do {                                                                \
         OMPI_REQUEST_INIT(&(sendreq->req_send.req_base.req_ompi),       \
                           false);                                       \
-        MCA_PML_CM_SEND_REQUEST_INIT_COMMON(                       \
-                                            (&sendreq->req_send),       \
-                                            ompi_proc,                  \
-                                            comm,                       \
-                                            tag,                        \
-                                            datatype,                   \
-                                            sendmode,                   \
-                                            buf,                        \
-                                            count);                     \
+        MCA_PML_CM_SEND_REQUEST_INIT_COMMON( (&sendreq->req_send),      \
+                                             ompi_proc,                 \
+                                             comm,                      \
+                                             tag,                       \
+                                             datatype,                  \
+                                             sendmode,                  \
+                                             buf,                       \
+                                             count);                    \
         sendreq->req_send.req_base.req_pml_complete = false;            \
-    }while(0);                                                          \
-}
+    } while(0)
 
 
 #define MCA_PML_CM_SEND_REQUEST_START_SETUP(req_send)                   \
     do {                                                                \
-                                                                        \
-    req_send->req_base.req_pml_complete = false;                        \
-    req_send->req_base.req_ompi.req_complete = false;                   \
-    req_send->req_base.req_ompi.req_state =                             \
-        OMPI_REQUEST_ACTIVE;                                            \
-    req_send->req_base.req_ompi.req_status._cancelled = 0;              \
-                                                                        \
-} while (0)
+        (req_send)->req_base.req_pml_complete = false;                  \
+        (req_send)->req_base.req_ompi.req_complete = false;             \
+        (req_send)->req_base.req_ompi.req_state =                       \
+            OMPI_REQUEST_ACTIVE;                                        \
+        (req_send)->req_base.req_ompi.req_status._cancelled = 0;        \
+    } while (0)
 
 
 #define MCA_PML_CM_THIN_SEND_REQUEST_START(sendreq,                     \
@@ -213,7 +204,7 @@ do {                                                                    \
                                            blocking,                    \
                                            ret)                         \
 do {                                                                    \
-    MCA_PML_CM_SEND_REQUEST_START_SETUP((&sendreq->req_send));          \
+    MCA_PML_CM_SEND_REQUEST_START_SETUP(&(sendreq)->req_send);          \
     ret = OMPI_MTL_CALL(isend(ompi_mtl,                                 \
                               comm,                                     \
                               dst,                                      \
@@ -250,28 +241,28 @@ do {                                                                    \
  } while(0);
         
 
-#define MCA_PML_CM_HVY_SEND_REQUEST_START(sendreq, ret)                 \
-do {                                                                    \
-    ret = OMPI_SUCCESS;                                                 \
-    MCA_PML_CM_SEND_REQUEST_START_SETUP((&sendreq->req_send));          \
-    if (sendreq->req_send.req_send_mode == MCA_PML_BASE_SEND_BUFFERED) {  \
-        MCA_PML_CM_HVY_SEND_REQUEST_BSEND_ALLOC(sendreq, ret);          \
-    }                                                                   \
-    if (OMPI_SUCCESS == ret) {                                          \
-        ret = OMPI_MTL_CALL(isend(ompi_mtl,                             \
-                                  sendreq->req_send.req_base.req_comm,  \
-                                  sendreq->req_peer,                    \
-                                  sendreq->req_tag,                     \
-                                  &sendreq->req_send.req_base.req_convertor, \
-                                  sendreq->req_send.req_send_mode,      \
-                                  sendreq->req_blocking,                \
-                                  &sendreq->req_mtl));                  \
-        if(OMPI_SUCCESS == ret &&                                       \
-           sendreq->req_send.req_send_mode == MCA_PML_BASE_SEND_BUFFERED) { \
-            sendreq->req_send.req_base.req_ompi.req_status.MPI_ERROR = 0; \
-            MCA_PML_BASE_REQUEST_MPI_COMPLETE(&(sendreq->req_send.req_base.req_ompi)); \
-        }                                                               \
-    }                                                                   \
+#define MCA_PML_CM_HVY_SEND_REQUEST_START(sendreq, ret)                          \
+do {                                                                             \
+    ret = OMPI_SUCCESS;                                                          \
+    MCA_PML_CM_SEND_REQUEST_START_SETUP(&(sendreq)->req_send);                   \
+    if (sendreq->req_send.req_send_mode == MCA_PML_BASE_SEND_BUFFERED) {         \
+        MCA_PML_CM_HVY_SEND_REQUEST_BSEND_ALLOC(sendreq, ret);                   \
+    }                                                                            \
+    if (OMPI_SUCCESS == ret) {                                                   \
+        ret = OMPI_MTL_CALL(isend(ompi_mtl,                                      \
+                                  sendreq->req_send.req_base.req_comm,           \
+                                  sendreq->req_peer,                             \
+                                  sendreq->req_tag,                              \
+                                  &sendreq->req_send.req_base.req_convertor,     \
+                                  sendreq->req_send.req_send_mode,               \
+                                  sendreq->req_blocking,                         \
+                                  &sendreq->req_mtl));                           \
+        if(OMPI_SUCCESS == ret &&                                                \
+           sendreq->req_send.req_send_mode == MCA_PML_BASE_SEND_BUFFERED) {      \
+            sendreq->req_send.req_base.req_ompi.req_status.MPI_ERROR = 0;        \
+            ompi_request_complete(&(sendreq)->req_send.req_base.req_ompi, true); \
+        }                                                                        \
+    }                                                                            \
  } while (0)
 
 /*
@@ -281,36 +272,36 @@ do {                                                                    \
  * This macro will never be called directly from the upper level, as it should
  * only be an internal call to the PML.
  */
-#define MCA_PML_CM_HVY_SEND_REQUEST_PML_COMPLETE(sendreq)               \
-do {                                                                    \
-    assert( false == sendreq->req_send.req_base.req_pml_complete );     \
-                                                                        \
-    if (sendreq->req_send.req_send_mode == MCA_PML_BASE_SEND_BUFFERED && \
-        sendreq->req_count > 0 ) {                                      \
-        mca_pml_base_bsend_request_free(sendreq->req_buff);             \
-    }                                                                   \
-                                                                        \
-    OPAL_THREAD_LOCK(&ompi_request_lock);                               \
-    if( false == sendreq->req_send.req_base.req_ompi.req_complete ) {   \
-        /* Should only be called for long messages (maybe synchronous) */ \
-        MCA_PML_BASE_REQUEST_MPI_COMPLETE(&(sendreq->req_send.req_base.req_ompi)); \
-    }                                                                   \
-    sendreq->req_send.req_base.req_pml_complete = true;                 \
-                                                                        \
-    if( sendreq->req_send.req_base.req_free_called ) {                           \
-        MCA_PML_CM_HVY_SEND_REQUEST_RETURN( sendreq );                  \
-    } else {                                                            \
-        if(sendreq->req_send.req_base.req_ompi.req_persistent) {                 \
-            /* rewind convertor */                                      \
-            size_t offset = 0;                                          \
+#define MCA_PML_CM_HVY_SEND_REQUEST_PML_COMPLETE(sendreq)                          \
+do {                                                                               \
+    assert( false == sendreq->req_send.req_base.req_pml_complete );                \
+                                                                                   \
+    if (sendreq->req_send.req_send_mode == MCA_PML_BASE_SEND_BUFFERED &&           \
+        sendreq->req_count > 0 ) {                                                 \
+        mca_pml_base_bsend_request_free(sendreq->req_buff);                        \
+    }                                                                              \
+                                                                                   \
+    OPAL_THREAD_LOCK(&ompi_request_lock);                                          \
+    if( false == sendreq->req_send.req_base.req_ompi.req_complete ) {              \
+        /* Should only be called for long messages (maybe synchronous) */          \
+        ompi_request_complete(&(sendreq->req_send.req_base.req_ompi), true);       \
+    }                                                                              \
+    sendreq->req_send.req_base.req_pml_complete = true;                            \
+                                                                                   \
+    if( sendreq->req_send.req_base.req_free_called ) {                             \
+        MCA_PML_CM_HVY_SEND_REQUEST_RETURN( sendreq );                             \
+    } else {                                                                       \
+        if(sendreq->req_send.req_base.req_ompi.req_persistent) {                   \
+            /* rewind convertor */                                                 \
+            size_t offset = 0;                                                     \
             ompi_convertor_set_position(&sendreq->req_send.req_base.req_convertor, \
-                                        &offset);                       \
-        }                                                               \
-    }                                                                   \
-    OPAL_THREAD_UNLOCK(&ompi_request_lock);                             \
+                                        &offset);                                  \
+        }                                                                          \
+    }                                                                              \
+    OPAL_THREAD_UNLOCK(&ompi_request_lock);                                        \
  } while (0)
-    
-    
+
+
 /*
  * Release resources associated with a request
  */
@@ -321,8 +312,8 @@ do {                                                                    \
         OBJ_RELEASE(sendreq->req_send.req_base.req_comm);               \
         OMPI_REQUEST_FINI(&sendreq->req_send.req_base.req_ompi);        \
         ompi_convertor_cleanup( &(sendreq->req_send.req_base.req_convertor) ); \
-        OMPI_FREE_LIST_RETURN(&mca_pml_base_send_requests,              \
-                              (ompi_free_list_item_t*)sendreq);         \
+        OMPI_FREE_LIST_RETURN( &mca_pml_base_send_requests,             \
+                               (ompi_free_list_item_t*)sendreq);        \
     }
 
 /*
@@ -332,21 +323,21 @@ do {                                                                    \
  * This macro will never be called directly from the upper level, as it should
  * only be an internal call to the PML.
  */
-#define MCA_PML_CM_THIN_SEND_REQUEST_PML_COMPLETE(sendreq)              \
-do {                                                                    \
-    assert( false == sendreq->req_send.req_base.req_pml_complete );              \
-                                                                        \
-    OPAL_THREAD_LOCK(&ompi_request_lock);                               \
-    if( false == sendreq->req_send.req_base.req_ompi.req_complete ) {   \
-        /* Should only be called for long messages (maybe synchronous) */ \
-        MCA_PML_BASE_REQUEST_MPI_COMPLETE(&(sendreq->req_send.req_base.req_ompi)); \
-    }                                                                   \
-    sendreq->req_send.req_base.req_pml_complete = true;                 \
-                                                                        \
-    if( sendreq->req_send.req_base.req_free_called ) {                  \
-        MCA_PML_CM_THIN_SEND_REQUEST_RETURN( sendreq );                 \
-    }                                                                   \
-    OPAL_THREAD_UNLOCK(&ompi_request_lock);                             \
+#define MCA_PML_CM_THIN_SEND_REQUEST_PML_COMPLETE(sendreq)                   \
+do {                                                                         \
+    assert( false == sendreq->req_send.req_base.req_pml_complete );          \
+                                                                             \
+    OPAL_THREAD_LOCK(&ompi_request_lock);                                    \
+    if( false == sendreq->req_send.req_base.req_ompi.req_complete ) {        \
+        /* Should only be called for long messages (maybe synchronous) */    \
+        ompi_request_complete(&(sendreq->req_send.req_base.req_ompi), true); \
+    }                                                                        \
+    sendreq->req_send.req_base.req_pml_complete = true;                      \
+                                                                             \
+    if( sendreq->req_send.req_base.req_free_called ) {                       \
+        MCA_PML_CM_THIN_SEND_REQUEST_RETURN( sendreq );                      \
+    }                                                                        \
+    OPAL_THREAD_UNLOCK(&ompi_request_lock);                                  \
  } while (0)
     
     
@@ -360,9 +351,8 @@ do {                                                                    \
         OBJ_RELEASE(sendreq->req_send.req_base.req_comm);               \
         OMPI_REQUEST_FINI(&sendreq->req_send.req_base.req_ompi);        \
         ompi_convertor_cleanup( &(sendreq->req_send.req_base.req_convertor) ); \
-        OMPI_FREE_LIST_RETURN(                                          \
-                              &mca_pml_base_send_requests,              \
-                              (ompi_free_list_item_t*)sendreq);         \
+        OMPI_FREE_LIST_RETURN( &mca_pml_base_send_requests,             \
+                               (ompi_free_list_item_t*)sendreq);        \
     }
 
 extern void
