@@ -1132,7 +1132,7 @@ static void mca_pml_ob1_put_completion( mca_btl_base_module_t* btl,
 
     mca_pml_ob1_send_fin(sendreq->req_send.req_base.req_proc, 
                          bml_btl,
-                         frag->rdma_hdr.hdr_rdma.hdr_des.pval,
+                         frag->rdma_hdr.hdr_rdma.hdr_des,
                          des->order, 0);
     
     /* check for request completion */
@@ -1177,7 +1177,7 @@ int mca_pml_ob1_send_request_put_frag( mca_pml_ob1_rdma_frag_t* frag )
 
             /* tell receiver to unregister memory */
             mca_pml_ob1_send_fin(sendreq->req_send.req_base.req_proc,
-                    bml_btl, frag->rdma_hdr.hdr_rdma.hdr_des.pval,
+                    bml_btl, frag->rdma_hdr.hdr_rdma.hdr_des,
                     MCA_BTL_NO_ORDER, 1);
 
             /* send fragment by copy in/out */
@@ -1247,7 +1247,7 @@ void mca_pml_ob1_send_request_put( mca_pml_ob1_send_request_t* sendreq,
 
     /* setup fragment */
     for( i = 0; i < hdr->hdr_seg_cnt; i++ ) {
-        frag->rdma_segs[i].seg_addr.lval = hdr->hdr_segs[i].seg_addr.lval;
+        frag->rdma_segs[i].seg_addr      = hdr->hdr_segs[i].seg_addr;
         frag->rdma_segs[i].seg_len       = hdr->hdr_segs[i].seg_len;
         frag->rdma_segs[i].seg_key.key64 = hdr->hdr_segs[i].seg_key.key64;
 
