@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,15 +25,15 @@
  * An MCA component type that provides the P2P interface functionality
  * required by the MPI layer. The PML is a relatively thin layer that
  * primarily provides for the fragmentation and scheduling of messages
- * over multiple transports (instances of the P2P Transport Layer
- * (PTL) MCA component type) as depicted below:
+ * over multiple transports (instances of the Byte Transfer Layer
+ * (BTL) MCA component type) as depicted below:
  *
  *   ------------------------------------
  *   |                MPI               |
  *   ------------------------------------
  *   |                PML               |
  *   ------------------------------------
- *   | PTL (TCP) | PTL (SM) | PTL (...) |
+ *   | BTL (TCP) | BTL (SM) | BTL (...) |
  *   ------------------------------------
  *
  * A single PML component is selected by the MCA framework during
@@ -46,13 +48,13 @@
  * processes (ompi_proc_t instances), and notification of changes
  * (add/delete).
  * 
- * The PML module must select the set of PTL components that are to be
+ * The PML module must select the set of BTL components that are to be
  * used to reach a given destination. These should be cached on a PML
  * specific data structure that is hung off the ompi_proc_t.
  *
  * The PML should then apply a scheduling algorithm (round-robin,
  * weighted distribution, etc), to schedule the delivery of messages
- * over the available PTLs.
+ * over the available BTLs.
  *
  */
                                                                                          
@@ -74,19 +76,28 @@ extern "C" {
 
 typedef uint64_t mca_pml_sequence_t;
 
-struct mca_pml_proc_t {
-    opal_list_item_t super;
-    struct ompi_proc_t *proc_ompi;    /**< back-pointer to ompi_proc_t */
-    opal_mutex_t proc_lock;           /**< lock to protect against concurrent access */
-};
-typedef struct mca_pml_proc_t mca_pml_proc_t; 
+/** 
+ * Base PML endpoint structure
+ *
+ * Base PML structure for caching endpoint information on a proc.  A
+ * pointer to an mca_pml_endpoint_t is maintained on each ompi_proc_t,
+ * in the proc_pml field, to provide per-process cache information.
+ * The data is opaque to the active PML -- no other subsystem will
+ * attempt to access the information in the cache.
+ *
+ * The PML is responsible for allocation and deallocation of the
+ * endpoint data during pml_add_procs and pml_del_procs.
+ */
+struct mca_pml_endpoint_t;
+struct ompi_proc_t;
 
 typedef enum {
-    MCA_PML_BASE_SEND_STANDARD,
-    MCA_PML_BASE_SEND_BUFFERED,
     MCA_PML_BASE_SEND_SYNCHRONOUS,
+    MCA_PML_BASE_SEND_COMPLETE,
+    MCA_PML_BASE_SEND_BUFFERED,
     MCA_PML_BASE_SEND_READY,
-    MCA_PML_BASE_SEND_COMPLETE
+    MCA_PML_BASE_SEND_STANDARD,
+    MCA_PML_BASE_SEND_SIZE
 } mca_pml_base_send_mode_t;
 
 
@@ -94,7 +105,6 @@ typedef enum {
 #define OMPI_ANY_SOURCE MPI_ANY_SOURCE
 #define OMPI_PROC_NULL  MPI_PROC_NULL
 
-
 /**
  * MCA->PML Called by MCA framework to initialize the component.
  * 
@@ -120,14 +130,14 @@ typedef int (*mca_pml_base_component_finalize_fn_t)(void);
  * PML component version and interface functions.
  */
 
-struct mca_pml_base_component_1_0_0_t {
+struct mca_pml_base_component_2_0_0_t {
    mca_base_component_t pmlm_version;
-   mca_base_component_data_1_0_0_t pmlm_data;
+   mca_base_component_data_t pmlm_data;
    mca_pml_base_component_init_fn_t pmlm_init;
    mca_pml_base_component_finalize_fn_t pmlm_finalize;
 };
-typedef struct mca_pml_base_component_1_0_0_t mca_pml_base_component_1_0_0_t;
-typedef mca_pml_base_component_1_0_0_t mca_pml_base_component_t;
+typedef struct mca_pml_base_component_2_0_0_t mca_pml_base_component_2_0_0_t;
+typedef mca_pml_base_component_2_0_0_t mca_pml_base_component_t;
 
 
 /**
@@ -144,7 +154,7 @@ typedef mca_pml_base_component_1_0_0_t mca_pml_base_component_t;
  *
  * Provides a notification to the PML that new processes have been
  * created, and provides the PML the opportunity to cache data
- * (e.g. list of PTLs to use) on the ompi_proc_t data structure.
+ * (e.g. list of BTLs to use) on the ompi_proc_t data structure.
  */
 typedef int (*mca_pml_base_module_add_procs_fn_t)(struct ompi_proc_t **procs, size_t nprocs);
 
@@ -163,7 +173,7 @@ typedef int (*mca_pml_base_module_add_procs_fn_t)(struct ompi_proc_t **procs, si
 typedef int (*mca_pml_base_module_del_procs_fn_t)(struct ompi_proc_t **procs, size_t nprocs);
 
 /**
- * Downcall from MCA layer to enable the PML/PTLs.
+ * Downcall from MCA layer to enable the PML/BTLs.
  *
  * @param   enable  Enable/Disable PML forwarding
  * @return          OMPI_SUCCESS or failure status.
@@ -177,7 +187,9 @@ typedef int (*mca_pml_base_module_enable_fn_t)(
  * For non-threaded case, provides MCA the opportunity to
  * progress outstanding requests on all btls.
  *
- * @return         OMPI_SUCCESS or failure status.
+ *  * @return        Count of "completions", a metric of 
+ *                   how many items where completed in the call 
+ *                   to progress.
 */
 typedef int (*mca_pml_base_module_progress_fn_t)(void);
 
@@ -446,6 +458,7 @@ typedef int (*mca_pml_base_module_null_fn_t)(
  * Diagnostics function.
  *
  * @param request (IN)    Communicator
+ * @param verbose (IN)    Verbosity level (passed to BTL)
  * @return                OMPI_SUCCESS or failure status.
  *
  */
@@ -454,6 +467,14 @@ typedef int (*mca_pml_base_module_dump_fn_t)(
     int verbose
 );
 
+/**
+ * Fault Tolerance Awareness function
+ * @param status     Checkpoint status
+ * @return           OMPI_SUCCESS or failure status
+ */
+typedef int (*mca_pml_base_module_ft_event_fn_t) (int status);
+
+
 
 /**
  *  PML instance.
@@ -483,6 +504,9 @@ struct mca_pml_base_module_1_0_0_t {
     /* diagnostics */
     mca_pml_base_module_dump_fn_t         pml_dump;
 
+    /* FT Event */
+    mca_pml_base_module_ft_event_fn_t     pml_ft_event;
+
     /* maximum constant sizes */
     int                                   pml_max_contextid;
     int                                   pml_max_tag;
@@ -491,20 +515,18 @@ typedef struct mca_pml_base_module_1_0_0_t mca_pml_base_module_1_0_0_t;
 typedef mca_pml_base_module_1_0_0_t mca_pml_base_module_t;
 
 /*
- * Macro for use in components that are of type pml v1.0.0
+ * Macro for use in components that are of type pml
  */
-#define MCA_PML_BASE_VERSION_1_0_0 \
-  /* pml v1.0 is chained to MCA v1.0 */ \
-  MCA_BASE_VERSION_1_0_0, \
-  /* pml v1.0 */ \
-  "pml", 1, 0, 0
+#define MCA_PML_BASE_VERSION_2_0_0 \
+  MCA_BASE_VERSION_2_0_0, \
+  "pml", 2, 0, 0
 
     /*
      * macro for doing direct call / call through struct
      */
 #if MCA_pml_DIRECT_CALL
 
-#include "ompi/mca/pml/pml_direct_call.h"
+#include MCA_pml_DIRECT_CALL_HEADER
 
 #define MCA_PML_CALL_STAMP(a, b) mca_pml_ ## a ## _ ## b
 #define MCA_PML_CALL_EXPANDER(a, b) MCA_PML_CALL_STAMP(a,b)
