@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,6 +22,7 @@
 
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/datatype/datatype.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Allgather = PMPI_Allgather
@@ -34,11 +36,33 @@ static const char FUNC_NAME[] = "MPI_Allgather";
 
 
 int MPI_Allgather(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
-		  void *recvbuf, int recvcount, MPI_Datatype recvtype,
-		  MPI_Comm comm) 
+                  void *recvbuf, int recvcount, MPI_Datatype recvtype,
+                  MPI_Comm comm)
 {
     int err;
 
+    MEMCHECKER(
+        int rank;
+        ptrdiff_t ext;
+
+        rank = ompi_comm_rank(comm);
+        ompi_ddt_type_extent(recvtype, &ext);
+
+        memchecker_datatype(recvtype);
+        memchecker_comm(comm);
+        /* check whether the actual send buffer is defined. */
+        if (MPI_IN_PLACE == sendbuf) {
+            memchecker_call(&opal_memchecker_base_isdefined, 
+                            (char *)(recvbuf)+rank*ext, 
+                            recvcount, recvtype);
+        } else {
+            memchecker_datatype(sendtype);
+            memchecker_call(&opal_memchecker_base_isdefined, sendbuf, sendcount, sendtype);
+        }
+        /* check whether the receive buffer is addressable. */
+        memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
+    );
+
     if (MPI_PARAM_CHECK) {
 
         /* Unrooted operation -- same checks for all ranks on both
@@ -48,7 +72,7 @@ int MPI_Allgather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if (ompi_comm_invalid(comm)) {
           OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM, FUNC_NAME);
-        } else if (MPI_DATATYPE_NULL == recvtype) {
+        } else if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
           err = MPI_ERR_TYPE;
         } else if (recvcount < 0) {
           err = MPI_ERR_COUNT;
@@ -60,18 +84,23 @@ int MPI_Allgather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
         OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
     }
 
-    /* Can we optimize?  Everyone had to give the same send signature,
-       which means that everyone must have given a sendcount > 0 if
-       there's anything to send. */
+    /* Do we need to do anything?  Everyone had to give the same send
+       signature, which means that everyone must have given a
+       sendcount > 0 if there's anything to send.  If we're doing
+       IN_PLACE, however, check recvcount, not sendcount. */
 
-    if (sendcount == 0) {
-      return MPI_SUCCESS;
+    if ((MPI_IN_PLACE != sendbuf && 0 == sendcount) ||
+        (0 == recvcount)) {
+        return MPI_SUCCESS;
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* Invoke the coll component to perform the back-end operation */
 
     err = comm->c_coll.coll_allgather(sendbuf, sendcount, sendtype, 
-                                      recvbuf, recvcount, recvtype, comm);
+                                      recvbuf, recvcount, recvtype, comm,
+                                      comm->c_coll.coll_allgather_module);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
 
