@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007-2008 Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,6 +20,7 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Comm_get_errhandler = PMPI_Comm_get_errhandler
@@ -34,12 +36,20 @@ static const char FUNC_NAME[] = "MPI_Comm_get_errhandler";
 
 int MPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *errhandler)
 {
+    MPI_Errhandler tmp;
+
+    /* Error checking */
+    MEMCHECKER(
+        memchecker_comm(comm);
+    );
+
+    OPAL_CR_NOOP_PROGRESS();
+
   /* Error checking */
 
   if (MPI_PARAM_CHECK) {
     OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
-    if (NULL == comm || 
-        MPI_COMM_NULL == comm) {
+    if (ompi_comm_invalid(comm)) {
       return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG,
                                     FUNC_NAME);
     } else if (NULL == errhandler) {
@@ -48,10 +58,16 @@ int MPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *errhandler)
     }
   }
 
-  /* Retain the errhandler, corresponding to object refcount
-     decrease in errhandler_free.c. */
-  OBJ_RETAIN(comm->error_handler);
-  *errhandler = comm->error_handler;
+  /* On 64 bits environments we have to make sure the reading of the
+     error_handler became atomic. */
+  do {
+      tmp = comm->error_handler;
+  } while (!OPAL_ATOMIC_CMPSET(&(comm->error_handler), tmp, tmp));
+
+  /* Retain the errhandler, corresponding to object refcount decrease
+     in errhandler_free.c. */
+  *errhandler = tmp;
+  OBJ_RETAIN(tmp);
 
   /* All done */
 
