@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -36,7 +36,9 @@ static const char FUNC_NAME[] = "MPI_Error_string";
 int MPI_Error_string(int errorcode, char *string, int *resultlen) 
 {
     char *tmpstring;
-    
+
+    OPAL_CR_NOOP_PROGRESS();
+
     if ( MPI_PARAM_CHECK ) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
@@ -47,7 +49,7 @@ int MPI_Error_string(int errorcode, char *string, int *resultlen)
     }
  
     tmpstring = ompi_mpi_errnum_get_string (errorcode);
-    strcpy(string, tmpstring);
+    strncpy(string, tmpstring, MPI_MAX_ERROR_STRING);
     *resultlen = (int)strlen(string);
     
     return MPI_SUCCESS;
