@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -50,11 +50,12 @@ int MPI_File_delete(char *filename, MPI_Info info)
         OMPI_ERRHANDLER_CHECK(rc, MPI_FILE_NULL, rc, FUNC_NAME);
     }
 
-    /* Note that MPI-2:9.7 (p265) says that errors in MPI_FILE_OPEN
-       (before the file handle is created) should invoke the default
-       error handler on MPI_FILE_NULL.  Hence, if we get a file handle
-       out of ompi_file_open(), invoke the error handler on that.  If
-       not, invoke the error handler on MPI_FILE_NULL. */
+    /* Note that MPI-2:9.7 (p265 in the ps; 261 in the pdf) says that
+       errors in MPI_FILE_OPEN (before the file handle is created)
+       should invoke the default error handler on MPI_FILE_NULL.
+       Hence, if we get a file handle out of ompi_file_open(), invoke
+       the error handler on that.  If not, invoke the error handler on
+       MPI_FILE_NULL. */
 
     /* The io framework is only initialized lazily.  If it hasn't
        already been initialized, do so now (note that MPI_FILE_OPEN
@@ -73,6 +74,8 @@ int MPI_File_delete(char *filename, MPI_Info info)
         }
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* Since there is no MPI_File handle associated with this
        function, the MCA has to do a selection and perform the
        action */
