@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,6 +21,7 @@
 
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/request/grequest.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Grequest_complete = PMPI_Grequest_complete
@@ -35,22 +37,24 @@ static const char FUNC_NAME[] = "MPI_Grequest_complete";
 int MPI_Grequest_complete(MPI_Request request) 
 {
     int rc = MPI_SUCCESS;
+
+    MEMCHECKER(
+        memchecker_request(&request);
+    );
+
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
-        if (request == MPI_REQUEST_NULL) {
+        if (MPI_REQUEST_NULL == request || NULL == request) {
            rc = MPI_ERR_REQUEST;
+        } else if (OMPI_REQUEST_GEN != request->req_type) {
+            rc = MPI_ERR_REQUEST;
         }
         OMPI_ERRHANDLER_CHECK(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
     }
 
-    switch(request->req_type) {
-        case OMPI_REQUEST_GEN:
-            rc = ompi_grequest_complete((ompi_grequest_t*)request);
-            break;
-        default:
-            rc = MPI_ERR_REQUEST;
-            break;
-    }
+    OPAL_CR_ENTER_LIBRARY();
+
+    rc = ompi_grequest_complete(request);
     OMPI_ERRHANDLER_RETURN(rc, MPI_COMM_WORLD, MPI_ERR_INTERN, FUNC_NAME);
 }
 
