@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -22,6 +23,8 @@
 #endif
 #include <stdio.h>
 
+#include MCA_timer_IMPLEMENTATION_HEADER
+#include "opal/prefetch.h"
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/runtime/mpiruntime.h"
 
@@ -38,10 +41,24 @@ static const char FUNC_NAME[] = "MPI_Wtime";
 
 double MPI_Wtime(void)
 {
-    struct timeval tv;
     double wtime;
+
+#if OPAL_TIMER_USEC_NATIVE
+    /* We may or may not have native usec precision on Windows, so put
+       this #if before the #ifdef checking for Windows. */
+    wtime = ((double) opal_timer_base_get_usec()) / 1000000.0;
+#elif defined(__WINDOWS__)
+    wtime = ((double) opal_timer_base_get_cycles()) /
+        ((double) opal_timer_base_get_freq());
+#else
+    /* Fall back to gettimeofday() if we have nothing else */
+    struct timeval tv;
     gettimeofday(&tv, NULL);
     wtime = tv.tv_sec;
     wtime += (double)tv.tv_usec / 1000000.0;
+#endif
+
+    OPAL_CR_NOOP_PROGRESS();
+
     return wtime;
 }
