@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -26,7 +27,11 @@
 #include <stdio.h>
 
 #if OMPI_WANT_LIBLTDL
-#include "opal/libltdl/ltdl.h"
+  #ifndef __WINDOWS__ 
+    #include "opal/libltdl/ltdl.h" 
+  #else 
+    #include "ltdl.h" 
+  #endif 
 #endif
 
 #include "opal/class/opal_list.h"
@@ -43,7 +48,7 @@
 struct repository_item_t {
   opal_list_item_t super;
 
-  char ri_type[MCA_BASE_MAX_TYPE_NAME_LEN];
+  char ri_type[MCA_BASE_MAX_TYPE_NAME_LEN + 1];
   lt_dlhandle ri_dlhandle;
   const mca_base_component_t *ri_component_struct;
   opal_list_t ri_dependencies;
@@ -85,6 +90,10 @@ static opal_list_t repository;
 static repository_item_t *find_component(const char *type, const char *name);
 static int link_items(repository_item_t *src, repository_item_t *depend);
 
+#if OPAL_HAVE_LTDL_ADVISE
+lt_dladvise opal_mca_dladvise;
+#endif
+
 #endif /* OMPI_WANT_LIBLTDL */
 
 
@@ -103,6 +112,20 @@ int mca_base_component_repository_init(void)
       return OPAL_ERR_OUT_OF_RESOURCE;
     }
 
+#if OPAL_HAVE_LTDL_ADVISE
+    if (lt_dladvise_init(&opal_mca_dladvise)) {
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+
+    if (lt_dladvise_ext(&opal_mca_dladvise)) {
+        return OPAL_ERROR;
+    }
+
+    if (lt_dladvise_global(&opal_mca_dladvise)) {
+        return OPAL_ERROR;
+    }
+#endif
+
     OBJ_CONSTRUCT(&repository, opal_list_t);
 #endif
     initialized = true;
@@ -135,7 +158,8 @@ int mca_base_component_repository_retain(char *type,
 
   /* Initialize the repository item */
 
-  strcpy(ri->ri_type, type);
+  strncpy(ri->ri_type, type, MCA_BASE_MAX_TYPE_NAME_LEN);
+  ri->ri_type[MCA_BASE_MAX_TYPE_NAME_LEN] = '\0';
   ri->ri_dlhandle = component_handle;
   ri->ri_component_struct = component_struct;
 
@@ -209,10 +233,12 @@ int mca_base_component_repository_link(const char *src_type,
 void mca_base_component_repository_release(const mca_base_component_t *component)
 {
 #if OMPI_WANT_LIBLTDL
-  repository_item_t *ri = find_component(component->mca_type_name, 
-                                         component->mca_component_name);
-  if (NULL != ri) {
-    OBJ_RELEASE(ri);
+  if (initialized) {
+    repository_item_t *ri = find_component(component->mca_type_name, 
+                                           component->mca_component_name);
+    if (NULL != ri) {
+      OBJ_RELEASE(ri);
+    }
   }
 #endif
 }
@@ -253,6 +279,12 @@ void mca_base_component_repository_finalize(void)
       }
     } while (opal_list_get_size(&repository) > 0);
 
+#if OPAL_HAVE_LTDL_ADVISE
+    if (lt_dladvise_destroy(&opal_mca_dladvise)) {
+        return;
+    }
+#endif
+
     /* Close down libltdl */
 
     lt_dlexit();
@@ -345,7 +377,6 @@ static void ri_destructor(opal_object_t *obj)
   opal_list_item_t *item;
 
   /* Close the component (and potentially unload it from memory */
-
   lt_dlclose(ri->ri_dlhandle);
 
   /* It should be obvious, but I'll state it anyway because it bit me
