@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -28,7 +29,6 @@
  *
  * - Creating MCA parameters
  * -# Register a parameter, get an index back
- * -# Optionally associate that index with an attribute keyval
  * - Using MCA parameters
  * -# Lookup a "normal" parameter value on a specific index, or
  * -# Lookup an attribute parameter on a specific index and
@@ -40,8 +40,6 @@
  *
  * - An "override" location that is only available to be set via the
  *   mca_base_param API.
- * - If the parameter has an MPI attribute keyval associated with it,
- *   see if there is a value assigned that can be used.
  * - Look for an environment variable corresponding to the MCA
  *   parameter.
  * - See if a file contains the MCA parameter (MCA parameter files are
@@ -55,8 +53,8 @@
  * itself; it should not be required by any other Open MPI entities.
  */
 
-#ifndef OMPI_MCA_BASE_PARAM_H
-#define OMPI_MCA_BASE_PARAM_H
+#ifndef OPAL_MCA_BASE_PARAM_H
+#define OPAL_MCA_BASE_PARAM_H
 
 #include "opal_config.h"
 
@@ -80,6 +78,24 @@ typedef enum {
 } mca_base_param_type_t;
 
 
+/**
+ * Source of an MCA parameter's value
+ */
+typedef enum {
+    /** The default value */
+    MCA_BASE_PARAM_SOURCE_DEFAULT,
+    /** The value came from the environment (or command line!) */
+    MCA_BASE_PARAM_SOURCE_ENV,
+    /** The value came from a file */
+    MCA_BASE_PARAM_SOURCE_FILE,
+    /** The value came a "set" API call */
+    MCA_BASE_PARAM_SOURCE_OVERRIDE,
+
+    /** Maximum source type */
+    MCA_BASE_PARAM_SOURCE_MAX
+} mca_base_param_source_t;
+
+
 /**
  * Struct for holding name/type info.  Used in mca_base_param_dump(),
  * below.
@@ -102,7 +118,20 @@ struct mca_base_param_info_t {
     /** Full, assembled parameter name */
     char *mbpp_full_name;
 
-    /** Is this value changable? */
+    /** Is this parameter deprecated? */
+    bool mbpp_deprecated;
+
+    /** Array of pointers of synonyms of this parameter */
+    struct mca_base_param_info_t **mbpp_synonyms;
+    /** Length of mbpp_synonyms array */
+    int mbpp_synonyms_len;
+    /** Back pointer to another mca_base_param_info_t that *this*
+        param is a synonym of (or NULL) */
+    struct mca_base_param_info_t *mbpp_synonym_parent;
+
+    /** Is this parameter internal? */
+    bool mbpp_internal;
+    /** Is this parameter changable? */
     bool mbpp_read_only;
     /** Help message associated with this parameter */
     char *mbpp_help_msg;
@@ -111,11 +140,6 @@ struct mca_base_param_info_t {
  * Convenience typedef
  */
 typedef struct mca_base_param_info_t mca_base_param_info_t;
-/**
- * Make a real object for the info
- */
-OBJ_CLASS_DECLARATION(mca_base_param_info_t);
-
 
 /*
  * Global functions for MCA
@@ -124,6 +148,12 @@ OBJ_CLASS_DECLARATION(mca_base_param_info_t);
 #if defined(c_plusplus) || defined(__cplusplus)
 extern "C" {
 #endif
+
+    /**
+     * Make a real object for the info
+     */
+    OPAL_DECLSPEC OBJ_CLASS_DECLARATION(mca_base_param_info_t);
+
     /**
      * Initialize the MCA parameter system.
      *
@@ -133,13 +163,23 @@ extern "C" {
      * invoked internally (by mca_base_open()) and is only documented
      * here for completeness.
      */
-    OMPI_DECLSPEC int mca_base_param_init(void);
+    OPAL_DECLSPEC int mca_base_param_init(void);
+
+    /**
+     * Recache the MCA param files
+     *
+     * @param rel_path_search If a relative path is found, search the path even
+     * if the relative path in pointing to the current working directory.
+     * @retval OPAL_SUCCESS
+     *
+     */
+    OPAL_DECLSPEC int mca_base_param_recache_files(bool rel_path_search);
 
     /**
      * Register an integer MCA parameter.
      *
-     * @param component [in] Pointer to the componet for which the
-     * parameter is being registered (string), or NULL.
+     * @param component [in] Pointer to the component for which the
+     * parameter is being registered.
      * @param param_name [in] The name of the parameter being
      * registered (string).
      * @param help_msg [in] A string describing the use and valid
@@ -171,7 +211,7 @@ extern "C" {
      * valid values are.  This message is used in help messages, such
      * as the output from the ompi_info executable.
      *
-     * If {internal} is set to true, this parameter is now shown by
+     * If {internal} is set to true, this parameter is not shown by
      * default in the output of ompi_info.  That is, this parameter is
      * considered internal to the Open MPI implementation and is not
      * supposed to be viewed / changed by the user.
@@ -187,7 +227,7 @@ extern "C" {
      * complete, the parameter system will look up the current value
      * of the parameter and return it in {current_value}.
      */
-    OMPI_DECLSPEC int mca_base_param_reg_int(const mca_base_component_t *component,
+    OPAL_DECLSPEC int mca_base_param_reg_int(const mca_base_component_t *component,
                                              const char *param_name, 
                                              const char *help_msg,
                                              bool internal,
@@ -228,12 +268,28 @@ extern "C" {
      * MPI layer, or the MCA base system framework itself).  Typical
      * "type" strings are:
      *
-     * * "mca": for the MCA base framework itself
-     * * framework name: for any given framework
-     * * "mpi": for parameters that apply to the overall MPI layer
-     * * "orte": for parameters that apply to the overall ORTE layer
+     * "mca": for the MCA base framework itself
+     * framework name: for any given framework
+     * "mpi": for parameters that apply to the overall MPI layer
+     * "orte": for parameters that apply to the overall ORTE layer
+     * "btl": for parameters to the OMPI BTL framework
+     * ...etc.
+     *
+     * Note that the type should always be a framework or a level name
+     * (e.g., "btl" or "mpi") -- it should not include the component
+     * name, even if the component is the base of a framework.  Hence,
+     * "btl_base" is not a valid type name.  Specifically, registering
+     * a parameter with an unrecognized type is not an error, but
+     * ompi_info has a hard-coded list of frameworks and levels;
+     * parameters that have recongized types, although they can be
+     * used by the user, will not be displayed by ompi_info.
+     *
+     * Note that if you use mca_base_param_find() to lookup the index
+     * of the registered parameter, the "component" argument should be
+     * NULL (because it is not specified in this registration
+     * function, and is therefore registered with a NULL value).
      */
-    OMPI_DECLSPEC int mca_base_param_reg_int_name(const char *type,
+    OPAL_DECLSPEC int mca_base_param_reg_int_name(const char *type,
                                                   const char *param_name, 
                                                   const char *help_msg,
                                                   bool internal,
@@ -244,8 +300,8 @@ extern "C" {
     /**
      * Register a string MCA parameter.
      *
-     * @param component [in] Pointer to the componet for which the
-     * parameter is being registered (string), or NULL.
+     * @param component [in] Pointer to the component for which the
+     * parameter is being registered.
      * @param param_name [in] The name of the parameter being
      * registered (string).
      * @param help_msg [in] A string describing the use and valid
@@ -274,7 +330,7 @@ extern "C" {
      * string default value (which is \em not allowed to be NULL).
      * See mca_base_param_reg_int() for all other details.
      */
-    OMPI_DECLSPEC int mca_base_param_reg_string(const mca_base_component_t *component,
+    OPAL_DECLSPEC int mca_base_param_reg_string(const mca_base_component_t *component,
                                                 const char *param_name,
                                                 const char *help_msg,
                                                 bool internal,
@@ -320,12 +376,28 @@ extern "C" {
      * (e.g., the MPI layer, or the MCA base system framework itself).
      * Typical "type" strings are:
      *
-     * * "mca": for the MCA base framework itself
-     * * framework name: for any given framework
-     * * "mpi": for parameters that apply to the overall MPI layer
-     * * "orte": for parameters that apply to the overall ORTE layer
+     * "mca": for the MCA base framework itself
+     * framework name: for any given framework
+     * "mpi": for parameters that apply to the overall MPI layer
+     * "orte": for parameters that apply to the overall ORTE layer
+     * "btl": for parameters to the OMPI BTL framework
+     * ...etc.
+     *
+     * Note that the type should always be a framework or a level name
+     * (e.g., "btl" or "mpi") -- it should not include the component
+     * name, even if the component is the base of a framework.  Hence,
+     * "btl_base" is not a valid type name.  Specifically, registering
+     * a parameter with an unrecognized type is not an error, but
+     * ompi_info has a hard-coded list of frameworks and levels;
+     * parameters that have recongized types, although they can be
+     * used by the user, will not be displayed by ompi_info.
+     *
+     * Note that if you use mca_base_param_find() to lookup the index
+     * of the registered parameter, the "component" argument should be
+     * NULL (because it is not specified in this registration
+     * function, and is therefore registered with a NULL value).
      */
-    OMPI_DECLSPEC int mca_base_param_reg_string_name(const char *type,
+    OPAL_DECLSPEC int mca_base_param_reg_string_name(const char *type,
                                                      const char *param_name,
                                                      const char *help_msg,
                                                      bool internal,
@@ -334,29 +406,77 @@ extern "C" {
                                                      char **current_value);
 
     /**
-     * Associate a communicator/datatype/window keyval with an MCA
-     * parameter.
+     * Register a synonym name for an MCA parameter.
      *
-     * @param index The index of the parameter to use.
-     * @param keyval The keyval to associate it with.
+     * @param original_index [in] The index of the original parameter to
+     * create a synonym for.
+     * @param syn_component [in] Pointer to the component for which the
+     * synonym is being registered.
+     * @param syn_param_name [in] Parameter name of the synonym to be
+     * created (string)
+     * @param deprecated If true, a warning will be shown if this
+     * synonym is used to set the parameter's value (unless the
+     * warnings are silenced)
      *
      * @returns OPAL_SUCCESS Upon success.
-     * @returns OPAL_ERROR If the index value is invalid.
+     * @returns OPAL_ERR_BAD_PARAM If the index value is invalid.
+     * @returns OPAL_ERROR Otherwise
+     * 
+     * Upon success, this function creates a synonym MCA parameter
+     * that will be treated almost exactly like the original.  The
+     * type (int or string) is irrelevant; this function simply
+     * creates a new name that by which the same parameter value is
+     * accessible.  
      *
-     * For an index value that was previously returned by
-     * mca_base_param_register_int() or
-     * mca_base_param_register_string(), the corresponding MCA parameter
-     * can be associated with a communicator, datatype, or window
-     * attribute keyval.  
-     *
-     * After using this function, you can use any of the four lookup
-     * functions (mca_base_param_lookup_int(),
-     * mca_base_param_lookup_string(), mca_base_param_kv_lookup_int(),
-     * and mca_base_param_kv_lookup_string()), but only the "kv"
-     * versions will cross reference and attempt to find parameter
-     * values on attributes.
+     * Note that the original parameter name has precendence over all
+     * synonyms.  For example, consider the case if parameter is
+     * originally registered under the name "A" and is later
+     * registered with synonyms "B" and "C".  If the user sets values
+     * for both MCA parameter names "A" and "B", the value associated
+     * with the "A" name will be used and the value associated with
+     * the "B" will be ignored (and will not even be visible by the
+     * mca_base_param_*() API).  If the user sets values for both MCA
+     * parameter names "B" and "C" (and does *not* set a value for
+     * "A"), it is undefined as to which value will be used.
      */
-    OMPI_DECLSPEC int mca_base_param_kv_associate(int index, int keyval);
+    OPAL_DECLSPEC int mca_base_param_reg_syn(int orignal_index, 
+                                             const mca_base_component_t *syn_component,
+                                             const char *syn_param_name, 
+                                             bool deprecated);
+
+    /**
+     * Register an MCA parameter synonym that is not associated with a
+     * component.
+     *
+     * @param original_index [in] The index of the original parameter to
+     * create a synonym for.
+     * @param type [in] Although this synonym is not associated with
+     * a component, it still must have a string type name that will
+     * act as a prefix (string).
+     * @param syn_param_name [in] Parameter name of the synonym to be
+     * created (string)
+     * @param deprecated If true, a warning will be shown if this
+     * synonym is used to set the parameter's value (unless the
+     * warnings are silenced)
+     *
+     * Essentially the same as mca_base_param_reg_syn(), but using a
+     * type name instead of a component.
+     *
+     * See mca_base_param_reg_int_name() for guidence on type string
+     * values.
+     */
+    OPAL_DECLSPEC int mca_base_param_reg_syn_name(int orignal_index, 
+                                                  const char *syn_type,
+                                                  const char *syn_param_name, 
+                                                  bool deprecated);
+
+    /**
+     * Deregister a MCA parameter
+     *
+     * @param index Index returned from mca_base_param_register_init()
+     *
+     */
+    OPAL_DECLSPEC int mca_base_param_deregister(int index);
 
     /**
      * Look up an integer MCA parameter.
@@ -374,31 +494,7 @@ extern "C" {
      * The value of a specific MCA parameter can be looked up using the
      * return value from mca_base_param_register_int().
      */
-    OMPI_DECLSPEC int mca_base_param_lookup_int(int index, int *value);
-    
-    /**
-     * Look up an integer MCA parameter, to include looking in
-     * attributes.
-     *
-     * @param index Index previous returned from
-     * mca_base_param_register_int().
-     * @param attrs Object containing attributes to be searched.
-     * @param value Pointer to int where the parameter value will
-     * be stored.
-     *
-     * @return OPAL_ERROR Upon failure.  The contents of value are
-     * undefined.
-     * @return OPAL_SUCCESS Upon success.  value will be filled with the
-     * parameter's current value.
-     *
-     * This function is identical to mca_base_param_lookup_int() except
-     * that it looks in attributes \em first to find the parameter
-     * value.  The function mca_base_param_kv_associate() must have been
-     * called first to associate a keyval with the index.
-     */
-    OMPI_DECLSPEC int mca_base_param_kv_lookup_int(int index,
-                                                   struct opal_hash_table_t *attrs, 
-                                                   int *value);
+    OPAL_DECLSPEC int mca_base_param_lookup_int(int index, int *value);
     
     /**
      * Look up a string MCA parameter.
@@ -423,30 +519,25 @@ extern "C" {
      * The value of a specific MCA parameter can be looked up using the
      * return value from mca_base_param_register_string().
      */
-    OMPI_DECLSPEC int mca_base_param_lookup_string(int index, char **value);
+    OPAL_DECLSPEC int mca_base_param_lookup_string(int index, char **value);
 
     /**
-     * Look up a string MCA parameter, to include looking in attributes.
+     * Lookup the source of an MCA parameter's value
      *
-     * @param index [in] Index previous returned from
-     * mca_base_param_register_string().
-     * @param attrs [in] Object containing attributes to be searched.
-     * @param value [out] Pointer to (char *) where the parameter value
-     * will be stored.
+     * @param index [in] Index of MCA parameter to set
+     * @param source [out] Enum value indicating source
+     * @param source_file [out] If value came from source, name of the file
+     * that set it.  The caller should not modify or free this string.
      *
-     * @return OPAL_ERROR Upon failure.  The contents of value are
-     * undefined.
-     * @return OPAL_SUCCESS Upon success.  value will be filled with the
-     * parameter's current value.
+     * @retval OPAL_ERROR If the parameter was not found.
+     * @retval OPAL_SUCCESS Upon success.
      *
-     * This function is identical to mca_base_param_lookup_string()
-     * except that it looks in attributes \em first to find the
-     * parameter value.  The function mca_base_param_kv_associate() must
-     * have been called first to associate a keyval with the index.
+     * This function looks up to see where the value of an MCA
+     * parameter came from.
      */
-    OMPI_DECLSPEC int mca_base_param_kv_lookup_string(int index, 
-                                                      struct opal_hash_table_t *attrs, 
-                                                      char **value);
+    OPAL_DECLSPEC bool mca_base_param_lookup_source(int index, 
+                                                    mca_base_param_source_t *source,
+                                                    char **source_file);
 
     /**
      * Sets an "override" value for an integer MCA parameter.
@@ -466,7 +557,7 @@ extern "C" {
      * This function may be invoked multiple times; each time, the
      * last "set" value is replaced with the newest value.
      */
-    OMPI_DECLSPEC int mca_base_param_set_int(int index, int value);
+    OPAL_DECLSPEC int mca_base_param_set_int(int index, int value);
 
     /**
      * Sets an "override" value for an string MCA parameter.
@@ -490,7 +581,7 @@ extern "C" {
      * last "set" value is replaced with the newest value (the old
      * value is discarded).
      */
-    OMPI_DECLSPEC int mca_base_param_set_string(int index, char *value);
+    OPAL_DECLSPEC int mca_base_param_set_string(int index, char *value);
 
     /**
      * Unset a parameter that was previously set by
@@ -504,7 +595,7 @@ extern "C" {
      * Resets previous value that was set (if any) on the given MCA
      * parameter.
      */
-    OMPI_DECLSPEC int mca_base_param_unset(int index);
+    OPAL_DECLSPEC int mca_base_param_unset(int index);
 
     /**
      * Get the string name corresponding to the MCA parameter
@@ -519,7 +610,7 @@ extern "C" {
      * The string that is returned is owned by the caller; if
      * appropriate, it must be eventually freed by the caller.
      */
-    OMPI_DECLSPEC char *mca_base_param_env_var(const char *param_name);
+    OPAL_DECLSPEC char *mca_base_param_env_var(const char *param_name);
 
     /**
      * Find the index for an MCA parameter based on its names.
@@ -540,7 +631,7 @@ extern "C" {
      * can be used with mca_base_param_lookup_int() and
      * mca_base_param_lookup_string().
      */
-    OMPI_DECLSPEC int mca_base_param_find(const char *type, 
+    OPAL_DECLSPEC int mca_base_param_find(const char *type, 
                                           const char *component, 
                                           const char *param);
 
@@ -563,7 +654,7 @@ extern "C" {
      * MPI_INIT (at least, they're not displayed by default), thus
      * keeping them away from prying user eyes.
      */
-    OMPI_DECLSPEC int mca_base_param_set_internal(int index, bool internal);
+    OPAL_DECLSPEC int mca_base_param_set_internal(int index, bool internal);
 
     /**
      * Obtain a list of all the MCA parameters currently defined as
@@ -589,7 +680,7 @@ extern "C" {
      * mca_base_param_dump_release() when finished with the returned
      * info list to release all associated memory.
      */
-    OMPI_DECLSPEC int mca_base_param_dump(opal_list_t **info, bool internal);
+    OPAL_DECLSPEC int mca_base_param_dump(opal_list_t **info, bool internal);
 
     /**
      * Obtain a list of all the MCA parameters currently defined as
@@ -609,7 +700,7 @@ extern "C" {
      * its output is in terms of an argv-style array of key=value
      * strings, suitable for using in an environment.
      */
-    OMPI_DECLSPEC int mca_base_param_build_env(char ***env, int *num_env,
+    OPAL_DECLSPEC int mca_base_param_build_env(char ***env, int *num_env,
                                                bool internal);
 
     /**
@@ -629,7 +720,7 @@ extern "C" {
      * the caller is finished with the info list, invoke this
      * function and all memory associated with the list will be freed.
      */
-    OMPI_DECLSPEC int mca_base_param_dump_release(opal_list_t *info);
+    OPAL_DECLSPEC int mca_base_param_dump_release(opal_list_t *info);
 
     /**
      * Shut down the MCA parameter system (normally only invoked by the
@@ -645,7 +736,7 @@ extern "C" {
      * when the process is shutting down (e.g., during MPI_FINALIZE).  It
      * is only documented here for completeness.
      */
-    OMPI_DECLSPEC int mca_base_param_finalize(void);
+    OPAL_DECLSPEC int mca_base_param_finalize(void);
 
     /***************************************************************
      * Deprecated interface
@@ -697,11 +788,11 @@ extern "C" {
      * returned, but the default value will be changed to reflect the
      * last registration.
      */
-    OMPI_DECLSPEC int mca_base_param_register_int(const char *type_name, 
+    OPAL_DECLSPEC int mca_base_param_register_int(const char *type_name, 
                                                   const char *component_name,
                                                   const char *param_name, 
                                                   const char *mca_param_name,
-                                                  int default_value);
+                                                  int default_value) /* __opal_attribute_deprecated__ */;
     
     /**
      * \deprecated
@@ -734,11 +825,11 @@ extern "C" {
      * associated string default value (which is \em not allowed to be NULL).
      * See mca_base_param_register_int() for all other details.
      */
-    OMPI_DECLSPEC int mca_base_param_register_string(const char *type_name, 
+    OPAL_DECLSPEC int mca_base_param_register_string(const char *type_name, 
                                                      const char *component_name,
                                                      const char *param_name, 
                                                      const char *mca_param_name,
-                                                     const char *default_value);
+                                                     const char *default_value) /* __opal_attribute_deprecated__ */;
 
     /**
      * \deprecated
@@ -760,12 +851,12 @@ extern "C" {
      * The string that is returned is owned by the caller; if
      * appropriate, it must be eventually freed by the caller.
      */
-    OMPI_DECLSPEC char *mca_base_param_environ_variable(const char *type,
+    OPAL_DECLSPEC char *mca_base_param_environ_variable(const char *type,
                                                         const char *comp,
-                                                        const char *param);
+                                                        const char *param) /* __opal_attribute_deprecated__ */;
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
 #endif
 
-#endif /* OMPI_MCA_BASE_PARAM_H */
+#endif /* OPAL_MCA_BASE_PARAM_H */
