@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -50,7 +51,7 @@ int  __munmap(void* addr, size_t len);
 OPAL_DECLSPEC int 
 munmap(void* addr, size_t len)
 {
-    return opal_mem_free_ptmalloc2_munmap(addr, len, 0);
+    return opal_mem_free_ptmalloc2_munmap(addr, len, 0, 1);
 }
 
 
@@ -59,36 +60,43 @@ munmap(void* addr, size_t len)
    possible, try calling __munmap from munmap and let __munmap go.  If
    that doesn't work, try dlsym */
 int
-opal_mem_free_ptmalloc2_munmap(void *start, size_t length, int from_alloc)
+opal_mem_free_ptmalloc2_munmap(void *start, size_t length, int from_alloc,
+                               int call_hooks)
 {
-#if !defined(HAVE___MUNMAP) && \
-    !(defined(HAVE_SYSCALL) && defined(__NR_munmap)) && defined(HAVE_DLSYM)
-    static int (*realmunmap)(void*, size_t);
-#endif
-
     {
       extern bool opal_memory_ptmalloc2_munmap_invoked;
       opal_memory_ptmalloc2_munmap_invoked = true;
     }
 
-    opal_mem_hooks_release_hook(start, length, from_alloc);
+    if (call_hooks) {
+        opal_mem_hooks_release_hook(start, length, from_alloc);
+    }
 
 #if defined(HAVE___MUNMAP)
     return __munmap(start, length);
 #elif defined(HAVE_SYSCALL) && defined(__NR_munmap)
     return syscall(__NR_munmap, start, length);
 #elif defined(HAVE_DLSYM)
-    if (NULL == realmunmap) {
-        union { 
-            int (*munmap_fp)(void*, size_t);
-            void *munmap_p;
-        } tmp;
+    {
+        /* Must use a typedef here because we need volatile to be an
+           attribute of the variable, not the function (which would be
+           meaningless, anyway). */
+        typedef int (*munmap_fn_t)(void*, size_t);
+        static volatile munmap_fn_t realmunmap = NULL;
 
-        tmp.munmap_p = dlsym(RTLD_NEXT, "munmap");
-        realmunmap = tmp.munmap_fp;
+        if (NULL == realmunmap) {
+            union { 
+                int (*munmap_fp)(void*, size_t);
+                void *munmap_p;
+            } tmp;
+        
+            tmp.munmap_p = dlsym(RTLD_NEXT, "munmap");
+            realmunmap = tmp.munmap_fp;
+            ++count;
+        }
+
+        return realmunmap(start, length);
     }
-
-    return realmunmap(start, length);
 #else
     #error "Can not determine how to call munmap"
 #endif
