@@ -5,11 +5,12 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2009 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -113,6 +114,7 @@ struct opal_if_t {
 };
 typedef struct opal_if_t opal_if_t;
 
+static OBJ_CLASS_INSTANCE(opal_if_t, opal_list_item_t, NULL, NULL);
 static opal_list_t opal_if_list;
 static bool already_done = false;
 static bool do_not_resolve = false;
@@ -205,9 +207,8 @@ static int opal_ifinit(void)
             return OPAL_ERROR;
         }
 
-        /* initialize the memory so valgrind and purify won't complain
-         * can't use OMPI_DEBUG_ZERO because sizeof(ifconf.ifc_req)
-         * isn't useful.  Since this isn't performance critical, just
+        /* initialize the memory so valgrind and purify won't
+         * complain.  Since this isn't performance critical, just
          * always memset.
          */
         memset(ifconf.ifc_req, 0, ifconf.ifc_len);
@@ -256,12 +257,16 @@ static int opal_ifinit(void)
     /* loop through all interfaces */
     while (rem > 0) {
         struct ifreq* ifr = (struct ifreq*) ptr;
-        opal_if_t intf;
-        opal_if_t *intf_ptr;
+        opal_if_t *intf;
         int length;
 
-        OMPI_DEBUG_ZERO(intf);
-        OBJ_CONSTRUCT(&intf, opal_list_item_t);
+        intf = OBJ_NEW(opal_if_t);
+        if (NULL == intf) {
+            opal_output(0, "opal_ifinit: unable to allocated %lu bytes\n", (unsigned long)sizeof(opal_if_t));
+            free(ifconf.ifc_req);
+            close(sd);
+            return OPAL_ERR_OUT_OF_RESOURCE;
+        }
 
         /* compute offset for entries */
 #ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
@@ -302,26 +307,26 @@ static int opal_ifinit(void)
 #endif
 
         /* copy entry over into our data structure */
-        strcpy(intf.if_name, ifr->ifr_name);
-        intf.if_flags = ifr->ifr_flags;
+        strcpy(intf->if_name, ifr->ifr_name);
+        intf->if_flags = ifr->ifr_flags;
 
         /* every new address gets its own internal if_index */
-        intf.if_index = opal_list_get_size(&opal_if_list)+1;
+        intf->if_index = opal_list_get_size(&opal_if_list)+1;
 
         /* assign the kernel index to distinguish different NICs */
 #ifndef SIOCGIFINDEX
-        intf.if_kernel_index = intf.if_index;
+        intf->if_kernel_index = intf->if_index;
 #else
         if(ioctl(sd, SIOCGIFINDEX, ifr) < 0) {
             opal_output(0,"opal_ifinit: ioctl(SIOCGIFINDEX) failed with errno=%d", errno);
             continue;
         }
 #if defined(ifr_ifindex)
-        intf.if_kernel_index = ifr->ifr_ifindex;
+        intf->if_kernel_index = ifr->ifr_ifindex;
 #elif defined(ifr_index)
-        intf.if_kernel_index = ifr->ifr_index;
+        intf->if_kernel_index = ifr->ifr_index;
 #else
-        intf.if_kernel_index = -1;
+        intf->if_kernel_index = -1;
 #endif
 #endif /* SIOCGIFINDEX */
 
@@ -335,7 +340,7 @@ static int opal_ifinit(void)
         }
 
         /* based on above, we know this is an IPv4 address... */
-        memcpy(&intf.if_addr, &ifr->ifr_addr, sizeof(struct sockaddr_in));
+        memcpy(&intf->if_addr, &ifr->ifr_addr, sizeof(struct sockaddr_in));
 
         if(ioctl(sd, SIOCGIFNETMASK, ifr) < 0) {
             opal_output(0, "opal_ifinit: ioctl(SIOCGIFNETMASK) failed with errno=%d", errno);
@@ -343,18 +348,9 @@ static int opal_ifinit(void)
         }
 
         /* generate CIDR and assign to netmask */
-        intf.if_mask = prefix(((struct sockaddr_in*) &ifr->ifr_addr)->sin_addr.s_addr);
+        intf->if_mask = prefix(((struct sockaddr_in*) &ifr->ifr_addr)->sin_addr.s_addr);
 
-        intf_ptr = (opal_if_t*) malloc(sizeof(opal_if_t));
-        OMPI_DEBUG_ZERO(*intf_ptr);
-        if(intf_ptr == 0) {
-            opal_output(0, "opal_ifinit: unable to allocated %lu bytes\n", (unsigned long)sizeof(opal_if_t));
-            free(ifconf.ifc_req);
-            close(sd);
-            return OPAL_ERR_OUT_OF_RESOURCE;
-        }
-        memcpy(intf_ptr, &intf, sizeof(intf));
-        opal_list_append(&opal_if_list, (opal_list_item_t*)intf_ptr);
+        opal_list_append(&opal_if_list, &(intf->super));
     }
     free(ifconf.ifc_req);
     close(sd);
@@ -377,49 +373,41 @@ static int opal_ifinit(void)
                       &addrbyte[8], &addrbyte[9], &addrbyte[10], &addrbyte[11],
                       &addrbyte[12], &addrbyte[13], &addrbyte[14], &addrbyte[15],
                       &idx, &pfxlen, &scope, &dadstat, ifname) != EOF) {
-                opal_if_t intf;
-                opal_if_t *intf_ptr;
+                opal_if_t *intf;
 
                 /* we don't want any other scope than global */
                 if (scope != 0) {
                    continue;
                 }
 
-                OMPI_DEBUG_ZERO(intf);
-                OBJ_CONSTRUCT(&intf, opal_list_item_t);
+                intf = OBJ_NEW(opal_if_t);
+                if (NULL == intf) {
+                    opal_output(0, "opal_ifinit: unable to allocate %lu bytes\n",
+                                (unsigned long)sizeof(opal_if_t));
+                    fclose(f);
+                    return OPAL_ERR_OUT_OF_RESOURCE;
+                }
         
                 for (iter = 0; iter < 16; iter++) {
                     a6.s6_addr[iter] = addrbyte[iter];
                 }
 
                 /* now construct the opal_if_t */
-                strncpy(intf.if_name, ifname, IF_NAMESIZE);
-                intf.if_index = opal_list_get_size(&opal_if_list)+1;
-                intf.if_kernel_index = (uint16_t) idx;
-                ((struct sockaddr_in6*) &intf.if_addr)->sin6_addr = a6;
-                ((struct sockaddr_in6*) &intf.if_addr)->sin6_family = AF_INET6;
-                ((struct sockaddr_in6*) &intf.if_addr)->sin6_scope_id = scope;
-                intf.if_mask = pfxlen;
+                strncpy(intf->if_name, ifname, IF_NAMESIZE);
+                intf->if_index = opal_list_get_size(&opal_if_list)+1;
+                intf->if_kernel_index = (uint16_t) idx;
+                ((struct sockaddr_in6*) &intf->if_addr)->sin6_addr = a6;
+                ((struct sockaddr_in6*) &intf->if_addr)->sin6_family = AF_INET6;
+                ((struct sockaddr_in6*) &intf->if_addr)->sin6_scope_id = scope;
+                intf->if_mask = pfxlen;
                 if (OPAL_SUCCESS == opal_ifindextoflags(opal_ifnametoindex (ifname), &flag)) {
-                    intf.if_flags = flag;
+                    intf->if_flags = flag;
                 } else {
-                    intf.if_flags = IFF_UP;
+                    intf->if_flags = IFF_UP;
                 }
                 
-                /* copy new interface information to heap and append
-                   to list
-                */
-                intf_ptr = (opal_if_t*) malloc(sizeof(opal_if_t));
-                OMPI_DEBUG_ZERO(*intf_ptr);
-                if(NULL == intf_ptr) {
-                    opal_output(0, "opal_ifinit: unable to allocate %lu bytes\n",
-                                    (unsigned long)sizeof(opal_if_t));
-                    OBJ_DESTRUCT(&intf);
-                    fclose(f);
-                    return OPAL_ERR_OUT_OF_RESOURCE;
-                }
-                memcpy(intf_ptr, &intf, sizeof(intf));
-                opal_list_append(&opal_if_list, (opal_list_item_t*)intf_ptr);
+                /* append to list */
+                opal_list_append(&opal_if_list, &(intf->super));
             } /* of while */
             fclose(f);
         }
@@ -466,8 +454,7 @@ static int opal_ifinit(void)
         for(cur_ifaddrs = *ifadd_list; NULL != cur_ifaddrs; 
                 cur_ifaddrs = cur_ifaddrs->ifa_next) {
 
-            opal_if_t intf;
-            opal_if_t *intf_ptr;
+            opal_if_t *intf;
             struct in6_addr a6;
 
             /* skip non-ipv6 interface addresses */
@@ -497,6 +484,12 @@ static int opal_ifinit(void)
                 continue;
             }
 
+            intf = OBJ_NEW(opal_if_t);
+            if (NULL == intf) {
+                opal_output(0, "opal_ifinit: unable to allocate %lu bytes\n",
+                            sizeof(opal_if_t));
+                return OPAL_ERR_OUT_OF_RESOURCE;
+            }
             sin_addr = (struct sockaddr_in6 *) cur_ifaddrs->ifa_addr;
 
             /* 
@@ -519,8 +512,6 @@ static int opal_ifinit(void)
                 continue;
             }
 
-            OMPI_DEBUG_ZERO(intf);
-            OBJ_CONSTRUCT(&intf, opal_list_item_t);
 #if 0
             char *addr_name = (char *) malloc(48*sizeof(char));
             inet_ntop(AF_INET6, &sin_addr->sin6_addr, addr_name, 48*sizeof(char));
@@ -532,39 +523,29 @@ static int opal_ifinit(void)
             /* fill values into the opal_if_t */
             memcpy(&a6, &(sin_addr->sin6_addr), sizeof(struct in6_addr));
             
-            strncpy(intf.if_name, cur_ifaddrs->ifa_name, IF_NAMESIZE);
-            intf.if_index = opal_list_get_size(&opal_if_list) + 1;
-            ((struct sockaddr_in6*) &intf.if_addr)->sin6_addr = a6;
-            ((struct sockaddr_in6*) &intf.if_addr)->sin6_family = AF_INET6;
+            strncpy(intf->if_name, cur_ifaddrs->ifa_name, IF_NAMESIZE);
+            intf->if_index = opal_list_get_size(&opal_if_list) + 1;
+            ((struct sockaddr_in6*) &intf->if_addr)->sin6_addr = a6;
+            ((struct sockaddr_in6*) &intf->if_addr)->sin6_family = AF_INET6;
 
             /* since every scope != 0 is ignored, we just set the scope to 0 */
-            ((struct sockaddr_in6*) &intf.if_addr)->sin6_scope_id = 0;
+            ((struct sockaddr_in6*) &intf->if_addr)->sin6_scope_id = 0;
 
             /*
              * hardcoded netmask, adrian says that's ok
              */
-            intf.if_mask = 64;
-            intf.if_flags = cur_ifaddrs->ifa_flags;
+            intf->if_mask = 64;
+            intf->if_flags = cur_ifaddrs->ifa_flags;
 
             /*
              * FIXME: figure out how to gain access to the kernel index
              * (or create our own), getifaddrs() does not contain such
              * data
              */
+            intf->if_kernel_index =
+                (uint16_t) if_nametoindex(cur_ifaddrs->ifa_name);
 
-            intf.if_kernel_index = (uint16_t) if_nametoindex(cur_ifaddrs->ifa_name);
-
-            intf_ptr = (opal_if_t*) malloc(sizeof(opal_if_t));
-            OMPI_DEBUG_ZERO(*intf_ptr);
-            if(NULL == intf_ptr) {
-                opal_output(0, "opal_ifinit: unable to allocate %lu bytes\n",
-                            sizeof(opal_if_t));
-                OBJ_DESTRUCT(&intf);
-                return OPAL_ERR_OUT_OF_RESOURCE;
-            }
-            memcpy(intf_ptr, &intf, sizeof(intf));
-            opal_list_append(&opal_if_list, (opal_list_item_t*) intf_ptr);
-            OBJ_DESTRUCT(&intf);
+            opal_list_append(&opal_if_list, &(intf->super));
         }   /*  of for loop over ifaddrs list */
 
     }
@@ -657,31 +638,25 @@ static int opal_ifinit(void)
                 if ( (! IN6_IS_ADDR_LOOPBACK (&my_addr->sin6_addr)) &&
                      (! IN6_IS_ADDR_LINKLOCAL (&my_addr->sin6_addr))) {
                     /* create interface for newly found address */
-                    opal_if_t intf, *intf_ptr;
+                    opal_if_t *intf;
 
-                    OMPI_DEBUG_ZERO(intf);
-                    OBJ_CONSTRUCT(&intf, opal_list_item_t);
-
-                    strncpy (intf.if_name, lifreq->lifr_name, IF_NAMESIZE);
-                    intf.if_index = opal_list_get_size(&opal_if_list)+1;
-                    memcpy (&intf.if_addr, my_addr, sizeof (*my_addr));
-                    intf.if_mask = 64;
-                    /* lifrq flags are uint64_t */
-                    intf.if_flags = (uint32_t)(0x00000000ffffffff) & lifquery.lifr_flags;
-
-                    /* copy new interface to heap and append to list */
-                    intf_ptr = (opal_if_t*) malloc (sizeof (opal_if_t));
-                    OMPI_DEBUG_ZERO (*intf_ptr);
-                    if (NULL == intf_ptr) {
+                    intf = OBJ_NEW(opal_if_t);
+                    if (NULL == intf) {
                         opal_output (0,
                                      "opal_ifinit: unable to allocate %d bytes\n",
                                      sizeof (opal_if_t));
-                        OBJ_DESTRUCT(&intf);
                         return OPAL_ERR_OUT_OF_RESOURCE;
                     }
-                    memcpy (intf_ptr, &intf, sizeof (intf));
-                    opal_list_append (&opal_if_list, (opal_list_item_t*) intf_ptr);
-                    OBJ_DESTRUCT(&intf);
+
+                    strncpy (intf->if_name, lifreq->lifr_name, IF_NAMESIZE);
+                    intf->if_index = opal_list_get_size(&opal_if_list)+1;
+                    memcpy (&intf->if_addr, my_addr, sizeof (*my_addr));
+                    intf->if_mask = 64;
+                    /* lifrq flags are uint64_t */
+                    intf->if_flags = (uint32_t)(0x00000000ffffffff) & lifquery.lifr_flags;
+
+                    /* append to list */
+                    opal_list_append (&opal_if_list, &(intf->super));
                 }
             }
         } /* for */
@@ -712,15 +687,14 @@ static int opal_ifinit(void)
         unsigned long num_bytes_returned;
         int i;
         unsigned int interface_counter = 0;
-        opal_if_t intf;
-        opal_if_t *intf_ptr;
+        opal_if_t *intf;
 
         /* return if this has been done before */
         if (already_done) {
             return OPAL_SUCCESS;
         }
         already_done = true;
-  
+
         /* create a socket */
         sd = WSASocket (AF_INET, SOCK_DGRAM, IPPROTO_UDP, NULL, 0, 0);
         if (sd == SOCKET_ERROR) {
@@ -748,47 +722,42 @@ static int opal_ifinit(void)
         /* loop through all the interfaces and create the list */
         num_interfaces = num_bytes_returned / sizeof (INTERFACE_INFO);
         for (i = 0; i < num_interfaces; ++i) {
-            /* do all this only if the interface is up */
-            if (if_list[i].iiFlags & IFF_UP) {
+            /* do all this only if the interface is up, and skip loopback interface */ 
+            if (0 != (if_list[i].iiFlags & IFF_UP) &&
+                0 == (if_list[i].iiFlags & IFF_LOOPBACK)) {
 
-                OBJ_CONSTRUCT (&intf, opal_list_item_t);
-        
-                /* fill in the interface address */ 
-                memcpy (&intf.if_addr, &(if_list[i].iiAddress), sizeof(intf.if_addr));
-
-                /* fill in the netmask information */
-                memcpy (&intf.if_mask, &(if_list[i].iiNetmask), sizeof(intf.if_mask));
-
-                /* fill in the bcast address */
-                memcpy (&intf.if_bcast, &(if_list[i].iiBroadcastAddress), sizeof(intf.if_bcast));
-
-                /* fill in the flags */
-                intf.if_flags = if_list[i].iiFlags;
-
-                /* fill in the index in the table */
-                intf.if_index = opal_list_get_size(&opal_if_list)+1;
-
-                /* generate the interface name on your own ....
-                   loopback: lo
-                   Rest:    eth0, eth1, ..... */
-
-                if (if_list[i].iiFlags & IFF_LOOPBACK) {
-                    sprintf (intf.if_name, "lo");
-                } else {
-                    sprintf (intf.if_name, "eth%u", interface_counter++);
-                }
-
-                /* copy all this into a persistent form and store it in the list */
-                intf_ptr = (opal_if_t *) malloc(sizeof(opal_if_t));
-                if (NULL == intf_ptr) {
+                intf = OBJ_NEW(opal_if_t);
+                if (NULL == intf) {
                     opal_output (0,"opal_ifinit: Unable to malloc %d bytes",sizeof(opal_list_t));
-                    OBJ_DESTRUCT(&intf);
                     return OPAL_ERR_OUT_OF_RESOURCE;
                 }
+        
+                /* fill in the interface address */ 
+                memcpy(&intf->if_addr, &(if_list[i].iiAddress),
+                       sizeof(intf->if_addr));
 
-                memcpy (intf_ptr, &intf, sizeof(intf));
-                opal_list_append(&opal_if_list, (opal_list_item_t *)intf_ptr);
-                OBJ_DESTRUCT(&intf);
+                /* fill in the netmask information */
+                memcpy(&intf->if_mask, &(if_list[i].iiNetmask),
+                       sizeof(intf->if_mask));
+
+                /* fill in the bcast address */
+                memcpy(&intf->if_bcast, &(if_list[i].iiBroadcastAddress),
+                       sizeof(intf->if_bcast));
+
+                /* fill in the flags */
+                intf->if_flags = if_list[i].iiFlags;
+
+                /* fill in the index in the table */
+                intf->if_index = opal_list_get_size(&opal_if_list)+1;
+
+                /* fill in the kernel index */
+                intf->if_kernel_index = intf->if_index;
+
+                /* generate the interface name, e.g. eth0, eth1, ..... */ 
+                sprintf(intf->if_name, "eth%u", interface_counter++); 
+
+                /* append to list */
+                opal_list_append(&opal_if_list, &(intf->super));
             }
         }
     }
