@@ -1,4 +1,3 @@
-/* @file */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,6 +9,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -17,20 +19,27 @@
  * $HEADER$
  */
 
+/* @file */
+
 #ifndef OPAL_IF_UTIL_
 #define OPAL_IF_UTIL_
 
-#include "opal_config.h"
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
 #endif
 
+#ifndef IF_NAMESIZE
+#define IF_NAMESIZE 32
+#endif
+
+BEGIN_C_DECLS
+
 /**
  *  Lookup an interface by name and return its primary address.
  *  
@@ -39,7 +48,8 @@ extern "C" {
  *  @param size    (IN)   Interface address buffer size
  */
 OPAL_DECLSPEC int opal_ifnametoaddr(const char* if_name, 
-                                    struct sockaddr* if_addr, int size);
+                                    struct sockaddr* if_addr,
+                                    int size);
 
 /**
  *  Lookup an interface by address and return its name.
@@ -51,13 +61,29 @@ OPAL_DECLSPEC int opal_ifnametoaddr(const char* if_name,
 OPAL_DECLSPEC int opal_ifaddrtoname(const char* if_addr, 
                                     char* if_name, int size);
 
+/**
+ *  Lookup an interface by name and return its opal_list index.
+ *  
+ *  @param if_name (IN)  Interface name
+ *  @return              Interface opal_list index
+ */
+OPAL_DECLSPEC int opal_ifnametoindex(const char* if_name);
+
 /**
  *  Lookup an interface by name and return its kernel index.
  *  
  *  @param if_name (IN)  Interface name
- *  @return              Interface index
+ *  @return              Interface kernel index
  */
-OPAL_DECLSPEC int opal_ifnametoindex(const char* if_name);
+OPAL_DECLSPEC int16_t opal_ifnametokindex(const char* if_name);
+
+/**
+ *  Lookup an interface by opal_list index and return its kernel index.
+ *  
+ *  @param if_name (IN)  Interface opal_list index
+ *  @return              Interface kernel index
+ */
+OPAL_DECLSPEC int opal_ifindextokindex(int if_index);
 
 /**
  *  Returns the number of available interfaces.
@@ -88,13 +114,23 @@ OPAL_DECLSPEC int opal_ifnext(int if_index);
 OPAL_DECLSPEC int opal_ifindextoname(int if_index, char* if_name, int);
 
 /**
- *  Lookup an interface by index and return its primary address .
+ *  Lookup an interface by kernel index and return its name.
+ *
+ *  @param if_index (IN)  Interface kernel index
+ *  @param if_name (OUT)  Interface name buffer
+ *  @param size (IN)      Interface name buffer size
+ */
+OPAL_DECLSPEC int opal_ifkindextoname(int if_kindex, char* if_name, int);
+
+/**
+ *  Lookup an interface by index and return its primary address.
  *
  *  @param if_index (IN)  Interface index
  *  @param if_name (OUT)  Interface address buffer
  *  @param size (IN)      Interface address buffer size
  */
-OPAL_DECLSPEC int opal_ifindextoaddr(int if_index, struct sockaddr*, int);
+OPAL_DECLSPEC int opal_ifindextoaddr(int if_index, struct sockaddr*,
+                                     unsigned int);
 
 /**
  *  Lookup an interface by index and return its network mask.
@@ -103,7 +139,15 @@ OPAL_DECLSPEC int opal_ifindextoaddr(int if_index, struct sockaddr*, int);
  *  @param if_name (OUT)  Interface address buffer
  *  @param size (IN)      Interface address buffer size
  */
-OPAL_DECLSPEC int opal_ifindextomask(int if_index, struct sockaddr*, int);
+OPAL_DECLSPEC int opal_ifindextomask(int if_index, uint32_t*, int);
+
+/**
+ *  Lookup an interface by index and return its flags.
+ *
+ *  @param if_index (IN)  Interface index
+ *  @param if_flags (OUT) Interface flags
+ */
+OPAL_DECLSPEC int opal_ifindextoflags(int if_index, uint32_t*);
 
 /**
  * Determine if given hostname / IP address is a local address
@@ -111,20 +155,7 @@ OPAL_DECLSPEC int opal_ifindextomask(int if_index, struct sockaddr*, int);
  * @param hostname (IN)    Hostname (or stringified IP address)
  * @return                 true if \c hostname is local, false otherwise
  */
-OPAL_DECLSPEC bool opal_ifislocal(char *hostname);
-
-/**
- * Determine if given IP address is in the localhost range
- *
- * Determine if the given IP address is in the localhost range
- * (127.0.0.0/8), meaning that it can't be used to connect to machines
- * outside the current host.
- *
- * @param addr             struct sockaddr_in of IP address
- * @return                 true if \c addr is a localhost address,
- *                         false otherwise.
- */
-OPAL_DECLSPEC bool opal_ifislocalhost(struct sockaddr *addr);
+OPAL_DECLSPEC bool opal_ifislocal(const char *hostname);
 
 /**
  * Finalize the functions to release malloc'd data
@@ -135,9 +166,7 @@ OPAL_DECLSPEC bool opal_ifislocalhost(struct sockaddr *addr);
  */
 OPAL_DECLSPEC int opal_iffinalize(void);
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif
 
