@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2009      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -36,6 +37,7 @@
 #include "opal/util/os_path.h"
 #include "opal/util/malloc.h"
 #include "opal/util/basename.h"
+#include "opal/mca/paffinity/base/base.h"
 
 #include "orte/util/show_help.h"
 #include "orte/mca/rml/base/base.h"
@@ -103,6 +105,7 @@ static int rte_init(char flags)
     orte_job_t *jdata;
     orte_node_t *node;
     orte_proc_t *proc;
+    int value;
     
     /* initialize the global list of local children and job data */
     OBJ_CONSTRUCT(&orte_local_children, opal_list_t);
@@ -114,6 +117,34 @@ static int rte_init(char flags)
         goto error;
     }
     
+    /* if we are using xml for output, put a basename start tag */
+    if (orte_xml_output) {
+        fprintf(orte_xml_fp, "<%s>\n", orte_cmd_basename);
+        fflush(orte_xml_fp);
+    }
+    
+    /* determine the topology info */
+    if (0 == orte_default_num_sockets_per_board) {
+        /* we weren't given a number, so try to determine it */
+        if (OPAL_SUCCESS != opal_paffinity_base_get_socket_info(&value)) {
+            /* can't get any info - default to 1 */
+            value = 1;
+        }
+        orte_default_num_sockets_per_board = (uint8_t)value;
+    }
+
+    if (0 == orte_default_num_cores_per_socket) {
+        /* we weren't given a number, so try to determine it */
+        if (OPAL_SUCCESS != (ret = opal_paffinity_base_get_core_info(0, &value))) {
+            /* don't have topo info - can we at least get #processors? */
+            if (OPAL_SUCCESS != opal_paffinity_base_get_processor_info(&value)) {
+                /* can't get any info - default to 1 */
+                value = 1;
+            }
+        }
+        orte_default_num_cores_per_socket = (uint8_t)value;
+    }
+    
     /* Since we are the HNP, then responsibility for
      * defining the name falls to the PLM component for our
      * respective environment - hence, we have to open the PLM
@@ -490,6 +521,19 @@ static int rte_finalize(void)
     
     /* clean out the global structures */
     orte_proc_info_finalize();
+    if (NULL != orte_job_ident) {
+        free(orte_job_ident);
+    }
+
+    
+    /* close the xml output file, if open */
+    if (orte_xml_output) {
+        fprintf(orte_xml_fp, "</%s>\n", orte_cmd_basename);
+        fflush(orte_xml_fp);
+        if (stdout != orte_xml_fp) {
+            fclose(orte_xml_fp);
+        }
+    }
     
     return ORTE_SUCCESS;    
 }
@@ -646,7 +690,7 @@ static orte_local_rank_t proc_get_local_rank(orte_process_name_t *proc)
     
     if (NULL == (pdata = find_proc(proc))) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-        return UINT8_MAX;
+        return ORTE_LOCAL_RANK_INVALID;
     }
     
     OPAL_OUTPUT_VERBOSE((2, orte_ess_base_output,
@@ -664,7 +708,7 @@ static orte_node_rank_t proc_get_node_rank(orte_process_name_t *proc)
     
     if (NULL == (pdata = find_proc(proc))) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-        return UINT8_MAX;
+        return ORTE_NODE_RANK_INVALID;
     }
     
     OPAL_OUTPUT_VERBOSE((2, orte_ess_base_output,
