@@ -18,6 +18,7 @@
  */
 
 #include "orte_config.h"
+#include "opal/util/output.h"
 #include "orte/constants.h"
 
 #include <errno.h>
@@ -36,9 +37,7 @@
 #endif
 #endif
 
-#include "orte/util/show_help.h"
 
-#include "orte/mca/rml/rml.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
