@@ -18,10 +18,10 @@
  */
 
 #include "orte_config.h"
+#include "orte/constants.h"
 
 #include <stdio.h>
 
-#include "orte/orte_constants.h"
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "orte/mca/oob/oob.h"
@@ -31,13 +31,12 @@
 int mca_oob_base_close(void)
 {
   opal_list_item_t* item;
-  static bool already_closed = false;
 
   /* Sanity check.  This may be able to be removed when the rml/oob
      interface is re-worked (the current infrastructure may invoke
      this function twice: once as a standalone, and once via the rml
      oob component). */
-  if (already_closed) {
+  if (!orte_oob_base_already_opened) {
       return ORTE_SUCCESS;
   }
     
@@ -58,15 +57,10 @@ int mca_oob_base_close(void)
 
   OBJ_DESTRUCT(&mca_oob_base_modules);
   OBJ_DESTRUCT(&mca_oob_base_components);
-  OBJ_DESTRUCT(&mca_oob_base_exception_handlers);
-
-  if( NULL != mca_oob_base_include )
-    free(mca_oob_base_include);
-  if( NULL != mca_oob_base_exclude )
-    free(mca_oob_base_exclude);
 
   /* All done */
-  already_closed = true;
+  orte_oob_base_already_opened = false;
+
   return ORTE_SUCCESS;
 }
 
