@@ -356,6 +356,11 @@ int orte_dt_print_node(char **output, char *prefix, orte_node_t *src, opal_data_
         }
     }
     
+    asprintf(&tmp2, "%s\n%s\tNum boards: %ld\tNum sockets/board: %ld\tNum cores/socket: %ld", tmp, pfx2,
+             (long)src->boards, (long)src->sockets_per_board, (long)src->cores_per_socket);
+    free(tmp);
+    tmp = tmp2;
+    
     if (NULL == src->daemon) {
         asprintf(&tmp2, "%s\n%s\tDaemon: %s\tDaemon launched: %s", tmp, pfx2,
                  "Not defined", src->daemon_launched ? "True" : "False");
@@ -622,9 +627,8 @@ int orte_dt_print_map(char **output, char *prefix, orte_job_map_t *src, opal_dat
     asprintf(&pfx, "%s\t", pfx2);
     
     if (orte_devel_level_output) {
-        asprintf(&tmp, "\n%sMap generated by mapping policy: %x\n%s\tPernode: %s\tNpernode: %ld\tOversubscribe allowed: %s\tCPU Lists: %s",
-                 pfx2, src->policy, pfx2,
-                 (src->pernode) ? "TRUE" : "FALSE", (long)src->npernode,
+        asprintf(&tmp, "\n%sMap generated by mapping policy: %04x\n%s\tNpernode: %ld\tOversubscribe allowed: %s\tCPU Lists: %s",
+                 pfx2, src->policy, pfx2, (long)src->npernode,
                  (src->oversubscribe) ? "TRUE" : "FALSE",
                  (src->cpu_lists) ? "TRUE" : "FALSE");
         
