@@ -100,6 +100,7 @@
  */
 static struct opal_event term_handler;
 static struct opal_event int_handler;
+static struct opal_event epipe_handler;
 #ifndef __WINDOWS__
 static struct opal_event sigusr1_handler;
 static struct opal_event sigusr2_handler;
@@ -418,6 +419,7 @@ static void terminated(int trigpipe, short event, void *arg);
 static void timeout_callback(int fd, short ign, void *arg);
 static void abort_signal_callback(int fd, short flags, void *arg);
 static void abort_exit_callback(int fd, short event, void *arg);
+static void epipe_signal_callback(int fd, short flags, void *arg);
 static void signal_forward_callback(int fd, short event, void *arg);
 static int create_app(int argc, char* argv[], orte_app_context_t **app,
                       bool *made_app, char ***app_env);
@@ -579,6 +581,10 @@ int orterun(int argc, char *argv[])
         goto DONE;
     }
 
+    /* setup callback for SIGPIPE */
+    opal_signal_set(&epipe_handler, SIGPIPE,
+                    epipe_signal_callback, &epipe_handler);
+    opal_signal_add(&epipe_handler, NULL);
     /** setup callbacks for abort signals - from this point
      * forward, we need to abort in a manner that allows us
      * to cleanup
@@ -879,6 +885,8 @@ static void terminated(int trigpipe, short event, void *arg)
         free(timeout_ev);
     }
     
+    /* Remove the epipe handler */
+    opal_signal_del(&epipe_handler);
     /* Remove the TERM and INT signal handlers */
     opal_signal_del(&term_handler);
     opal_signal_del(&int_handler);
@@ -1218,6 +1226,18 @@ static void abort_signal_callback(int fd, short flags, void *arg)
     ORTE_TIMER_EVENT(0, 0, abort_exit_callback);
 }
 
+/**
+ * Deal with sigpipe errors
+ */
+static void epipe_signal_callback(int fd, short flags, void *arg)
+{
+    /* for now, we just announce and ignore them */
+    OPAL_OUTPUT_VERBOSE((1, orte_debug_verbosity,
+                         "%s reports a SIGPIPE error on fd %d",
+                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), fd));
+    return;
+}
+
 /**
  * Pass user signals to the remote application processes
  */
